import { JSONSchema7 } from 'json-schema';
import { TMstViewKindElement } from './models/MstViewDescr';
/**
 * Constant that indicates that a tester is not capable of handling
 * a combination of schema/data.
 * @type {number}
 */
export declare const NOT_APPLICABLE = -1;
/**
 * A tester is a function that receives an UI schema and a JSON schema and returns a boolean.
 */
export type Tester = (viewKindElement: TMstViewKindElement, schema: JSONSchema7) => boolean;
/**
 * A tester that allow composing other testers by && them.
 *
 * @param {Array<Tester>} testers the testers to be composed
 */
export declare const and: (...testers: Tester[]) => Tester;
/**
 * A tester that allow composing other testers by || them.
 *
 * @param {Array<Tester>} testers the testers to be composed
 */
export declare const or: (...testers: Tester[]) => Tester;
/**
 * Create a ranked tester that will associate a number with a given tester, if the
 * latter returns true.
 *
 * @param {number} rank the rank to be returned in case the tester returns true
 * @param {Tester} tester a tester
 */
export declare const rankWith: (rank: number, tester: Tester) => (viewKindElement: TMstViewKindElement, schema: JSONSchema7) => number;
/**
 * Checks whether the given UI schema has the expected type.
 *
 * @param {string} expected the expected UI schema type
 */
export declare const uiTypeIs: (expected: string) => Tester;
/**
 * Checks whether the given UI schema has an option with the given
 * name and whether it has the expected value. If no options property
 * is set, returns false.
 *
 * @param {string} optionName the name of the option to check
 * @param {any} optionValue the expected value of the option
 */
export declare const optionIs: (optionName: string, optionValue: any) => Tester;
/**
 * A ranked tester associates a tester with a number.
 */
export declare type RankedTester = (viewKindElement: TMstViewKindElement, schema: JSONSchema7) => number;
export declare const hasType: (jsonSchema: JSONSchema7, expected: string) => boolean;
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and applies
 * the given predicate
 *
 * @param {(JsonSchema) => boolean} predicate the predicate that should be
 *        applied to the resolved sub-schema
 */
export declare const schemaMatches: (predicate: (schema: JSONSchema7) => boolean) => Tester;
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and checks
 * whether the type of the sub-schema matches the expected one.
 *
 * @param {string} expectedType the expected type of the resolved sub-schema
 */
export declare const schemaTypeIs: (expectedType: string) => Tester;
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and checks
 * whether the format of the sub-schema matches the expected one.
 *
 * @param {string} expectedFormat the expected format of the resolved sub-schema
 */
export declare const formatIs: (expectedFormat: string) => Tester;
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * has a 'date' format.
 * @type {Tester}
 */
export declare const isDateControl: Tester;
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * has a 'date-time' format.
 * @type {Tester}
 */
export declare const isDateTimeControl: Tester;
/**
 * Default tester for boolean.
 * @type {RankedTester}
 */
export declare const isBooleanControl: Tester;
export declare const isEnumControl: Tester;
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type integer
 * @type {Tester}
 */
export declare const isIntegerControl: Tester;
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type number
 * @type {Tester}
 */
export declare const isNumberControl: Tester;
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type string
 * @type {Tester}
 */
export declare const isStringControl: Tester;
/**
 * Tests whether a given UI schema is of type Control,
 * if the schema is of type number or integer and
 * whether the schema defines a numerical range with a default value.
 * @type {Tester}
 */
export declare const isRangeControl: Tester;
//# sourceMappingURL=testers.d.ts.map