import React, { createContext, Component, useContext, useState, useMemo, useEffect, useRef, useCallback } from 'react';
import { Spin, Checkbox, DatePicker, Input, TimePicker, Select, Image, Rate, Card as Card$1, Row as Row$1, Col, Button, Form as Form$1, Radio, Typography, Slider, Table, Tree, Tooltip, InputNumber, Modal, Tag, List, Tabs, Divider, Dropdown, Menu, Transfer } from 'antd';
import { startCase, maxBy, get, cloneDeep, omit, isEqual, merge, isEmpty, includes, find, upperFirst, camelCase, sortBy } from 'lodash-es';
import { types, getSnapshot, applySnapshot } from 'mobx-state-tree';
import { observer } from 'mobx-react-lite';
import uuid62 from 'uuid62';
import { ErrorBoundary } from 'react-error-boundary';
import { reaction, autorun, when } from 'mobx';
import { MstJsObject, MstCollConstr, getParentOfName, arrDiff, getMstLiteralPropValue, MstRepository, registerMstCollSchema } from '@agentlab/sparql-jsld-client';
import dayjs from 'dayjs';
import { Editor } from '@tinymce/tinymce-react';
import { Link } from 'react-router-dom';
import * as allIcons from '@ant-design/icons/lib/icons';
import warning$1 from 'rc-util/lib/warning';
import { CheckCircleTwoTone, ExclamationCircleTwoTone, PlusOutlined, DownOutlined, UpOutlined, FolderOutlined, SortDescendingOutlined, SortAscendingOutlined, DeleteOutlined, EditOutlined, SearchOutlined, MoreOutlined, SettingOutlined, CheckOutlined } from '@ant-design/icons';
import SplitPane from 'react-split-pane';
import { VisibilityContext, ScrollMenu } from 'react-horizontal-scrolling-menu';
import 'react-virtualized/styles.css';
import { SplitPane as SplitPane$1 } from '@agentlab/react-collapse-pane';
import { SortableHandle, SortableElement, SortableContainer } from 'react-sortable-hoc';
import BaseTable, { Column, AutoResizer } from 'react-base-table';
import isEqual$1 from 'lodash-es/isEqual';
import 'react-dnd';
import difference from 'lodash-es/difference';

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const deriveLabel = (controlElement /*ControlElement*/, schemaElement) => {
    if (schemaElement && typeof schemaElement.title === 'string') {
        return schemaElement.title;
    }
    if (typeof controlElement.scope === 'string') {
        const ref = controlElement.scope;
        const label = ref.substr(ref.lastIndexOf('/') + 1);
        return startCase(label);
    }
    return '';
};
const createCleanLabel = (label) => {
    return startCase(label.replace('_', ' '));
};
/**
 * Return a label object based on the given control and schema element.
 * @param {ControlElement} withLabel the UI schema to obtain a label object for
 * @param {JSONSchema7} schema optional: the corresponding schema element
 * @returns {LabelDescription}
 */
const createLabelDescriptionFrom = (withLabel /*ControlElement*/, schema) => {
    const labelProperty = withLabel.label;
    if (typeof labelProperty === 'boolean') {
        return labelDescription(deriveLabel(withLabel, schema), labelProperty);
    }
    if (typeof labelProperty === 'string') {
        return labelDescription(labelProperty, true);
    }
    if (typeof labelProperty === 'object') {
        const label = typeof labelProperty.text === 'string' ? labelProperty.text : deriveLabel(withLabel, schema);
        const show = typeof labelProperty.show === 'boolean' ? labelProperty.show : true;
        return labelDescription(label, show);
    }
    return labelDescription(deriveLabel(withLabel, schema), true);
};
const labelDescription = (text, show) => ({
    text: text,
    show: show,
});

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/***********************
 * View Kind
 ***********************/
const MstViewKindElement = types.model('MstViewKindElement', {
    '@id': types.identifier, // JSON-LD object id
    '@type': types.string, //types.union(types.literal('aldkg:ViewElement'), types.literal('aldkg:DiagramEditor')), // JSON-LD class id of a View
    title: types.maybe(types.union(types.string, MstJsObject)),
    description: types.maybe(types.union(types.string, MstJsObject)),
    scope: types.maybe(types.string),
    resultsScope: types.maybe(types.string),
    options: types.maybe(MstJsObject),
    style: types.maybe(MstJsObject),
    // Container-specific (e.g. Layout, type: 'xxxLayout')
    elements: types.maybe(types.array(types.late(() => MstViewKindDataType))),
});
const mstViewKindSchemas = {};
function registerMstViewKindSchema(mstModel, noOverride = false) {
    const id = getMstLiteralPropValue(mstModel, '@type');
    if (id) {
        if (!noOverride || mstViewKindSchemas[id] === undefined) {
            console.log('register mstViewKindSchema', { id, mstModel });
            mstViewKindSchemas[id] = mstModel;
        }
        else {
            console.log('cannot register mstViewKindSchema: noOverride and existed schema', { mstModel });
        }
    }
    else {
        console.log('cannot register mstViewKindSchema', { mstModel });
    }
}
function unregisterMstViewKindSchema(id) {
    const t = mstViewKindSchemas[id];
    delete mstViewKindSchemas[id];
    return t;
}
const MstViewKindDataType = types.union({
    dispatcher: (snapshot) => {
        if (snapshot) {
            const mstModel = mstViewKindSchemas[snapshot['@type']];
            if (mstModel) {
                //console.log('ViewKindDataType, create mstModel for', {
                //  snapshotId: snapshot['@id'],
                //  mstModelName: mstModel.name,
                //});
                return mstModel;
            }
        }
        //console.log('ViewKindDataType, create ViewKindElement for', snapshot['@id']);
        return MstViewKindElement;
    },
}, MstViewKindElement);
/**
 * View Kind, which could be persisted in DB
 */
const MstViewKind = types
    .model('aldkg:ViewKind', {
    '@id': types.identifier, // JSON-LD object id of a viewKind
    '@type': types.literal('aldkg:ViewKind'), // JSON-LD class id of a View
    title: types.maybe(types.string), // mandatory title
    description: types.maybe(types.string),
    options: types.maybe(MstJsObject),
    // Container-specific (e.g. Layout, type: 'xxxLayout')
    elements: types.array(MstViewKindDataType),
    collsConstrs: types.array(MstCollConstr), // former 'queries'
})
    .actions((self) => {
    const rep = getParentOfName(self, 'MstRepository');
    //console.log('MstViewRind-rep', rep);
    const coll = getParentOfName(self, 'MstColl');
    //console.log('MstViewRind-coll', coll);
    let dispose;
    return {
        afterAttach() {
            //console.log('ViewKind afterAttach, @id=', self['@id']);
            if (coll.resolveCollConstrs) {
                dispose = reaction(() => self.collsConstrs, (newArr, oldArr) => {
                    //console.log('ViewKind reaction, add coll ref, @id=', self['@id']);
                    const { deleted, added } = arrDiff(newArr, oldArr);
                    //console.log('ViewKind reaction, add coll ref, {deleted,added}=', { deleted, added });
                    deleted.forEach((e) => rep.colls.delete(e['@id']));
                    added.forEach((e) => rep.addCollByConstrRef(e));
                }, { fireImmediately: true, name: 'ViewKind-Attach' });
            }
        },
        beforeDetach() {
            //console.log('ViewKind beforeDetach, @id=', self['@id']);
            if (coll.resolveCollConstrs) {
                if (dispose)
                    dispose();
                self.collsConstrs.forEach((e) => rep.colls.delete(e['@id']));
            }
        },
        setCollConstrs(collsConstrs) {
            const collsConstrsObservables = collsConstrs.map((cc) => MstCollConstr.create(cc));
            self.collsConstrs.push(...collsConstrsObservables);
        },
    };
});
/***********************
 * View Description
 ***********************/
const MstViewDescrElement = types.model('MstViewDescrElement', {
    '@id': types.identifier, // JSON-LD object id
    '@type': types.string, //types.union(types.literal('aldkg:ViewElement'), types.literal('aldkg:DiagramEditor')), // JSON-LD class id of a View
    '@parent': types.maybe(types.string),
    //'@parent': types.union(types.reference(types.late((): IAnyType => MstViewKindDataType)), types.undefined),
    title: types.maybe(types.union(types.string, MstJsObject)),
    description: types.maybe(types.union(types.string, MstJsObject)),
    scope: types.maybe(types.string),
    resultsScope: types.maybe(types.string),
    options: types.maybe(MstJsObject),
    style: types.maybe(MstJsObject),
    // Container-specific (e.g. Layout, type: 'xxxLayout')
    elements: types.maybe(types.array(types.late(() => MstViewDescrDataType))),
});
const mstViewDescrSchemas = {};
function registerMstViewDescrSchema(mstModel, noOverride = false) {
    const id = getMstLiteralPropValue(mstModel, '@type');
    if (id) {
        if (!noOverride || mstViewDescrSchemas[id] === undefined) {
            console.log('register mstViewDescrSchema', { id, mstModel });
            mstViewDescrSchemas[id] = mstModel;
        }
        else {
            console.log('cannot register mstViewDescrSchema: noOverride and existed schema', { mstModel });
        }
    }
    else {
        console.log('cannot register mstViewDescrSchema', { mstModel });
    }
}
function unregisterMstViewDescrSchema(id) {
    const t = mstViewDescrSchemas[id];
    delete mstViewDescrSchemas[id];
    return t;
}
const MstViewDescrDataType = types.union({
    dispatcher: (snapshot) => {
        if (snapshot) {
            const mstModel = mstViewDescrSchemas[snapshot['@type']];
            if (mstModel) {
                //console.log('ViewDescrDataType, create mstModel for', {
                //  snapshotId: snapshot['@id'],
                //  mstModelName: mstModel.name,
                //});
                return mstModel;
            }
        }
        //console.log('ViewDescrDataType, create ViewDescrElement for', snapshot['@id']);
        return MstViewDescrElement;
    },
}, MstViewDescrElement);
/**
 * View Description, which could be persisted in DB
 */
const MstViewDescr = types
    .model('aldkg:ViewDescr', {
    '@id': types.identifier, // JSON-LD object id of a viewKind
    '@type': types.literal('aldkg:ViewDescr'), // JSON-LD class id of a View
    viewKind: types.safeReference(MstViewKind),
    title: types.maybe(types.string), // mandatory title
    description: types.maybe(types.string),
    options: types.maybe(MstJsObject),
    // Container-specific (e.g. Layout, type: 'xxxLayout')
    elements: types.array(MstViewDescrDataType),
    collsConstrs: types.array(MstCollConstr), // former 'queries'
})
    .actions((self) => {
    const rep = getParentOfName(self, 'MstRepository');
    //console.log('MstViewDescr-re', rep);
    const coll = getParentOfName(self, 'MstColl');
    //console.log('MstViewDescr-coll', coll);
    let dispose;
    return {
        afterAttach() {
            //console.log('MstViewDescr afterAttach, @id=', self['@id']);
            if (coll.resolveCollConstrs) {
                dispose = reaction(() => self.collsConstrs, (newArr, oldArr) => {
                    //console.log('MstViewDescr reaction, add coll ref, @id=', self['@id']);
                    const { deleted, added } = arrDiff(newArr, oldArr);
                    //console.log('MstViewDescr reaction, add coll ref, {deleted,added}=', { deleted, added });
                    deleted.forEach((e) => rep.colls.delete(e['@id']));
                    added.forEach((e) => rep.addCollByConstrRef(e));
                }, { fireImmediately: true, name: 'MstViewDescr-Attach' });
            }
        },
        beforeDetach() {
            //console.log('MstViewDescr beforeDetach, @id=', self['@id']);
            if (coll.resolveCollConstrs) {
                if (dispose)
                    dispose();
                self.collsConstrs.forEach((e) => rep.colls.delete(e['@id']));
            }
        },
        setCollConstrs(collsConstrs) {
            const collsConstrsObservables = collsConstrs.map((cc) => MstCollConstr.create(cc));
            self.collsConstrs.push(...collsConstrsObservables);
        },
    };
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const MstContext = createContext({
    store: undefined,
    renderers: [],
    cells: [],
    actions: {},
});
const MstContextProvider = ({ store, renderers, cells = [], children, actions = {}, }) => {
    renderers.forEach((r) => {
        const mstVkeType = r.mstVkeType;
        if (mstVkeType) {
            registerMstViewKindSchema(mstVkeType, true);
        }
        const mstVdeType = r.mstVdeType;
        if (mstVdeType) {
            registerMstViewDescrSchema(mstVdeType, true);
        }
    });
    cells.forEach((с) => {
        const mstVkeType = с.mstVkeType;
        if (mstVkeType) {
            registerMstViewKindSchema(mstVkeType, true);
        }
        const mstVdeType = с.mstVdeType;
        if (mstVdeType) {
            registerMstViewDescrSchema(mstVdeType, true);
        }
    });
    return React.createElement(MstContext.Provider, { value: { store, renderers, cells, actions } }, children);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/**
 * A renderer that will be used in case no other renderer is applicable.
 */
class UnknownRenderer extends Component {
    render() {
        return (React.createElement("div", { style: { color: 'red' } },
            "No applicable ",
            this.props.type,
            " found for element with id=",
            this.props.elementId,
            " and type=",
            this.props.elementType,
            "."));
    }
}

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
function mstJsonLdIds(o) {
    if (o)
        return { '@id': o['@id'], '@type': o['@type'] };
    else
        return undefined;
}
function createViewDescrElementIri(viewKindElementIri) {
    return viewKindElementIri + '_' + uuid62.v4();
}
function compareByIri(iri1, iri2) {
    //console.log('compareByIri - raw', { iri1, iri2 });
    if (typeof iri1 === 'object')
        iri1 = iri1['@id'];
    if (typeof iri2 === 'object')
        iri2 = iri2['@id'];
    //console.log('compareByIri - norm', { iri1, iri2 });
    return iri1 === iri2;
}
const processViewKindOverride = (props, store) => {
    const { viewKindElement, viewDescr } = props;
    // if ViewElement extend-override exists
    const viewDescrElement = viewDescr.elements?.find((el) => {
        //console.log('processViewKindOverride', {
        //  el: mstJsonLdIds(el),
        //  el_parent: mstJsonLdIds(el['@parent']),
        //  viewKindElement: mstJsonLdIds(viewKindElement),
        //});
        return compareByIri(el['@parent'], viewKindElement['@id']);
    });
    const id = viewDescrElement ? viewDescrElement['@id'] : createViewDescrElementIri(viewKindElement['@id']);
    const [collIri, inCollPath] = viewKindElement.resultsScope?.split('/') || [];
    let collIriOverride = collIri;
    if (collIriOverride) {
        // if CollConstr extend-override exists switch to extCollConstr
        if (viewDescr.collsConstrs) {
            const extCollConstr = viewDescr.collsConstrs?.find((el) => compareByIri(el['@parent'], collIri));
            if (extCollConstr) {
                collIriOverride = extCollConstr['@id'] || '';
            }
        }
    }
    return [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement];
};
const FormsDispatch = observer((props) => {
    const { store, renderers, actions } = useContext(MstContext);
    const { viewKind, viewDescr, form, enabled } = props;
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    let schema;
    if (collIri !== collIriOverride) {
        const parentColl = collIri ? store.rep.getColl(collIri) : undefined;
        const parentSchema = parentColl?.collConstr?.entConstrs[0]?.schemaJs;
        const childColl = collIriOverride ? store.rep.getColl(collIriOverride) : undefined;
        const childSchema = childColl?.collConstr?.entConstrs[0]?.schemaJs;
        schema = childSchema ? childSchema : parentSchema;
    }
    else {
        const coll = collIri ? store.rep.getColl(collIri) : undefined;
        const collConstr = coll?.collConstr;
        //console.log('FormsDispatch1', { collConstr, collConstrSnap: collConstr ? getSnapshot(collConstr) : undefined });
        const entConstr = collConstr?.entConstrs[0];
        schema = entConstr?.schemaJs;
    }
    if ((collIri || collIriOverride) && !schema)
        return React.createElement(Spin, null);
    if (inCollPath && inCollPath.length > 0)
        schema = schema.properties[inCollPath];
    const renderer = maxBy(renderers, (r) => r.tester(viewKindElement, schema));
    //const isModal = viewKindElement.options && viewKindElement.options.modal;
    if (renderer === undefined || renderer.tester(viewKindElement, schema) === -1) {
        return (React.createElement(UnknownRenderer, { type: 'renderer', elementId: viewKindElement['@id'], elementType: viewKindElement['@type'] }));
    }
    else {
        const Render = renderer.renderer;
        return (React.createElement(ErrorBoundary, { FallbackComponent: ErrorFallback, onReset: () => { } },
            React.createElement(Render, { id: id, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, actions: actions, enabled: enabled, form: form })));
    }
});
function ErrorFallback({ error, resetErrorBoundary }) {
    return (React.createElement("div", { role: 'alert' },
        React.createElement("p", null, "Something went wrong:"),
        React.createElement("pre", null, error?.message),
        React.createElement("button", { onClick: resetErrorBoundary }, "Try again")));
}
const Form = observer((props) => {
    const { store } = useContext(MstContext);
    if (!store) {
        console.log('!store', store);
        return React.createElement(Spin, null);
    }
    if (Object.keys(store.rep.ns.currentJs).length < 6) {
        console.log('!ns');
        return React.createElement(Spin, null);
    }
    const { viewDescrId, viewDescrCollId } = props;
    console.log('Form', { viewDescrId, viewDescrCollId });
    const collWithViewDescrsObs = store.rep.getColl(viewDescrCollId);
    if (!collWithViewDescrsObs) {
        console.log('!collWithViewDescrsObs', viewDescrCollId);
        return React.createElement(Spin, null);
    }
    const viewDescrObs = collWithViewDescrsObs?.dataByIri(viewDescrId);
    if (!viewDescrObs) {
        console.log('!viewDescrObs', viewDescrId);
        return React.createElement(Spin, null);
    }
    //const collWithViewKindsObs = store.rep.getColl(viewKindCollId);
    //if (!collWithViewKindsObs) {
    //  console.log('!collWithViewKindsObs', viewKindCollId);
    //  return <Spin />;
    //}
    const viewKindObs = viewDescrObs.viewKind;
    if (!viewKindObs) {
        console.log('!viewKindObs for viewDescr', getSnapshot(viewDescrObs));
        return React.createElement(Spin, null);
    }
    return (React.createElement(ErrorBoundary, { FallbackComponent: ErrorFallback, onReset: () => { } }, viewKindObs.elements.map((el, idx) => (React.createElement(FormsDispatch, { key: idx, ...props, viewKind: viewKindObs, viewKindElement: el, viewDescr: viewDescrObs })))));
});

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const regExpValidation = (data, propsToValidator) => {
    const re = new RegExp(propsToValidator.regExp);
    return re.test(data.toString());
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const validators = {
    RegExp: regExpValidation,
};

const addObjects = ({ root, coll, selection, options }) => {
    console.log('SELECTION', selection);
    //coll.testOnAddObjs([selection]);
};
const addTreeObj = ({ root, coll, selection, options }) => {
    const node = selection[0];
    const newObj = {
        [node.titlePropName]: 'New node',
        [node.parentKey]: node['@id'],
    };
    return coll.testOnAddObjs([newObj]);
};
const deleteObjects = ({ root, coll, selection, options }) => {
    coll.testOnDeleteObjs(selection.map((obj) => obj['@id']));
};
const addConnectionToTarget = ({ root, coll, selection, options }) => {
    const target = options?.target;
    if (target) {
        const targetColl = root.getColl(target);
        const targetData = targetColl?.data;
        if (targetData) {
            const snapData = getSnapshot(targetData);
            const newData = [...snapData, ...selection];
            applySnapshot(targetData, newData);
        }
    }
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const NoTitleProvided = 'No title provided';

const mapViewKindPropsToActions = ({ actions, viewKindActionProps, coll, root, }) => viewKindActionProps &&
    viewKindActionProps.reduce((acc, props) => {
        const actionType = props['@type'];
        if (actionType in actions) {
            acc.push({
                title: props.title || NoTitleProvided,
                action: (selection) => actions[actionType]?.({ coll, options: props.options, selection, root }),
            });
        }
        return acc;
    }, []);

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const actions = {
    'ldkg:addObjects': addObjects,
    'ldkg:deleteObjects': deleteObjects,
    'ldkg:addConnectionToTarget': addConnectionToTarget,
    'ldkg:addTreeObj': addTreeObj,
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const withStoreToControlProps = (Component) => observer((props) => {
    const { store } = useContext(MstContext);
    const successValidation = {
        validateStatus: 'success',
    };
    const [validateObj, setValidateObj] = useState(successValidation);
    const { form } = props;
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const coll = collIriOverride ? store.rep.getColl(collIriOverride) : undefined;
    const dataArray = coll ? coll.dataJs : [];
    const data = dataArray.length > 0 ? dataArray[0] : {};
    const controlProps = mapStateToControlProps({ ...props, data: data });
    const onValidate = (data) => {
        if (viewKindElement.options && Array.isArray(viewKindElement.options.validation)) {
            const validation = viewKindElement.options.validation;
            const idx = validation.findIndex((el) => !validators[el.validator](data, el.propsToValidator));
            if (idx !== -1) {
                const { validateStatus, help } = validation[idx];
                store.setOnValidate(form || '', viewKindElement.resultsScope || '', false);
                setValidateObj({
                    validateStatus,
                    help,
                });
            }
            else {
                setValidateObj(successValidation);
                store.setOnValidate(form || '', viewKindElement.resultsScope || '', true);
            }
        }
    };
    return (React.createElement(Component, { id: id || '', data: data[inCollPath], formData: data, editing: form ? store.editingData.get(form) : store.editingData.get(collIriOverride), form: form, validateObj: validateObj, onValidation: form ? onValidate : () => { }, ...controlProps, handleChange: (data) => {
            form
                ? store.onChangeFormData(form, viewKindElement.resultsScope, data)
                : store.onChangeData(viewKindElement.resultsScope, data);
        } }));
});
const withStoreToFormProps = (Component) => observer(({ viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, form }) => {
    if (!viewKind['@id']) {
        return null;
    }
    const title = viewKindElement.options ? viewKindElement.options.title : '';
    const id = viewKindElement['@id'];
    const enabledLayout = enabled && checkProperty('editable', id, viewKindElement, viewKind);
    const visible = checkProperty('visible', id, viewKindElement, viewKind);
    const { store } = useContext(MstContext);
    return (React.createElement(Component, { id: id, title: title, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, enabled: enabledLayout, visible: visible, onSave: () => store.onSaveFormData(id), editing: store.editingData.get(id), onEdit: () => store.setEditing(id, true), onCancel: () => {
            store.onCancelForm(id);
        } }));
});
const withStoreToViewClassProps = (Component) => observer(({ ...props }) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement } = props;
    const { store } = useContext(MstContext);
    const scope = viewKindElement.resultsScope;
    if (!store.getSelectedDataJs(scope)) {
        return React.createElement(Spin, null);
    }
    //const id = store.getSelectedDataJs(scope).type;
    return (React.createElement(Component, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement }));
});
const withStoreToViewProps = (Component) => observer(({ ...props }) => {
    const { viewKind, viewDescr } = props;
    const { store } = useContext(MstContext);
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const coll = store.rep.getColl(collIriOverride);
    let data = coll?.data;
    if (!data) {
        //if (scope === 'rm:dataModelView') {
        //  store.loadData(scope);
        //}
        return React.createElement(Spin, null);
    }
    data = getSnapshot(data);
    //if (viewKindElement.resultsScope && !store.saveLogicTree[viewKindElement.resultsScope]) {
    //  store.setSaveLogic(viewKindElement.resultsScope);
    //}
    //const id = store.getSelectedDataJs(scope)['@type'];
    //const selection = getSnapshot(store.selectedData);
    const newView = store.getSelectedDataJs(collIriOverride);
    if (!newView) {
        return React.createElement(Spin, null);
    }
    const newViewElement = newView;
    console.log('withStoreToViewProps', { viewKind, viewKindElement, newView, newViewElement });
    return (React.createElement(Component, { id: id, viewKind: newView, viewKindElement: newViewElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, onChange: (state) => store.setEditing(id, state) }));
});
const withStoreToModalProps = (Component) => observer(({ ...props }) => {
    return React.createElement(Component, { ...props });
});
const withStoreToButtonProps = (Component) => observer(({ ...props }) => {
    const { schema, viewKindElement } = props;
    const { store } = useContext(MstContext);
    if (viewKindElement.resultsScope && !store.saveLogicTree[viewKindElement.resultsScope]) {
        store.setSaveLogic(viewKindElement.resultsScope);
    }
    const options = viewKindElement.options || {};
    return React.createElement(Component, { schema: schema, handleChange: () => { }, options: options });
});
const withStoreToCellProps = (Component) => observer((props) => {
    const { data, onMeasureChange, height, uri, CKey, rowData, viewKindElement } = props;
    const path = viewKindElement.scope ? viewKindElement.scope.split('/').join('.') : null;
    /*const { store } = useRootCtx();
    const onSave = (data: any) => {
      let newData: any = {};
      newData[CKey] = data;
      store.saveCellData(newData, uri, rowData['@id']);
    };*/
    const cellData = path ? get(data, path) : data;
    const controlProps = mapStateToControlProps({ ...props, data: cellData });
    return (React.createElement(Component, { data: cellData, height: height, rowData: rowData, onMeasureChange: onMeasureChange, editing: false, ...controlProps, width: props.width, isScrolling: props.isScrolling, handleChange: () => { } }));
});
const withStoreToDataControlProps = (Component) => observer(({ ...props }) => {
    const { viewKind, viewDescr, actions } = props;
    const { store } = useContext(MstContext);
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const coll = store.rep.getColl(collIriOverride);
    if (!coll)
        return React.createElement(Spin, null);
    let data = [];
    if (!coll.isLoading) {
        data = cloneDeep(coll.dataJs);
    }
    const options = viewKindElement?.options || {};
    const withConnections = options.connections;
    const onSelect = (data) => {
        if (data) {
            store.setSelectedData(collIriOverride, data);
            withConnections && store.rep.editConn(withConnections, data);
        }
    };
    const getData = (parentId) => {
        const conditions = { ...store.queries[collIriOverride].shapes[0].conditions, parent: parentId };
        const newQuery = cloneDeep(store.queries[collIriOverride]);
        newQuery.shapes[0].conditions = conditions;
        return store.getDataByQuery(newQuery);
    };
    const onDnD = ({ childId, parentId }) => {
        store.updateObjectData({ parent: parentId }, collIriOverride, childId);
    };
    const actionsMap = useMemo(() => mapViewKindPropsToActions({ actions, viewKindActionProps: options.selectActions, coll, root: store }), [coll, actions, options]);
    const onCreateFolder = (data) => {
        return coll.testOnAddObjs([data]);
    };
    const onDeleteFolder = (id) => {
        if (id) {
            return coll.testOnDeleteObjs([id]);
        }
    };
    const onRename = (newTitle, id) => {
        coll.testOnUpdateObj(id, { title: newTitle });
    };
    return (React.createElement(Component, { uri: id, dataSource: data, actionsMap: actionsMap, editing: store.editingData.get(collIriOverride), viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, handleChange: onSelect, onCreateFolder: onCreateFolder, getData: getData, onDnD: onDnD, onRename: onRename, onDeleteFolder: onDeleteFolder, ...props }));
});
const withStoreToSelectControlProps = (Component) => observer(({ ...props }) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement } = props;
    const { store } = useContext(MstContext);
    viewKind['@id'];
    const scope = viewKindElement.resultsScope;
    const coll = store.rep.getColl(scope);
    let data = coll?.data;
    if (!data) {
        return React.createElement(Spin, null);
    }
    data = getSnapshot(data);
    const options = viewKindElement.options || {};
    const withConnections = options.connections;
    const onChange = (data) => {
        store.setSelectedData(/*id || */ scope, data);
        withConnections &&
            options.connections.forEach((e) => {
                const condition = {};
                condition[e.toProp] = data['@id'];
                //view2.editCondition(e.toObj, condition, scope, e.toProp, data);
            });
    };
    return (React.createElement(Component, { dataSource: data, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, options: viewKindElement.options, handleChange: onChange, ...props }));
});
const withStoreToMenuProps = (Component) => observer(({ ...props }) => {
    const { schema, viewKind, viewDescr } = props;
    const { store } = useContext(MstContext);
    //if (viewKindElement.resultsScope && !store.saveLogicTree[viewKindElement.resultsScope]) {
    //  store.setSaveLogic(viewKindElement.resultsScope);
    //}
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const options = viewKindElement.options || {};
    const coll = store.rep.getColl(collIriOverride);
    let data = coll?.data;
    if (!data) {
        return React.createElement(Spin, null);
    }
    data = getSnapshot(data);
    return (React.createElement(Component, { modals: viewKindElement.elements ? viewKindElement.elements.filter((e) => e.options && e.options.modal) : [], schema: schema, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, uri: id, tabs: data, handleChange: (data) => store.setSelectedData(collIriOverride, data), options: options, setModalVisible: (uri, state) => store.setModalVisible(uri, state) }));
});
const withStoreToCollapseProps = (Component) => observer(({ ...props }) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement } = props;
    const options = viewKindElement.options || {};
    return (React.createElement(Component, { options: options, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement }));
});
const withLayoutProps = (Component) => observer(({ viewKind, viewKindElement, viewDescr, viewDescrElement, schema, enabled, form, readOnly }) => {
    const id = viewKindElement['@id'] || '';
    const enabledLayout = enabled && checkProperty('editable', id, viewKindElement, viewKind);
    const visible = checkProperty('visible', id, viewKindElement, viewKind);
    const { store } = useContext(MstContext);
    if (viewKindElement.options && viewKindElement.options.connections) {
        viewKindElement.options.connections.forEach((e) => store.setSaveLogic(e.from, e.toObj));
    }
    return (React.createElement(Component, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, id: id, schema: schema, enabled: enabledLayout, visible: visible, form: form, readOnly: readOnly }));
});
const withStoreToSaveButtonProps = (Component) => observer(({ viewKindElement, enabled }) => {
    const { store } = useContext(MstContext);
    if (viewKindElement.resultsScope && !store.saveLogicTree[viewKindElement.resultsScope]) {
        store.setSaveLogic(viewKindElement.resultsScope);
    }
    const key = viewKindElement.resultsScope;
    return (React.createElement(Component, { editing: store.editingData.get(key), setEditing: (state) => store.setEditing(key, state), saveData: () => store.saveData(key), enabled: enabled, visible: true }));
});
const withStoreToSaveDialogProps = (Component) => observer(({ visible, schemaUri, onCancel, onOk }) => {
    const { store } = useContext(MstContext);
    const onSave = () => {
        store.setEditing(schemaUri, false, true);
        onOk();
    };
    return React.createElement(Component, { visible: visible, onOk: onSave, onCancel: onCancel });
});
const mapStateToControlProps = ({ id, schema, viewKindElement, viewKind, data }) => {
    const pathSegments = viewKindElement?.resultsScope?.split('/') || [];
    const path = pathSegments.join('.properties.');
    const visible = checkProperty('visible', path, viewKindElement, viewKind);
    const editable = viewKindElement.options && 'editable' in viewKindElement.options ? viewKindElement.options.editable : true;
    const required = true;
    const uiOptions = viewKindElement.options;
    const description = schema.description || '';
    const labelDesc = createLabelDescriptionFrom(viewKindElement, schema);
    const label = labelDesc.show ? labelDesc.text : '';
    const key = pathSegments[1];
    const enabled = data[key] !== undefined && data[key] !== null ? (editable ?? true) : false;
    return {
        description,
        label,
        visible,
        enabled,
        required,
        uiOptions,
        key,
        schema,
        errors: [],
    };
};
const checkProperty = (property, path, viewKindElement, viewKind) => {
    const viewClassProp = viewKindElement.options;
    const viewProp = get(viewKind, path);
    if (viewClassProp && viewClassProp[property]) {
        return viewClassProp[property];
    }
    else if (viewProp && viewProp[property]) {
        return viewProp[property];
    }
    else {
        return true;
    }
};
function treeify(list, idAttr = 'id', parentAttr = 'parent', childrenAttr = 'children', cmpFunc) {
    const lookup = {};
    list.forEach((obj) => {
        lookup[obj[idAttr]] = obj;
        obj[childrenAttr] = [];
        obj.key = obj[idAttr];
    });
    const treeList = [];
    let ob;
    list.forEach((obj) => {
        if (obj[parentAttr] != null && obj[parentAttr] !== obj[idAttr]) {
            ob = lookup[obj[parentAttr]];
            if (ob !== undefined) {
                ob[childrenAttr].push(obj);
                //ob[childrenAttr] = ob[childrenAttr].sort(cmpFunc);
            }
            else {
                treeList.push(obj);
            }
        }
        else {
            treeList.push(obj);
        }
    });
    return treeList.sort((a, b) => cmpFunc(a[childrenAttr], b[childrenAttr]));
}
function strcmp(a, b) {
    if (a.length < b.length) {
        return -1;
    }
    if (a.length > b.length) {
        return 1;
    }
    return a < b ? -1 : a > b ? 1 : 0;
}
/*
export const withContextToSaveControlProps =
  (Component: any): any => ({...props}: any) => {
    const ctx = props.context;
    if ( props.contextTree[ctx] === undefined ) return null;
    const { data, editing, onSave } = useContext(props.contextTree[ctx]);
    return <Component dataSource={data} editing={editing} onSave={onSave} {...props} />;
  };
export const withContextToControlProps =
  (Component: any): any => ({...props}: any) => {
    const ctx = props.context;
    if ( props.contextTree[ctx] === undefined ) return null;
    const { data, editing } = useContext(props.contextTree[ctx]);
    const { setField } = useContext(SaveControlContext);
    const controlProps = mapStateToControlProps(props);
    const pathSegments = props.uischema.scope.split('/');
    return <Component data={data[pathSegments[1]]} editing={editing} {...controlProps} handleChange={setField || voidFunction} />;
  };

export const withContextToArrayProps =
  (Component: any): any => ({...props}: any) => {
    const ctx = props.context;
    if ( props.contextTree[ctx] === undefined ) return null;
    const { arrayData, setData } = useContext(props.contextTree[ctx]);
    return  <Component data={arrayData} handleChange={setData} enabled={true} visible={true} {...props} />;
  };

export const withContextFormsArrayProps =
  (Component: any): any =>
    withContextToArrayProps(React.memo(
      Component,
      (prevProps: any, nextProps: any) => isEqual(prevProps, nextProps)
      ));
export const withContextFormsControlProps =
    (Component: any): any =>
      withContextToControlProps(React.memo(
        Component,
        (prevProps: any, nextProps: any) => isEqual(prevProps, nextProps)
      ));
export const withContextFormsSaveControlProps =
    (Component: any): any =>
      withContextToSaveControlProps(React.memo(
        Component,
        (prevProps: any, nextProps: any) => isEqual(prevProps, nextProps)
      ));
*/
//type FormsPropTypes = ControlProps | CombinatorProps | LayoutProps | CellProps | ArrayLayoutProps | StatePropsOfControlWithDetail | OwnPropsOfRenderer;
const areEqual = (prevProps /*FormsPropTypes*/, nextProps /*FormsPropTypes*/) => {
    const prev = omit(prevProps, ['handleChange']);
    const next = omit(nextProps, ['handleChange']);
    return isEqual(prev, next);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
//import { ControlComponent } from '../Form';
const AntdCheckbox = React.memo((props) => {
    const { data, id, enabled, handleChange, uiOptions = {}, editing } = props;
    const autoFocus = !!uiOptions.focus;
    const [currentData, setCurrentData] = useState(data);
    const onChange = (ev) => {
        const newData = ev.target.checked;
        setCurrentData(newData);
        handleChange(newData);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Checkbox, { checked: currentData || false, onChange: onChange, id: id, disabled: !enabled, autoFocus: autoFocus }));
}, areEqual);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/*export interface DateControl {
  dateLocale?: Dayjs;
}*/
const AntdInputDate = (props) => {
    const { data, enabled, handleChange, editing, inputRef } = props;
    const localeDateTimeFormat = 'L';
    const [currentData, setCurrentData] = useState(data);
    const onChange = (date) => {
        const dateNonNullStr = date ? date.format(localeDateTimeFormat) : '';
        setCurrentData(dateNonNullStr);
        handleChange(dateNonNullStr);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(DatePicker
    // id={id + '-input'}
    , { 
        // id={id + '-input'}
        ref: inputRef, value: dayjs(currentData) || null, onChange: onChange, format: localeDateTimeFormat, disabled: !enabled }));
};
/*export interface StatePropsOfDateControl extends StatePropsOfControl {
  defaultLabel: string;
  cancelLabel: string;
  clearLabel: string;
}*/

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdInputNumber = React.memo((props) => {
    const { data, className, id, enabled, handleChange, editing, inputRef } = props;
    const toNumber = (value) => (value === '' ? undefined : parseInt(value, 10));
    const appliedUiSchemaOptions = {};
    const [currentData, setCurrentData] = useState(data);
    const onChange = (ev) => {
        const newData = toNumber(ev.target.value);
        setCurrentData(newData);
        handleChange(newData);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Input, { ref: inputRef, type: 'number', value: currentData || '', onChange: onChange, className: className, id: id, disabled: !enabled, autoFocus: appliedUiSchemaOptions.focus }));
}, areEqual);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdInputText = React.memo((props) => {
    const { data, id, enabled, handleChange, editing, inputRef, onValidation = () => { } } = props;
    const appliedUiSchemaOptions = {};
    const [currentData, setCurrentData] = useState(data);
    const onChange = (ev) => {
        const newData = ev.target.value;
        setCurrentData(newData);
    };
    const onSave = (ev) => {
        const newData = ev.target.value;
        setCurrentData(newData);
        handleChange(newData);
        onValidation(newData);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Input, { ref: inputRef, type: appliedUiSchemaOptions.format === 'password' ? 'password' : 'text', onChange: onChange, onPressEnter: onSave, onBlur: onSave, value: currentData || '', id: id, disabled: !enabled, autoFocus: appliedUiSchemaOptions.focus }));
}, (prevProps, nextProps) => isEqual(prevProps, nextProps));

const AntdInputTime = React.memo((props /*CellProps & WithClassname*/) => {
    const { data, className, enabled, path, handleChange } = props;
    return (React.createElement(TimePicker, { value: data || '', onChange: (time) => handleChange(path, time ? time.format('LTS') : ''), className: className, disabled: !enabled }));
}, areEqual);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdSelect = React.memo((props /*EnumCellProps & WithClassname*/) => {
    const { data, className, id, enabled, path, handleChange, options = [], config } = props;
    const appliedUiSchemaOptions = merge({}, config);
    console.log('OPTIONS', options);
    return (React.createElement(Select, { className: className, id: id, style: { width: 120 }, disabled: !enabled, autoFocus: appliedUiSchemaOptions.focus, value: data || '', onChange: (value) => handleChange(path, value) }, [
        React.createElement(Select.Option, { value: '', key: 'empty' }, ''),
    ].concat(options.map((optionValue) => (React.createElement(Select.Option, { value: optionValue, key: optionValue }, optionValue))))));
}, areEqual);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TinyMCE$1 = React.memo((props) => {
    const { data, handleChange, onMeasureChange = () => { }, editing } = props;
    const [activeEditor, setActiveEditor] = useState();
    const { store } = useContext(MstContext);
    const localeId = store.getLocaleId();
    let language = undefined;
    if (localeId === 'de_DE')
        language = 'de';
    else if (localeId === 'it_IT')
        language = 'it';
    else if (localeId === 'pt_PT')
        language = 'pt_PT';
    else if (localeId === 'pt_BR')
        language = 'pt_BR';
    else if (localeId === 'ru_RU')
        language = 'ru';
    // eslint-disable-next-line @typescript-eslint/naming-convention
    let language_url = undefined; // en_US by default
    if (localeId === 'de_DE')
        language_url = `${process.env.PUBLIC_URL}/lang/de.js`;
    else if (localeId === 'it_IT')
        language_url = `${process.env.PUBLIC_URL}/lang/it.js`;
    else if (localeId === 'pt_PT')
        language_url = `${process.env.PUBLIC_URL}/lang/pt_PT.js`;
    else if (localeId === 'pt_BR')
        language_url = `${process.env.PUBLIC_URL}/lang/pt_BR.js`;
    else if (localeId === 'ru_RU')
        language_url = `${process.env.PUBLIC_URL}/lang/ru.js`;
    console.log('TinyMCE', { localeId, language_url });
    const onChange = (ev) => {
        onMeasureChange();
    };
    const onSave = (ev) => {
        const newData = ev.target.targetElm.innerHTML;
        handleChange(newData);
    };
    useEffect(() => {
        if (!editing) {
            if (activeEditor)
                activeEditor.setContent(data);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [editing, data]);
    return (React.createElement("div", { style: { position: 'relative' } },
        React.createElement(Editor, { initialValue: data || '', inline: true, init: {
                setup: (editor) => setActiveEditor(editor),
                branding: false,
                menubar: false,
                toolbar: false,
                plugins: ['autolink codesample link lists media paste table image quickbars textpattern'],
                quickbars_insert_toolbar: 'paste quicktable image',
                quickbars_selection_toolbar: 'bold italic | h2 h3 | blockquote quicklink',
                contextmenu: 'inserttable | cell row column deletetable | link image imagetools | codesample',
                language_url,
                language,
                skin: 'oxide',
                skin_url: `${process.env.PUBLIC_URL}/skins/ui/oxide`,
                content_css: `${process.env.PUBLIC_URL}/skins/ui/oxide/content.min.css`,
                theme: 'silver',
                theme_url: `${process.env.PUBLIC_URL}/themes/silver/theme.min.js`,
            }, 
            //plugins='link image code'
            //toolbar='undo redo | bold italic | alignleft aligncenter alignright | code'
            onEditorChange: onChange, onBlur: onSave })));
}, (prevProps, nextProps) => isEqual(prevProps, nextProps));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/**
 * Constant that indicates that a tester is not capable of handling
 * a combination of schema/data.
 * @type {number}
 */
const NOT_APPLICABLE = -1;
/**
 * A tester that allow composing other testers by && them.
 *
 * @param {Array<Tester>} testers the testers to be composed
 */
const and = (...testers) => (viewKindElement, schema) => testers.reduce((acc, tester) => acc && tester(viewKindElement, schema), true);
/**
 * A tester that allow composing other testers by || them.
 *
 * @param {Array<Tester>} testers the testers to be composed
 */
const or = (...testers) => (viewKindElement, schema) => testers.reduce((acc, tester) => acc || tester(viewKindElement, schema), false);
/**
 * Create a ranked tester that will associate a number with a given tester, if the
 * latter returns true.
 *
 * @param {number} rank the rank to be returned in case the tester returns true
 * @param {Tester} tester a tester
 */
const rankWith = (rank, tester) => (viewKindElement, schema) => {
    if (tester(viewKindElement, schema)) {
        return rank;
    }
    return NOT_APPLICABLE;
};
/**
 * Checks whether the given UI schema has the expected type.
 *
 * @param {string} expected the expected UI schema type
 */
const uiTypeIs = (expected) => (viewKindElement) => !isEmpty(viewKindElement) && viewKindElement['@type'] === expected;
/**
 * Checks whether the given UI schema has an option with the given
 * name and whether it has the expected value. If no options property
 * is set, returns false.
 *
 * @param {string} optionName the name of the option to check
 * @param {any} optionValue the expected value of the option
 */
const optionIs = (optionName, optionValue) => (viewKindElement) => {
    if (isEmpty(viewKindElement)) {
        return false;
    }
    const options = viewKindElement.options;
    return !isEmpty(options) && options[optionName] === optionValue;
};
const hasType = (jsonSchema, expected) => {
    return includes(deriveTypes(jsonSchema), expected);
};
/**
 * Derives the type of the jsonSchema element
 */
const deriveTypes = (jsonSchema) => {
    if (isEmpty(jsonSchema)) {
        return [];
    }
    if (!isEmpty(jsonSchema.type) && typeof jsonSchema.type === 'string') {
        return [jsonSchema.type];
    }
    if (Array.isArray(jsonSchema.type)) {
        return jsonSchema.type;
    }
    if (!isEmpty(jsonSchema.properties) || !isEmpty(jsonSchema.additionalProperties)) {
        return ['object'];
    }
    if (!isEmpty(jsonSchema.items)) {
        return ['array'];
    }
    if (!isEmpty(jsonSchema.allOf)) {
        const allOfType = find(jsonSchema.allOf, (schema) => deriveTypes(schema).length !== 0);
        if (allOfType) {
            return deriveTypes(allOfType);
        }
    }
    // ignore all remaining cases
    return [];
};
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and applies
 * the given predicate
 *
 * @param {(JsonSchema) => boolean} predicate the predicate that should be
 *        applied to the resolved sub-schema
 */
const schemaMatches = (predicate) => (viewKindElement, schema) => {
    if (isEmpty(viewKindElement) /*|| !isControl(uischema)*/) {
        return false;
    }
    if (isEmpty(schema)) {
        return false;
    }
    //const schemaPath = uischema.scope;
    //if (isEmpty(schemaPath)) {
    //  return false;
    //}
    const currentDataSchema = schema;
    //if (hasType(schema, 'object')) {
    //  currentDataSchema = resolveSchema(schema, schemaPath);
    //}
    //if (currentDataSchema === undefined) {
    //  return false;
    //}
    return predicate(currentDataSchema);
};
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and checks
 * whether the type of the sub-schema matches the expected one.
 *
 * @param {string} expectedType the expected type of the resolved sub-schema
 */
const schemaTypeIs = (expectedType) => schemaMatches((schema) => !isEmpty(schema) && hasType(schema, expectedType));
/**
 * Only applicable for Controls.
 *
 * This function checks whether the given UI schema is of type Control
 * and if so, resolves the sub-schema referenced by the control and checks
 * whether the format of the sub-schema matches the expected one.
 *
 * @param {string} expectedFormat the expected format of the resolved sub-schema
 */
const formatIs = (expectedFormat) => schemaMatches((schema) => !isEmpty(schema) && schema.format === expectedFormat && schema.type === 'string');
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * has a 'date' format.
 * @type {Tester}
 */
const isDateControl = /*and(uiTypeIs('aldkg:Control'),*/ formatIs('date' /*)*/);
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * has a 'date-time' format.
 * @type {Tester}
 */
const isDateTimeControl = /*and(uiTypeIs('aldkg:Control'),*/ formatIs('date-time' /*)*/);
/**
 * Default tester for boolean.
 * @type {RankedTester}
 */
const isBooleanControl = /*and(uiTypeIs('aldkg:Control'),*/ schemaTypeIs('boolean' /*)*/);
const isEnumControl = 
/*and(
uiTypeIs('aldkg:Control'),*/
or(schemaMatches((schema) => schema.enum !== undefined), schemaMatches((schema) => schema.const !== undefined));
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type integer
 * @type {Tester}
 */
const isIntegerControl = /*and(uiTypeIs('aldkg:Control'),*/ schemaTypeIs('integer' /*)*/);
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type number
 * @type {Tester}
 */
const isNumberControl = /*and(uiTypeIs('aldkg:Control'),*/ schemaTypeIs('number' /*)*/);
/**
 * Tests whether the given UI schema is of type Control and if the schema
 * is of type string
 * @type {Tester}
 */
const isStringControl = /*and(uiTypeIs('aldkg:Control'),*/ schemaTypeIs('string' /*)*/);
/**
 * Tests whether a given UI schema is of type Control,
 * if the schema is of type number or integer and
 * whether the schema defines a numerical range with a default value.
 * @type {Tester}
 */
const isRangeControl = and(
/*uiTypeIs('aldkg:Control'),*/
or(schemaTypeIs('number'), schemaTypeIs('integer')), schemaMatches((schema) => schema.maximum !== undefined && schema.minimum !== undefined && schema.default !== undefined), optionIs('slider', true));

const spriteSheet = {
    height: {
        small: 16,
        normal: 24,
        big: 32,
    },
    width: {
        small: 16,
        normal: 24,
        big: 32,
    },
    sprites: {
        act: {
            x: 0,
            y: 338,
            small: {
                x: 400,
                y: 388,
            },
        },
        actor: {
            x: 25,
            y: 338,
            small: {
                x: 375,
                y: 363,
            },
        },
        'allocated-requirement': {
            x: 50,
            y: 338,
            small: {
                x: 350,
                y: 338,
            },
        },
        'analysis-document': {
            x: 75,
            y: 338,
            small: {
                x: 446,
                y: 75,
            },
        },
        'architectural-requirement': {
            x: 100,
            y: 338,
            small: {
                x: 446,
                y: 92,
            },
        },
        'behavioral-requirement': {
            x: 125,
            y: 338,
            small: {
                x: 446,
                y: 109,
            },
        },
        'blue-diagram': {
            x: 371,
            y: 100,
            small: {
                x: 446,
                y: 330,
            },
        },
        blue_file: {
            x: 175,
            y: 363,
            small: {
                x: 187,
                y: 438,
            },
        },
        'blue-heading': {
            x: 396,
            y: 25,
            small: {
                x: 340,
                y: 438,
            },
        },
        'blue-requirement': {
            x: 200,
            y: 388,
            small: {
                x: 255,
                y: 455,
            },
        },
        'blue-subheading': {
            x: 421,
            y: 275,
            small: {
                x: 471,
                y: 119,
            },
        },
        'business-rule': {
            x: 150,
            y: 338,
            small: {
                x: 446,
                y: 126,
            },
        },
        'CSV-file': {
            x: 200,
            y: 338,
            small: {
                x: 446,
                y: 160,
            },
        },
        critique: {
            x: 175,
            y: 338,
            small: {
                x: 446,
                y: 143,
            },
        },
        'customer-input': {
            x: 225,
            y: 338,
            small: {
                x: 446,
                y: 177,
            },
        },
        'customer-requirement': {
            x: 250,
            y: 338,
            small: {
                x: 446,
                y: 194,
            },
        },
        'defect-input': {
            x: 275,
            y: 338,
            small: {
                x: 446,
                y: 211,
            },
        },
        'derived-requirement': {
            x: 300,
            y: 338,
            small: {
                x: 446,
                y: 228,
            },
        },
        'design-brief': {
            x: 325,
            y: 338,
            small: {
                x: 446,
                y: 245,
            },
        },
        'design-constraints': {
            x: 371,
            y: 0,
            small: {
                x: 446,
                y: 262,
            },
        },
        'design-requirement': {
            x: 371,
            y: 25,
            small: {
                x: 446,
                y: 279,
            },
        },
        'detailed-usecase': {
            x: 371,
            y: 50,
            small: {
                x: 446,
                y: 296,
            },
        },
        'development-workitem': {
            x: 371,
            y: 75,
            small: {
                x: 446,
                y: 313,
            },
        },
        discussion: {
            x: 371,
            y: 250,
            small: {
                x: 0,
                y: 438,
            },
        },
        documentation: {
            x: 25,
            y: 363,
            small: {
                x: 85,
                y: 438,
            },
        },
        'elucidation-document': {
            x: 50,
            y: 363,
            small: {
                x: 102,
                y: 438,
            },
        },
        'enhancement-request': {
            x: 75,
            y: 363,
            small: {
                x: 119,
                y: 438,
            },
        },
        'enterprise-usecase': {
            x: 100,
            y: 363,
            small: {
                x: 136,
                y: 438,
            },
        },
        'experience-report': {
            x: 125,
            y: 363,
            small: {
                x: 153,
                y: 438,
            },
        },
        'feature-description': {
            x: 150,
            y: 363,
            small: {
                x: 170,
                y: 438,
            },
        },
        filtered: {
            x: 225,
            y: 363,
            small: {
                x: 221,
                y: 438,
            },
        },
        'focus-group-feedback': {
            x: 275,
            y: 363,
            small: {
                x: 255,
                y: 438,
            },
        },
        'functional-requirement': {
            x: 300,
            y: 363,
            small: {
                x: 272,
                y: 438,
            },
        },
        'general-document': {
            x: 371,
            y: 275,
            small: {
                x: 17,
                y: 438,
            },
        },
        glossary: {
            x: 325,
            y: 363,
            small: {
                x: 289,
                y: 438,
            },
        },
        'goal-modeling': {
            x: 350,
            y: 363,
            small: {
                x: 306,
                y: 438,
            },
        },
        'green-diagram': {
            x: 371,
            y: 125,
            small: {
                x: 446,
                y: 347,
            },
        },
        'green-file': {
            x: 200,
            y: 363,
            small: {
                x: 204,
                y: 438,
            },
        },
        'green-heading': {
            x: 396,
            y: 50,
            small: {
                x: 357,
                y: 438,
            },
        },
        'green-requirement': {
            x: 225,
            y: 388,
            small: {
                x: 272,
                y: 455,
            },
        },
        'green-subheading': {
            x: 421,
            y: 300,
            small: {
                x: 471,
                y: 136,
            },
        },
        guideline: {
            x: 396,
            y: 0,
            small: {
                x: 323,
                y: 438,
            },
        },
        'html-document': {
            x: 371,
            y: 300,
            small: {
                x: 34,
                y: 438,
            },
        },
        image: {
            x: 396,
            y: 100,
            small: {
                x: 391,
                y: 438,
            },
        },
        'interface-requirement': {
            x: 396,
            y: 125,
            small: {
                x: 408,
                y: 438,
            },
        },
        interviews: {
            x: 396,
            y: 150,
            small: {
                x: 425,
                y: 438,
            },
        },
        javascript: {
            x: 396,
            y: 175,
            small: {
                x: 442,
                y: 438,
            },
        },
        media: {
            x: 396,
            y: 200,
            small: {
                x: 0,
                y: 455,
            },
        },
        smallutes: {
            x: 396,
            y: 225,
            small: {
                x: 17,
                y: 455,
            },
        },
        module: {
            x: 396,
            y: 250,
            small: {
                x: 34,
                y: 455,
            },
        },
        'non-functional-requirement': {
            x: 396,
            y: 275,
            small: {
                x: 51,
                y: 455,
            },
        },
        observations: {
            x: 396,
            y: 300,
            small: {
                x: 68,
                y: 455,
            },
        },
        'orange-diagram': {
            x: 371,
            y: 150,
            small: {
                x: 446,
                y: 364,
            },
        },
        'PDF-document': {
            x: 371,
            y: 325,
            small: {
                x: 51,
                y: 438,
            },
        },
        part: {
            x: 396,
            y: 325,
            small: {
                x: 85,
                y: 455,
            },
        },
        'performance-requirement': {
            x: 396,
            y: 350,
            small: {
                x: 102,
                y: 455,
            },
        },
        persona: {
            x: 0,
            y: 388,
            small: {
                x: 119,
                y: 455,
            },
        },
        plan: {
            x: 25,
            y: 338,
            small: {
                x: 136,
                y: 455,
            },
        },
        presentation: {
            x: 50,
            y: 388,
            small: {
                x: 153,
                y: 455,
            },
        },
        process: {
            x: 75,
            y: 388,
            small: {
                x: 170,
                y: 455,
            },
        },
        'process-diagram': {
            x: 50,
            y: 413,
            small: {
                x: 471,
                y: 238,
            },
        },
        prototypes: {
            x: 100,
            y: 388,
            small: {
                x: 187,
                y: 455,
            },
        },
        'purple-diagram': {
            x: 371,
            y: 175,
            small: {
                x: 446,
                y: 381,
            },
        },
        'purple-heading': {
            x: 396,
            y: 75,
            small: {
                x: 374,
                y: 438,
            },
        },
        'purple-subheading': {
            x: 421,
            y: 325,
            small: {
                x: 471,
                y: 153,
            },
        },
        'quality-requirements': {
            x: 125,
            y: 388,
            small: {
                x: 204,
                y: 455,
            },
        },
        'RTTP-file': {
            x: 350,
            y: 388,
            small: {
                x: 357,
                y: 455,
            },
        },
        'red-diagram': {
            x: 371,
            y: 200,
            small: {
                x: 446,
                y: 398,
            },
        },
        'red-requirement': {
            x: 250,
            y: 388,
            small: {
                x: 289,
                y: 455,
            },
        },
        report: {
            x: 150,
            y: 388,
            small: {
                x: 221,
                y: 455,
            },
        },
        'reqpro-document': {
            x: 175,
            y: 388,
            small: {
                x: 238,
                y: 455,
            },
        },
        'requirement-list': {
            x: 325,
            y: 388,
            small: {
                x: 340,
                y: 455,
            },
        },
        reviews: {
            x: 300,
            y: 388,
            small: {
                x: 323,
                y: 455,
            },
        },
        scenario: {
            x: 375,
            y: 388,
            small: {
                x: 374,
                y: 455,
            },
        },
        scene: {
            x: 421,
            y: 0,
            small: {
                x: 391,
                y: 455,
            },
        },
        screenflow: {
            x: 421,
            y: 25,
            small: {
                x: 408,
                y: 455,
            },
        },
        set: {
            x: 421,
            y: 50,
            small: {
                x: 425,
                y: 455,
            },
        },
        'simple-usecase': {
            x: 421,
            y: 100,
            small: {
                x: 471,
                y: 0,
            },
        },
        sketch: {
            x: 421,
            y: 125,
            small: {
                x: 471,
                y: 17,
            },
        },
        'software-requirement-module': {
            x: 421,
            y: 75,
            small: {
                x: 442,
                y: 455,
            },
        },
        specifications: {
            x: 421,
            y: 150,
            small: {
                x: 471,
                y: 34,
            },
        },
        spreadsheet: {
            x: 421,
            y: 175,
            small: {
                x: 471,
                y: 51,
            },
        },
        'stakeholder-request': {
            x: 421,
            y: 200,
            small: {
                x: 471,
                y: 68,
            },
        },
        storyboard: {
            x: 421,
            y: 225,
            small: {
                x: 471,
                y: 85,
            },
        },
        'structural-requirements': {
            x: 421,
            y: 250,
            small: {
                x: 471,
                y: 102,
            },
        },
        summary: {
            x: 421,
            y: 350,
            small: {
                x: 471,
                y: 170,
            },
        },
        'support-document': {
            x: 421,
            y: 375,
            small: {
                x: 471,
                y: 187,
            },
        },
        'system-requirement': {
            x: 25,
            y: 413,
            small: {
                x: 471,
                y: 221,
            },
        },
        'system-requirement-module': {
            x: 0,
            y: 413,
            small: {
                x: 471,
                y: 204,
            },
        },
        'tape-archive': {
            x: 75,
            y: 413,
            small: {
                x: 471,
                y: 255,
            },
        },
        'teal-diagram': {
            x: 371,
            y: 225,
            small: {
                x: 446,
                y: 415,
            },
        },
        term: {
            x: 100,
            y: 413,
            small: {
                x: 471,
                y: 272,
            },
        },
        test: {
            x: 150,
            y: 413,
            small: {
                x: 471,
                y: 306,
            },
        },
        'test-plans': {
            x: 125,
            y: 413,
            small: {
                x: 471,
                y: 289,
            },
        },
        'text-document': {
            x: 200,
            y: 413,
            small: {
                x: 471,
                y: 340,
            },
        },
        'text-document-with-diagrams': {
            x: 175,
            y: 413,
            small: {
                x: 471,
                y: 323,
            },
        },
        'UI-specification': {
            x: 225,
            y: 413,
            small: {
                x: 471,
                y: 357,
            },
        },
        'unknown-object': {
            x: 250,
            y: 413,
            small: {
                x: 471,
                y: 374,
            },
        },
        'use-case': {
            x: 300,
            y: 413,
            small: {
                x: 471,
                y: 408,
            },
        },
        'usecase-diagram': {
            x: 275,
            y: 413,
            small: {
                x: 471,
                y: 391,
            },
        },
        'user-requirement': {
            x: 325,
            y: 413,
            small: {
                x: 371,
                y: 425,
            },
        },
        'user-stories': {
            x: 350,
            y: 413,
            small: {
                x: 471,
                y: 442,
            },
        },
        validate: {
            x: 375,
            y: 413,
            small: {
                x: 0,
                y: 472,
            },
        },
        vision: {
            x: 400,
            y: 413,
            small: {
                x: 17,
                y: 472,
            },
        },
        wireframe: {
            x: 446,
            y: 0,
            small: {
                x: 34,
                y: 472,
            },
        },
        'word-processing-document': {
            x: 0,
            y: 363,
            small: {
                x: 68,
                y: 438,
            },
        },
        'XML-file': {
            x: 446,
            y: 25,
            small: {
                x: 51,
                y: 472,
            },
        },
        'yellow-file': {
            x: 250,
            y: 363,
            small: {
                x: 238,
                y: 438,
            },
        },
        'yellow-requirement': {
            x: 275,
            y: 388,
            small: {
                x: 306,
                y: 455,
            },
        },
        'ZIP-file': {
            x: 446,
            y: 50,
            small: {
                x: 68,
                y: 472,
            },
        },
        default: {
            x: 175,
            y: 413,
            small: {
                x: 471,
                y: 323,
            },
        },
    },
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const Sprite = ({ filename, x = 0, y = 0, width = 0, height = 0 }) => {
    if (!filename) {
        return null;
    }
    const style = {
        backgroundImage: `url(${filename})`,
        backgroundPosition: `${x * -1}px ${y * -1}px`,
        backgroundRepeat: 'no-repeat',
        width,
        height,
    };
    return React.createElement("div", { style: style });
};
const base = ({ value }) => React.createElement("div", { style: { margin: '6px' } }, value);
const integer = ({ value }) => value.toString();
const identifier = ({ value }) => (React.createElement(Link, { to: `/artifacts/${value}` }, value.toString().padStart(5, '0')));
const image = ({ value }) => (React.createElement("div", { style: { height: 60, float: 'left', paddingRight: '3px', overflow: 'hidden' } },
    React.createElement(Image, { style: { height: 60, maxWidth: 60, width: 'auto' }, src: Array.isArray(value) ? (value.length > 0 ? value[0] : '') : value })));
const artifactTitle = ({ value, type }) => {
    const SpriteObj = {
        'rmUserTypes:_YwcOsRmREemK5LEaKhoOow_Module': 'allocated-requirement',
        'rmUserTypes:_YwcOsRmREemK5LEaKhoOow_Text': 'analysis-document',
        'rmUserTypes:_YwcOsRmREemK5LEaKhoOow_Collection': 'behavioral-requirement',
    };
    return (React.createElement("div", { style: { display: 'flex' } },
        type && (React.createElement("div", { style: { display: 'inline-block', padding: '3px 5px 0 0' } },
            React.createElement(Sprite, { filename: process.env.PUBLIC_URL + '/img/rm-artifact-icons-sprite.png', x: spriteSheet.sprites[SpriteObj[type]].x, y: spriteSheet.sprites[SpriteObj[type]].y, width: spriteSheet.width.normal, height: spriteSheet.width.normal }))),
        React.createElement("div", { style: { overflow: 'hidden', textOverflow: 'ellipsis' } }, value)));
};
const dateTime = ({ value }) => React.createElement("span", null, dayjs(value).format('DD.MM.YYYY'));
const link = ({ value, link, options }) => {
    const label = options.label || value || link;
    const specialImage = options.specialImage;
    return (React.createElement(React.Fragment, null,
        specialImage ? (React.createElement(Image, { style: { display: 'inline', width: '1em', height: '1em' }, src: specialImage, preview: false })) : null,
        React.createElement("a", { style: { verticalAlign: 'middle' }, href: link ? link : value }, label)));
};
const extLink = ({ value, link, options }) => {
    const label = options.label || value || link;
    const icon = options.icon;
    return (React.createElement(React.Fragment, null, icon ? (React.createElement(React.Fragment, null,
        label,
        React.createElement("a", { style: { verticalAlign: 'top', padding: '0px 16px 2px 6px ' }, href: link ? link : value, target: '_blank', rel: 'noreferrer' },
            React.createElement(Image, { style: { display: 'inline', width: '16px', height: '16px' }, src: icon, preview: false })))) : (React.createElement("a", { style: { verticalAlign: 'middle' }, href: link ? link : value, target: '_blank', rel: 'noreferrer' }, label))));
};
const labeledValue = ({ value, options }) => {
    const label = options.label || null;
    const specialChar = options.specialChar || '';
    return (React.createElement(React.Fragment, null,
        React.createElement("span", null, label),
        " : ",
        React.createElement("span", null, `${specialChar}${value}`)));
};
const comparison = ({ value, prevValue, options }) => {
    const delta = value - prevValue;
    const label = options.label;
    return (React.createElement(React.Fragment, null,
        label ? React.createElement("span", null, `${label} : `) : null,
        React.createElement("span", { style: { color: delta < 0 ? '#F7021B' : '#0CED1B' } }, `${delta < 0 ? '↓' : '↑'}${value}`),
        React.createElement("sup", null, `${delta < 0 ? '' : '+'}${delta}`)));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const StoreDataFormatter = observer(({ value, query, propKey }) => {
    const { store } = useContext(MstContext);
    const coll = store.rep.getColl(query);
    if (coll === undefined) {
        return React.createElement("span", null, value);
    }
    const data = coll.dataJs;
    const element = data.find((e) => e['@id'] === value);
    return React.createElement("span", null, element ? element[propKey || 'title'] : value);
});

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TinyMCE = React.memo(({ value, isScrolling, onMeasureChange, height }) => {
    return (React.createElement("div", { className: 'mce-content-body', style: { borderColor: 'rgba(0,0,0,0)' }, dangerouslySetInnerHTML: { __html: value } }));
}, (prevProps, nextProps) => isEqual(prevProps, nextProps));

const formatters = {
    base,
    identifier,
    dateTime,
    artifactTitle,
    image,
    integer,
    link,
    extLink,
    tinyMCE: TinyMCE,
    dataFormatter: StoreDataFormatter,
    labeledValue,
    comparison,
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const CellRenderer = ({ Cell, uiOptions = {}, handleChange, myRef, rowData, ...props }) => {
    const [editing, setEditing] = useState(false);
    const [currentData, setCurrentData] = useState(props.data);
    const [fontSize, setFontSize] = useState();
    const inputRef = useRef(null);
    const cellRef = useRef(null);
    const formatterId = uiOptions.formatter || 'base';
    const query = uiOptions.query;
    let width = 'auto';
    let editable = true;
    if (uiOptions.editable !== undefined) {
        editable = uiOptions.editable;
    }
    if (uiOptions && formatterId === 'tinyMCE') {
        width = uiOptions.tinyWidth === 'emptySpace' ? '100%' : uiOptions.tinyWidth === 'content' ? 'auto' : 'auto';
    }
    const Formatter = formatters[formatterId] || formatters['base'];
    useEffect(() => {
        if (editing && inputRef && inputRef.current) {
            inputRef.current.focus();
        }
    }, [editing]);
    const toggleEdit = () => {
        setEditing(!editing);
    };
    //console.log('RENDER');
    const save = async (data) => {
        setCurrentData(data);
        handleChange(data);
        toggleEdit();
    };
    const specialProps = {};
    if (uiOptions.dataToFormatter) {
        const formatterProps = uiOptions.dataToFormatter;
        for (const key in formatterProps) {
            specialProps[key] = rowData[formatterProps[key]];
        }
    }
    useEffect(() => {
        const resizeObserver = new ResizeObserver((entries) => {
            setFontSize(entries[0].contentRect.width * uiOptions.relativeFont);
        });
        if (uiOptions.relativeFont) {
            const antRow = cellRef.current.parentNode.parentNode;
            resizeObserver.observe(antRow);
        }
        return () => resizeObserver.disconnect();
    }, []);
    //console.log('RENDER', props.data);
    return (React.createElement("div", { ref: cellRef, style: {
            margin: '5px',
            display: 'block',
            alignSelf: 'start',
            width: width,
            fontSize,
            ...uiOptions.style,
        } }, editable ? (editing ? (React.createElement(Cell, { inputRef: inputRef, value: currentData, handleChange: save, ...props })) : (React.createElement("div", { onClick: toggleEdit },
        React.createElement(Formatter, { myRef: myRef, query: query, propKey: uiOptions.key, options: uiOptions, value: currentData, ...specialProps, ...props })))) : (React.createElement(Formatter, { value: props.data, query: query, propKey: uiOptions.key, options: uiOptions, ...specialProps, ...props }))));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/**
 * AntdBooleanCell
 */
const AntdBooleanCell = (props) => {
    return React.createElement(CellRenderer, { Cell: AntdCheckbox, ...props });
};
const antdBooleanCellTester = rankWith(2, isBooleanControl);
const AntdBooleanCellWithStore = withStoreToCellProps(AntdBooleanCell);
/**
 * AntdButtonCell
 */
const AntdButtonCell = (props) => {
    const options = props.uiOptions;
    const [fontSize, setFontSize] = useState();
    const cellRef = useRef(null);
    useEffect(() => {
        const resizeObserver = new ResizeObserver((entries) => {
            setFontSize(entries[0].contentRect.width * options.relativeFont);
        });
        if (options.relativeFont) {
            const antRow = cellRef.current.parentNode.parentNode;
            resizeObserver.observe(antRow);
        }
        return () => resizeObserver.disconnect();
    }, []);
    return (React.createElement("button", { ref: cellRef, style: { fontSize, verticalAlign: 'middle', ...options.style } }, options.label));
};
const antdButtonCellTester = rankWith(2, uiTypeIs('aldkg:Button'));
const AntdButtonCellWithStore = withStoreToCellProps(AntdButtonCell);
/**
 * AntdEnumCell
 */
const AntdEnumCell = (props /*: EnumCellProps & WithClassname*/) => React.createElement(AntdSelect, { ...props });
const antdEnumCellTester = rankWith(2, isEnumControl);
const AntdEnumCellWithStore = withStoreToCellProps(AntdEnumCell);
/**
 * AntdNumberCell & AntdIntegerCell
 */
const AntdNumberCell = (props) => {
    return React.createElement(CellRenderer, { Cell: AntdInputNumber, ...props });
};
const antdIntegerCellTester = rankWith(2, isIntegerControl);
const AntdIntegerCellWithStore = withStoreToCellProps(AntdNumberCell);
const antdNumberCellTester = rankWith(2, isNumberControl);
const AntdNumberCellWithStore = withStoreToCellProps(AntdNumberCell);
/**
 * AntdImageCell
 */
const AntdImageCell = (props) => {
    const { data } = props;
    return (React.createElement(Image, { width: '100%', src: Array.isArray(data) ? data[0] || '' : data && typeof data === 'string' ? data : '' }));
};
const antdImageCellTester = rankWith(2, uiTypeIs('aldkg:ImageCell'));
const AntdImageCellWithStore = withStoreToCellProps(AntdImageCell);
/**
 * AntdRateCell
 */
const AntdRateCell = (props /*: EnumCellProps & WithClassname*/) => {
    return (React.createElement(React.Fragment, null,
        React.createElement(Rate, { style: { fontSize: '14px', color: 'rgb(255,140,0)', marginTop: '7px' }, allowHalf: true, disabled: true, defaultValue: props.data }),
        React.createElement("span", { style: { padding: '4px', fontSize: '12px', fontWeight: 500 } }, `(${props.data})`)));
};
const antdRateCellTester = rankWith(5, uiTypeIs('aldkg:Rate'));
const AntdRateCellWithStore = withStoreToCellProps(AntdRateCell);
/**
 * AntdTextCell
 */
const AntdTextCell = (props) => {
    return React.createElement(CellRenderer, { Cell: AntdInputText, ...props });
};
const antdTextCellTester = rankWith(1, isStringControl);
const AntdTextCellWithStore = withStoreToCellProps(AntdTextCell);
/**
 * AntdTimeCell
 */
const AntdTimeCell = (props) => {
    return React.createElement(CellRenderer, { Cell: AntdInputDate, ...props });
};
const antdTimeCellTester = rankWith(2, isDateTimeControl);
const AntdTimeCellWithStore = withStoreToCellProps(AntdTimeCell);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TinyMCECell = (props) => {
    return React.createElement(CellRenderer, { Cell: TinyMCE$1, ...props });
};
/**
 * Default tester for text-based/string controls.
 * @type {RankedTester}
 */
const tinyMCECellTester = rankWith(10, (viewKindElement, schema) => viewKindElement.options !== undefined && viewKindElement.options.formatter === 'tinyMCE');
const TinyMCECellWithStore = withStoreToCellProps(TinyMCECell);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
/**
 * Dispatch renderer component for cells.
 */
const DispatchCell = React.memo(({ viewKind, viewKindElement, viewDescr, viewDescrElement, schema, data, onMeasureChange, uri, enabled, id, CKey, rowData, ...rest }) => {
    const { cells } = useContext(MstContext);
    if (schema && schema.items)
        schema = { ...schema, ...schema.items };
    const renderer = maxBy(cells, (r) => r.tester(viewKindElement, schema));
    if (renderer === undefined || renderer.tester(viewKindElement, schema) === -1) {
        return (React.createElement("td", null,
            React.createElement(UnknownRenderer, { type: 'renderer', elementId: viewKindElement['@id'], elementType: viewKindElement['@type'] })));
    }
    else {
        const Render = renderer.cell;
        return (React.createElement(ErrorBoundary, { FallbackComponent: ErrorFallback, onReset: () => { } },
            React.createElement(Render, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, CKey: CKey, data: data, rowData: rowData, onMeasureChange: onMeasureChange, uri: uri, enabled: enabled, id: id, ...rest })));
    }
}, (prevProps, nextProps) => isEqual(prevProps, nextProps));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdCellCardLayout = (props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, schema, data, id } = props;
    const style = viewKindElement?.options?.style || {};
    const createCardChilds = () => viewKindElement.elements
        ? viewKindElement.elements.map((e, idx) => {
            const newSchema = e.scope ? get(schema, 'properties.' + e.scope.replace(/\//, '.properties.')) : schema;
            return (React.createElement(DispatchCell, { id: id + String(idx), key: id + String(idx), viewKind: viewKind, viewKindElement: e, viewDescr: viewDescr, viewDescrElement: viewDescrElement, data: data, rowData: data, schema: newSchema || schema }));
        })
        : null;
    return (React.createElement(Card$1, { style: style, hoverable: true }, createCardChilds()));
};
/**
 * Default tester for text-based/string controls.
 * @type {RankedTester}
 */
const antdCellCardLayoutTester = rankWith(2, uiTypeIs('aldkg:CardLayout'));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdCellHorizontalLayoutRenderer = ({ viewKind, viewKindElement, viewDescr, viewDescrElement, data, schema, }) => {
    //const layout = viewKindElement as Layout;
    const Render = ({ idx, schema, viewKind, viewKindElement, viewDescr, viewDescrElement, data, enabled, form, }) => {
        const options = viewKindElement.options || {};
        //const style: any = options.style;
        const span = options.contentSize || !viewKindElement.elements ? undefined : Math.ceil(24 / viewKindElement.elements.length);
        const newSchema = viewKindElement.scope
            ? get(schema, 'properties.' + viewKindElement.scope.replace(/\//, '.properties.'))
            : schema;
        return (React.createElement(Col, { key: idx, span: span },
            React.createElement(DispatchCell, { id: String(idx), data: data, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, rowData: data, schema: newSchema || schema, enabled: enabled, form: form })));
    };
    const justify = viewKindElement.options ? viewKindElement.options.justify : 'center';
    const rowStyle = { flexWrap: 'nowrap' };
    if (viewKindElement.options && viewKindElement.options.width === 'all-empty-space')
        rowStyle.width = '100%';
    return (React.createElement(Row$1, { justify: justify, style: rowStyle, align: 'middle' }, (viewKindElement.elements || []).map((e, idx) => (React.createElement(Render, { key: idx, viewKind: viewKind, viewKindElement: e, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, idx: idx, data: data, id: String(idx) })))));
};
const antdCellHorizontalLayoutTester = rankWith(2, uiTypeIs('aldkg:CellHorizontalLayout'));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const template = {
    'https://www.wildberries.ru/catalog/15570386/detail.aspx': '/img/chart_1.png',
    'https://www.wildberries.ru/catalog/15622789/detail.aspx': '/img/chart_2.png',
    'https://www.wildberries.ru/catalog/16170086/detail.aspx': '/img/chart_3.png',
    'https://www.wildberries.ru/catalog/18247707/detail.aspx': '/img/chart_4.png',
};
const AntdCellG2 = (props) => {
    return React.createElement(Image, { style: { height: '217px' }, src: template[props.data['@id']] });
};
/**
 * Default tester for text-based/string controls.
 * @type {RankedTester}
 */
const antdCellG2Tester = rankWith(2, uiTypeIs('aldkg:G2'));
const AntdCellG2WithStore = withStoreToCellProps(AntdCellG2);

const antdCells = [
    { tester: antdBooleanCellTester, cell: AntdBooleanCellWithStore },
    { tester: antdEnumCellTester, cell: AntdEnumCellWithStore },
    { tester: antdIntegerCellTester, cell: AntdIntegerCellWithStore },
    { tester: antdNumberCellTester, cell: AntdNumberCellWithStore },
    { tester: antdTextCellTester, cell: AntdTextCellWithStore },
    { tester: antdTimeCellTester, cell: AntdTimeCellWithStore },
    { tester: tinyMCECellTester, cell: TinyMCECellWithStore },
    { tester: antdImageCellTester, cell: AntdImageCellWithStore },
    { tester: antdCellCardLayoutTester, cell: AntdCellCardLayout },
    { tester: antdCellHorizontalLayoutTester, cell: AntdCellHorizontalLayoutRenderer },
    { tester: antdRateCellTester, cell: AntdRateCellWithStore },
    { tester: antdCellG2Tester, cell: AntdCellG2 },
    { tester: antdButtonCellTester, cell: AntdButtonCellWithStore },
];

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdBooleanControl = (props) => {
    return (React.createElement(Row$1, null,
        React.createElement(Col, { span: 8 },
            React.createElement("span", null, props.label)),
        React.createElement(Col, { span: 16 },
            React.createElement(AntdCheckbox, { ...props }))));
};
const antdBooleanControlTester = rankWith(2, isBooleanControl);
const AntdBooleanControlWithStore = withStoreToControlProps(AntdBooleanControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
var warning = (valid, component, message) => {
    warning$1(valid, `[antd-compatible: ${component}] ${message}`);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
// moved from https://github.com/ant-design/ant-design/blob/master/components/icon/utils.ts
const fillTester = /-fill$/;
const outlineTester = /-o$/;
const twoToneTester = /-twotone$/;
function getThemeFromTypeName(type) {
    let result = null;
    if (fillTester.test(type)) {
        result = 'filled';
    }
    else if (outlineTester.test(type)) {
        result = 'outlined';
    }
    else if (twoToneTester.test(type)) {
        result = 'twoTone';
    }
    return result;
}
function removeTypeTheme(type) {
    return type.replace(fillTester, '').replace(outlineTester, '').replace(twoToneTester, '');
}
const themeMap = {
    filled: 'filled',
    outlined: 'outlined', // default theme
    twoTone: 'twoTone',
};
function withThemeSuffix(type, theme) {
    const result = upperFirst(camelCase(type));
    const realTheme = upperFirst(themeMap[theme]);
    if (theme !== 'outlined' && !realTheme) {
        warning(false, 'Icon', `This icon '${type}' has unknown theme '${theme}'`);
    }
    return result + realTheme;
}
// For alias or compatibility
function alias(type) {
    let newType = type;
    switch (type) {
        case 'cross':
            newType = 'close';
            break;
        // https://github.com/ant-design/ant-design/issues/13007
        case 'interation':
            newType = 'interaction';
            break;
        // https://github.com/ant-design/ant-design/issues/16810
        case 'canlendar':
            newType = 'calendar';
            break;
        // https://github.com/ant-design/ant-design/issues/17448
        case 'colum-height':
            newType = 'column-height';
            break;
    }
    warning(newType === type, 'Icon', `Icon '${type}' was a typo and is now deprecated, please use '${newType}' instead.`);
    return newType;
}

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const iconsMap = allIcons;
const LegacyTypeIcon = (props) => {
    const { type, theme } = props;
    if (theme) {
        const themeInName = getThemeFromTypeName(type);
        warning(!themeInName || theme === themeInName, 'Icon', `The icon name '${type}' already specify a theme '${themeInName}',` +
            ` the 'theme' prop '${theme}' will be ignored.`);
    }
    const computedType = withThemeSuffix(removeTypeTheme(alias(type)), theme || 'outlined');
    const targetIconComponent = iconsMap[computedType];
    warning(targetIconComponent, 'Icon', `The icon name '${type}'${theme ? `with ${theme}` : ''} doesn't exist, please check it at https://ant.design/components/icon`);
    return targetIconComponent ? React.createElement(targetIconComponent, props) : null;
};
/*const Icon: IconComponent<IconProps> = props => {
  const { type, component, children } = props;

  upgradeMessage('Icon');

  warning(
    Boolean(type || component || children),
    'Icon',
    'Should have `type` prop or `component` prop or `children`.',
  );

  if (component || children) {
    return <AntdIcon {...props} />;
  }

  if (typeof type === 'string') {
    return <LegacyTypeIcon {...props} type={type} />;
  }

  return <AntdIcon />;
};

Icon.createFromIconfontCN = createFromIconfontCN;
Icon.getTwoToneColor = getTwoToneColor;
Icon.setTwoToneColor = setTwoToneColor;

export default Icon;*/

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdButtonControl = ({ handleChange, options = {} }) => {
    return (React.createElement(Button, { onClick: handleChange, className: options.icon ? 'ant-btn-icon-only' : '' },
        options.icon ? React.createElement(LegacyTypeIcon, { type: options.icon, theme: 'outlined' }) : null,
        options.content ? options.content : null));
};
const antdButtonControlTester = rankWith(2, uiTypeIs('aldkg:Button'));
const AntdButtonControlWithStore = withStoreToButtonProps(AntdButtonControl);

/**
 * Adds an asterisk to the given label string based
 * on the required parameter.
 *
 * @param {string} label the label string
 * @param {boolean} required whether the label belongs to a control which is required
 * @returns {string} the label string
 */
const computeLabel = (label, required, hideRequiredAsterisk) => {
    return required && !hideRequiredAsterisk ? label + '*' : label;
};
const isPlainLabel = (label) => {
    return typeof label === 'string';
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdDateControl = (props) => {
    const { id, label, editing, enabled, required, handleChange, data, dateLocale, uiOptions = {} } = props;
    const defaultLabel = label;
    const localeDateTimeFormat = dateLocale ? `${dateLocale.localeData().longDateFormat('L')}` : 'YYYY-MM-DD';
    const [currentData, setCurrentData] = useState(data);
    let labelText;
    if (isPlainLabel(label)) {
        labelText = label;
    }
    else {
        labelText = defaultLabel;
    }
    const onChange = (date) => {
        const dateNonNullStr = date ? date.format(localeDateTimeFormat) : 'YYYY-MM-DD';
        setCurrentData(dateNonNullStr);
        handleChange(dateNonNullStr);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Row$1, null,
        React.createElement(Col, { span: 8 },
            React.createElement("label", { className: 'label-nowrap', htmlFor: id },
                computeLabel(labelText, required, uiOptions.hideRequiredAsterisk),
                ":")),
        React.createElement(Col, { span: 16 },
            React.createElement(DatePicker
            // id={id + '-input'}
            , { 
                // id={id + '-input'}
                value: dayjs(currentData) || null, onChange: onChange, format: localeDateTimeFormat, disabled: !enabled, autoFocus: uiOptions.focus }))));
};
const antdDateControlTester = rankWith(4, isDateControl);
const AntdDateControlWithStore = withStoreToControlProps(AntdDateControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdInputControl = (props) => {
    const { id, errors, label, visible, required, input, form, formData, validateObj, onValidation, editing, uiOptions = {}, } = props;
    const isValid = errors.length === 0;
    const appliedUiSchemaOptions = {};
    const InnerComponent = input;
    const formatterId = uiOptions.formatter || 'base';
    const readOnly = uiOptions.readOnly;
    const query = uiOptions.query;
    const specialProps = {};
    if (uiOptions.dataToFormatter) {
        const formatterProps = uiOptions.dataToFormatter;
        for (const key in formatterProps) {
            specialProps[key] = formData[formatterProps[key]];
        }
    }
    const Formatter = formatters[formatterId] || formatters['base'];
    return form ? (React.createElement(Form$1.Item, { label: React.createElement("span", { style: { display: 'block', width: '100%' } }, computeLabel(isPlainLabel(label) ? label : label.default, required, appliedUiSchemaOptions.hideRequiredAsterisk)), validateStatus: validateObj.validateStatus, help: validateObj.help }, editing && !readOnly ? (React.createElement(InnerComponent, { ...props, onValidation: onValidation, id: id + '-input', saveKey: id.split('/')[1], isValid: isValid, visible: visible })) : (React.createElement(Formatter, { query: query, propKey: uiOptions.key, value: props.data, ...specialProps, ...props })))) : (React.createElement(Row$1, { style: { width: '100%' } },
        React.createElement(Col, { span: 8 },
            React.createElement("label", { className: 'label-nowrap', htmlFor: id },
                computeLabel(isPlainLabel(label) ? label : label.default, required, appliedUiSchemaOptions.hideRequiredAsterisk),
                ":")),
        React.createElement(Col, { span: 16 },
            React.createElement("div", { className: 'input-margin' },
                React.createElement(InnerComponent, { ...props, id: id + '-input', saveKey: id.split('/')[1], isValid: isValid, visible: visible })))));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdEnumControl = (props) => React.createElement(AntdInputControl, { ...props, input: AntdSelect });
const antdEnumControlTester = rankWith(2, isEnumControl);
const AntdEnumControlWithStore = withStoreToControlProps(AntdEnumControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdNumberControl = (props) => (React.createElement(AntdInputControl, { ...props, input: AntdInputNumber }));
//export const antdNumberControlTester: RankedTester = rankWith(2, or(isNumberControl, isIntegerControl));
const antdNumberControlTester = rankWith(1, or(isIntegerControl, isNumberControl));
const AntdNumberControlWithStore = withStoreToControlProps(AntdNumberControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdRadioGroupControl = (props) => {
    const { id, data, enabled, label, schema, handleChange, required, editing, uiOptions = {} } = props;
    const [currentData, setCurrentData] = useState(data);
    const onChange = (value) => {
        setCurrentData(value);
        handleChange(value);
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Row$1, null,
        React.createElement(Col, { span: 8 },
            React.createElement("span", null, props.label)),
        React.createElement(Col, { span: 16 },
            React.createElement(Radio.Group, { value: currentData, onChange: onChange }, uiOptions.map((optionValue, idx) => (React.createElement(Radio, { key: idx, checked: data === optionValue }, optionValue)))))));
};
const antdRadioGroupControlTester = rankWith(2, optionIs('format', 'radio'));
const AntdRadioGroupControlWithStore = withStoreToControlProps(AntdRadioGroupControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdSliderControl = (props) => {
    const { id, data, enabled, label, schema, handleChange, required, editing, uiOptions = {} } = props;
    const [currentData, setCurrentData] = useState(data);
    const onChange = (value) => {
        setCurrentData(Number(value));
        handleChange(Number(value));
    };
    useEffect(() => {
        if (!editing) {
            setCurrentData(data);
        }
    }, [editing, data]);
    return (React.createElement(Row$1, { style: { width: '100%' } },
        React.createElement(Col, { span: 8 },
            React.createElement(Typography, { id: id + '-typo' }, computeLabel(isPlainLabel(label) ? label : label.default, required, uiOptions.hideRequiredAsterisk))),
        React.createElement(Col, { span: 16 },
            React.createElement("div", { className: 'input-margin' },
                React.createElement(Slider, { min: schema.minimum, max: schema.maximum, value: Number(currentData || ''), onChange: onChange, id: id + '-input', disabled: !enabled, step: schema.multipleOf || 1 })))));
};
const antdSliderControlTester = rankWith(4, isRangeControl);
const AntdSliderControlWithStore = withStoreToControlProps(AntdSliderControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdTextControl = (props) => (React.createElement(AntdInputControl, { ...props, input: AntdInputText }));
const antdTextControlTester = rankWith(1, isStringControl);
const AntdTextControlWithStore = withStoreToControlProps(AntdTextControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TinyMCEControl = (props) => {
    return React.createElement(AntdInputControl, { ...props, input: TinyMCE$1 });
};
const tinyMCEControlTester = rankWith(3, (v, s) => {
    return s && s.contentMediaType === 'text/html';
});
const TinyMCEControlWithStore = withStoreToControlProps(TinyMCEControl);

const AntdImageControl = (props) => {
    const { uiOptions, data } = props;
    return (React.createElement(Image, { src: Array.isArray(data) ? data[0] || '' : data && typeof data === 'string' ? data : '', fallback: uiOptions.fallback }));
};
const antdImageControlTester = rankWith(3, uiTypeIs('aldkg:Image'));
const AntdImageControlWithStore = withStoreToControlProps(AntdImageControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const positionIndependentComp = (lowVal) => (a, b) => {
    if (a === b) {
        return 0;
    }
    const bInd = b.toLowerCase().indexOf(lowVal);
    const aInd = a.toLowerCase().indexOf(lowVal);
    if (bInd === -1 && aInd === -1) {
        return a.localeCompare(b);
    }
    if (bInd === -1) {
        return -1;
    }
    if (aInd === -1) {
        return 1;
    }
    if (aInd < bInd) {
        return -1;
    }
    if (aInd > bInd) {
        return 1;
    }
    return 0;
};
const SearchableList = ({ dataSource, defaultSelectedKey, onClick = () => { }, }) => {
    const [optionList, setOptionList] = useState(dataSource);
    const [selectedItem, setSelectedItem] = useState(defaultSelectedKey);
    const [previousSearchValue, setPreviousSearchValue] = useState(null);
    const columns = [
        {
            title: 'title',
            dataIndex: 'title',
            key: 'key',
            render: (item) => {
                return {
                    props: {
                        style: item === selectedItem ? { background: 'rgba(24, 144, 255, 0.35)' } : {},
                    },
                    children: React.createElement("div", null, item),
                };
            },
        },
    ];
    const onInputSearch = (event) => {
        const lowVal = event.target.value.toLowerCase();
        if (previousSearchValue == null) {
            setPreviousSearchValue(lowVal);
        }
        let sortedOptionList = [];
        if (previousSearchValue && (!lowVal.startsWith(previousSearchValue) || lowVal === previousSearchValue)) {
            sortedOptionList = dataSource;
        }
        else {
            sortedOptionList = optionList.slice();
        }
        sortedOptionList = sortedOptionList
            .filter((word) => {
            return word.title.toLowerCase().indexOf(lowVal) !== -1;
        })
            .sort((a, b) => positionIndependentComp(lowVal)(a.title, b.title));
        if (sortedOptionList && sortedOptionList.length) {
            setPreviousSearchValue(lowVal);
        }
        setOptionList(sortedOptionList);
    };
    useEffect(() => {
        setOptionList(dataSource);
    }, [dataSource]);
    return (React.createElement("div", null,
        React.createElement(Input.Search, { style: { marginBottom: '20px', marginTop: '20px' }, placeholder: '\u041F\u043E\u0438\u0441\u043A \u0444\u0438\u043B\u044C\u0442\u0440\u0430', onChange: onInputSearch }),
        React.createElement(Table, { bordered: false, size: 'small', dataSource: optionList, onRow: (record, rowIndex) => {
                return {
                    onClick: () => {
                        setSelectedItem(record.item);
                        onClick(record, rowIndex);
                    },
                };
            }, columns: columns, style: { width: '100%' }, pagination: false, showHeader: false, scroll: { y: 240 } })));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
function renderTreeNode(listOfValues, searchValue) {
    return listOfValues.map((value, index) => {
        let foundIndex = -1;
        let beforeStr = '';
        let middleStr = '';
        let afterStr = '';
        let title;
        if (searchValue !== '') {
            foundIndex = value.toLowerCase().indexOf(searchValue);
            if (foundIndex !== -1) {
                beforeStr = value.substr(0, foundIndex);
                middleStr = value.substr(foundIndex, searchValue.length);
                afterStr = value.substr(foundIndex + searchValue.length);
                title = (React.createElement("span", null,
                    beforeStr,
                    React.createElement("span", { style: { color: '#f50' } }, middleStr),
                    afterStr));
            }
            else {
                title = '';
            }
        }
        else {
            title = React.createElement("span", null, value);
        }
        return {
            title,
            key: index.toString(),
        };
        // return <Tree.TreeNode title={title} key={index.toString()} active={true} />;
    });
}
const localeRus$3 = {
    searchValues: 'Поиск значения',
};
const BodyOfListFilter = ({ handleChange, defaultValues, property, context, type, }) => {
    const [searchValue, setSearchValue] = useState('');
    const [checkedKeys, setCheckedKeys] = useState([]);
    const [listOfAliasOfEnum, setListOfAliasOfEnum] = useState([]);
    const [listOfValuesOfEnum, setListOfValuesOfEnum] = useState([]);
    const [valuesIsLoad, setValuesIsLoad] = useState(true);
    const { store } = React.useContext(MstContext);
    useEffect(() => {
        const loadEnumValues = async (iri) => {
            const values = await store.getData(iri);
            setListOfValuesOfEnum(values.map((val) => val['@id']));
            setListOfAliasOfEnum(values.map((val) => val.title || val.name));
            setValuesIsLoad(true);
        };
        setValuesIsLoad(false);
        if (context !== undefined && typeof context === 'object' && context['@type'] !== undefined) {
            loadEnumValues(context['@type']);
        }
        else if (context === 'rdf:type' &&
            property.enum !== undefined &&
            property.enum.length > 0 &&
            typeof property.enum[0] === 'object') {
            setListOfValuesOfEnum(property.enum.map((item) => item && item.value));
            setListOfAliasOfEnum(property.enum.map((item) => item && item.alias));
            setValuesIsLoad(true);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [property]);
    const findKeysItemByValueNames = (values) => {
        const keys = [];
        listOfValuesOfEnum.forEach((val, index) => {
            if (values.includes(val)) {
                keys.push(index.toString());
            }
        });
        return keys;
    };
    const searchValueInTree = (e) => {
        const value = e.target.value.toLowerCase();
        setSearchValue(value);
    };
    useEffect(() => {
        setCheckedKeys(findKeysItemByValueNames(defaultValues.value));
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [defaultValues, listOfValuesOfEnum]);
    if (type === 'enumerate') {
        return (React.createElement("div", { style: { height: '300px', overflow: 'auto' } },
            React.createElement(Input.Search, { placeholder: localeRus$3.searchValues, style: { marginTop: 15 }, onChange: searchValueInTree }),
            valuesIsLoad && (property.type === 'object' || (property.type === 'string' && property.format === 'iri')) && (React.createElement(Tree, { style: { marginTop: 15 }, checkable: true, checkedKeys: checkedKeys, treeData: renderTreeNode(listOfAliasOfEnum || listOfValuesOfEnum, searchValue), onCheck: (keys) => {
                    setCheckedKeys(keys);
                    if (keys.length > 0) {
                        handleChange({
                            value: keys.map((key) => listOfValuesOfEnum[parseInt(key)]),
                            valueName: listOfAliasOfEnum ? keys.map((key) => listOfAliasOfEnum[parseInt(key)]) : [],
                        });
                    }
                    else {
                        handleChange({ value: [], valueName: [] });
                    }
                } }))));
    }
    return React.createElement(React.Fragment, null);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const localeRus$2 = {
    inputOneOrMoreNumbers: 'Введите одно или более чисел (через символ [Пробел] или запятая)',
    equal: 'равняется',
    any: 'любой из',
};
const equalTypesOfRelations = (typeA, typeB) => {
    const keyWord = 'hidden';
    let indexOfKeyword = typeA && typeA.indexOf(keyWord) !== -1 ? typeA.indexOf(keyWord) + keyWord.length : 0;
    const firstType = typeA && typeA.substring(indexOfKeyword);
    indexOfKeyword = typeB && typeB.indexOf(keyWord) !== -1 ? typeB.indexOf(keyWord) + keyWord.length : 0;
    const secondType = typeB && typeB.substring(indexOfKeyword);
    return firstType === secondType;
};
const BodyOfIntegerFilter = ({ handleChange, defaultValues, type }) => {
    const relationForOneNumber = { title: localeRus$2.equal, predicate: 'equal', type: 'singleNumber' };
    const relationForSomeNumbers = { title: localeRus$2.any, predicate: 'any', type: 'hiddenSingleNumber' };
    const [inputValue, setInputValue] = useState(defaultValues.value.join(', '));
    useEffect(() => {
        setInputValue(defaultValues.value.join(', '));
    }, [defaultValues]);
    const onChange = (e) => {
        const { value } = e.target;
        const reg = /^([1-9][0-9]*)([,\s][0-9]*)*$/;
        if (reg.test(value) || value === '') {
            setInputValue(value);
            handleChange({ value: value ? [value] : [], valueName: [] });
        }
    };
    const onBlur = () => {
        const arrayOfValue = inputValue.split(/[\s,]/).filter((val) => val !== '');
        setInputValue(arrayOfValue.join(', '));
        const newValue = { value: arrayOfValue, valueName: [] };
        if (arrayOfValue.length > 1) {
            newValue['relation'] = relationForSomeNumbers;
        }
        else {
            newValue['relation'] = relationForOneNumber;
        }
        handleChange(newValue);
    };
    if (equalTypesOfRelations(type, 'singleNumber')) {
        return (React.createElement(Tooltip, { trigger: 'focus', title: localeRus$2.inputOneOrMoreNumbers, placement: 'bottomLeft' },
            React.createElement(Input, { value: inputValue, style: { marginTop: 25 }, onChange: onChange, onBlur: onBlur })));
    }
    return React.createElement(React.Fragment, null);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const BodyOfDateTimeFilter = ({ handleChange, defaultValues, type }) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(QueryIRI);
    const timeValue = [locale.daysAgo, locale.monthsAgo, locale.yearsAgo];
    const [checkedOption, setCheckedOption] = useState(null);
    const [firstValueInThreeVariant, setFirstValueInThreeVariant] = useState(null);
    const [secondValueInThreeVariant, setSecondValueInThreeVariant] = useState(timeValue[0]);
    useEffect(() => {
        let newValue = {};
        if (firstValueInThreeVariant && secondValueInThreeVariant) {
            let timeOption = 'days';
            if (secondValueInThreeVariant.localeCompare(timeValue[0]) === 0) {
                timeOption = 'days';
            }
            else if (secondValueInThreeVariant.localeCompare(timeValue[1]) === 0) {
                timeOption = 'months';
            }
            else if (secondValueInThreeVariant.localeCompare(timeValue[2]) === 0) {
                timeOption = 'years';
            }
            const foundDate = dayjs().subtract(parseInt(firstValueInThreeVariant), timeOption).format('YYYY-MM-DD');
            newValue = {
                value: [`${foundDate}T00:00:00`],
                valueName: [dayjs(foundDate, 'YYYY-MM-DD').fromNow()],
            };
        }
        else {
            newValue = { value: [], valueName: [] };
        }
        handleChange(newValue);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [firstValueInThreeVariant, secondValueInThreeVariant]);
    const onChangeRatioButton = (e) => {
        const { value } = e.target;
        setCheckedOption(value);
        if (value === 1) {
            const today = `${dayjs().format('YYYY-MM-DD')}T00:00:00`;
            handleChange({ value: [today], valueName: [locale.today] });
        }
        else if (value === 2) {
            const yesterday = `${dayjs().subtract(1, 'days').format('YYYY-MM-DD')}T00:00:00`;
            handleChange({ value: [yesterday], valueName: [locale.yesterday] });
        }
        else if (value === 3) {
            if (firstValueInThreeVariant && secondValueInThreeVariant) {
                let timeOption = 'days';
                if (secondValueInThreeVariant.localeCompare(timeValue[0]) === 0) {
                    timeOption = 'days';
                }
                else if (secondValueInThreeVariant.localeCompare(timeValue[1]) === 0) {
                    timeOption = 'months';
                }
                else if (secondValueInThreeVariant.localeCompare(timeValue[2]) === 0) {
                    timeOption = 'years';
                }
                const foundDate = dayjs().subtract(parseInt(firstValueInThreeVariant), timeOption).format('YYYY-MM-DD');
                handleChange({
                    value: [`${foundDate}T00:00:00`],
                    valueName: [dayjs(foundDate, 'YYYY-MM-DD').fromNow()],
                });
            }
            else {
                handleChange({ value: [], valueName: [] });
            }
        }
        else {
            handleChange({ value: [], valueName: [] });
        }
    };
    if (type === 'singleDate') {
        return (React.createElement(DatePicker, { showTime: true, defaultValue: defaultValues.value[0], format: dayjs.defaultFormat, style: { marginTop: 25 }, onChange: (date, dateString) => {
                handleChange({
                    value: dateString === ''
                        ? []
                        : Array.isArray(dateString)
                            ? []
                            : [dayjs(dateString, dayjs.defaultFormat).format('YYYY-MM-DDThh:mm:ss')],
                    valueName: [dateString],
                });
            } }));
    }
    if (type === 'chooseVariant') {
        const radioStyle = {
            display: 'block',
            height: '40px',
            lineHeight: '30px',
        };
        return (React.createElement("div", { style: { height: '300px', overflow: 'auto' } },
            React.createElement(Radio.Group, { onChange: onChangeRatioButton, defaultValue: defaultValues.value[0] ? 4 : null },
                React.createElement(Radio, { style: radioStyle, value: 1 }, locale.today),
                React.createElement(Radio, { style: radioStyle, value: 2 }, locale.yesterday),
                React.createElement(Radio, { style: radioStyle, value: 3 },
                    React.createElement(InputNumber, { min: 0, onChange: (value) => {
                            if (value) {
                                let newValue = value.toString();
                                newValue = newValue !== '' ? newValue : null;
                                setFirstValueInThreeVariant(newValue);
                            }
                        } }),
                    React.createElement(Select, { style: { marginLeft: 10 }, defaultValue: 0, onChange: (key) => {
                            setSecondValueInThreeVariant(timeValue[key]);
                        } }, timeValue.map((value, index) => {
                        return (React.createElement(Select.Option, { key: index, value: index }, value));
                    }))),
                React.createElement(Radio, { style: radioStyle, value: 4 },
                    locale.date,
                    ":",
                    checkedOption === 4 || defaultValues.value[0] ? (React.createElement(DatePicker, { showTime: true, style: { marginLeft: 10 }, defaultValue: defaultValues.value[0], format: dayjs.defaultFormat, onChange: (date, dateString) => {
                            handleChange({
                                value: dateString === ''
                                    ? []
                                    : Array.isArray(dateString)
                                        ? []
                                        : [dayjs(dateString, dayjs.defaultFormat).format('YYYY-MM-DDThh:mm:ss')],
                                valueName: [],
                            });
                        } })) : null))));
    }
    if (type === 'twoDate') {
        return (React.createElement(DatePicker.RangePicker, { showTime: true, style: { marginTop: 25 }, defaultValue: defaultValues.value, format: dayjs.defaultFormat, onChange: (date, dateString) => {
                handleChange({
                    value: dateString[0] === '' || dateString[1] === ''
                        ? []
                        : dateString.map((rangeDate) => dayjs(rangeDate, dayjs.defaultFormat).format('YYYY-MM-DDThh:mm:ss')),
                    valueName: [],
                });
            } }));
    }
    return React.createElement(React.Fragment, null);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const localeRus$1 = {
    regExExample: 'Пример регулярного выражения: "^Привет"',
    regExStartString: '^ - Начало строки',
    regExEndString: '$ - Конец строки',
    regExDocTitle: 'Документация',
    regExDoc: 'Подробная инструкция по использованию на',
    regExDocLink: 'сайте документации',
};
const BodyOfStringFilter = ({ handleChange, defaultValues, type }) => {
    const [inputValue, setInputValue] = useState(defaultValues.value[0]);
    useEffect(() => {
        setInputValue(defaultValues.value[0]);
    }, [defaultValues]);
    const onChange = (e) => {
        const { value } = e.target;
        setInputValue(value);
        handleChange({ value: value ? [value] : [], valueName: [] });
    };
    if (type === 'singleString') {
        return React.createElement(Input, { style: { marginTop: '1em' }, value: inputValue, onChange: onChange });
    }
    if (type === 'regEx') {
        return (React.createElement(React.Fragment, null,
            React.createElement(Input, { style: { marginTop: '1em', marginBottom: '1em' }, value: inputValue, onChange: onChange }),
            React.createElement(Typography.Text, { type: 'secondary' }, localeRus$1.regExExample),
            React.createElement("br", null),
            React.createElement(Typography.Text, { type: 'secondary' }, localeRus$1.regExStartString),
            React.createElement("br", null),
            React.createElement(Typography.Text, { type: 'secondary' }, localeRus$1.regExEndString),
            React.createElement("br", null),
            React.createElement(Typography.Text, { type: 'secondary' },
                React.createElement("span", null, `${localeRus$1.regExDoc} `),
                React.createElement("a", { href: 'https://www.w3.org/TR/xpath-functions/#regex-syntax', title: localeRus$1.regExDocTitle, target: 'blank' }, localeRus$1.regExDocLink))));
    }
    return React.createElement(React.Fragment, null);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const genFormattedTypeName = (propName, type, format) => {
    let resultType = 'iri';
    if (propName === 'assetFolder') {
        return 'assetFolder';
    }
    if (type === 'integer') {
        resultType = 'integer';
    }
    else if (type === 'string') {
        if (format === 'date-time') {
            resultType = 'date-time';
        }
        else if (format === 'iri') {
            resultType = 'object';
        }
        else {
            resultType = 'string';
        }
    }
    else if (type === 'object') {
        resultType = 'object';
    }
    else {
        resultType = 'iri';
    }
    // if (resultType === 'iri' && propName === 'assetFolder') {
    //   resultType = 'assetFolder';
    // }
    return resultType;
};
const genFormattedTypeOfFilter = (propName, property) => {
    const formattedType = {
        type: 'none',
        isArray: false,
    };
    if (property.type === 'array' && property.items) {
        formattedType.isArray = true;
        const items = property.items;
        formattedType.type = genFormattedTypeName(propName, items.type, items.format);
    }
    else {
        formattedType.type = genFormattedTypeName(propName, property.type, property.format);
    }
    return formattedType;
};
const getTypeOfRelation = (currentRelations, nameOfSelectRelation) => {
    const foundRelation = currentRelations.find((relation) => relation.predicate === nameOfSelectRelation);
    return foundRelation && foundRelation.type;
};
const localeRus = {
    contains: 'содержит',
    notContains: 'не содержит',
    equal: 'равняется',
    startWith: 'начинается с',
    endWith: 'заканчивается на',
    regEx: 'регулярное выражение',
    exists: 'существует',
    notExists: 'не существует',
    unassigned: 'не назначен',
    any: 'любой из',
    notAny: 'ни один из',
    each: 'каждый из',
    notEach: 'все кроме',
    dateEqual: 'в',
    dateNotEqual: 'не в',
    after: 'после (включая)',
    before: 'до (включая)',
    between: 'между',
    today: 'Сегодня',
    fromYesterdayToToday: 'Со вчера на сегодня',
};
const Filter = ({ filterData, setFilter, prevFilterData, propName, property, context, isRequired, modalIsMount = false, }) => {
    const relations = {
        object: [
            { title: localeRus.any, predicate: 'any', type: 'enumerate' },
            { title: localeRus.notAny, predicate: 'notAny', type: 'enumerate' },
        ],
        array: [
            { title: localeRus.any, predicate: 'any', type: 'enumerate' },
            { title: localeRus.notAny, predicate: 'notAny', type: 'enumerate' },
            { title: localeRus.each, predicate: 'each', type: 'enumerate' },
            { title: localeRus.notEach, predicate: 'notEach', type: 'enumerate' },
        ],
        string: [
            { title: localeRus.contains, predicate: 'contains', type: 'singleString' },
            { title: localeRus.notContains, predicate: 'notContains', type: 'singleString' },
            { title: localeRus.equal, predicate: 'equal', type: 'singleString' },
            { title: localeRus.startWith, predicate: 'startWith', type: 'singleString' },
            { title: localeRus.endWith, predicate: 'endWith', type: 'singleString' },
            { title: localeRus.regEx, predicate: 'regEx', type: 'regEx' },
        ],
        'date-time': [
            { title: localeRus.dateEqual, predicate: 'equal', type: 'singleDate' },
            { title: localeRus.dateNotEqual, predicate: 'notEqual', type: 'singleDate' },
            { title: localeRus.after, predicate: 'after', type: 'chooseVariant' },
            { title: localeRus.before, predicate: 'before', type: 'chooseVariant' },
            { title: localeRus.between, predicate: 'between', type: 'twoDate' },
        ],
        integer: [
            { title: localeRus.equal, predicate: 'equal', type: 'singleNumber' },
            { title: localeRus.any, predicate: 'any', type: 'hiddenSingleNumber' },
        ],
        isNotRequired: [
            { title: localeRus.exists, predicate: 'exists', type: 'noValue' },
            { title: localeRus.notExists, predicate: 'notExists', type: 'noValue' },
            // { title: t('CreateFilter.unassigned'), predicate: 'unassigned', type: 'noValue' },
        ],
        assetFolder: [{ title: '', predicate: 'equal', type: 'singleString' }],
        noRelations: [{ title: localeRus.equal, predicate: 'equal', type: 'singleString' }],
    };
    const getDefaultValue = (typeOfFilter, prevFilter, typeOfCurrentRelation, typeOfPrevRelation) => {
        let defaultValue = { value: [], valueName: [] };
        if (typeOfFilter.type === 'date-time') {
            if (equalTypesOfRelations(typeOfCurrentRelation, 'twoDate')) {
                defaultValue = {
                    value: [dayjs().subtract(1, 'days'), dayjs()],
                    valueName: [localeRus.fromYesterdayToToday],
                };
            }
            else {
                defaultValue = {
                    value: [dayjs()],
                    valueName: [localeRus.today],
                };
            }
        }
        else if (typeOfFilter.type === 'integer') {
            defaultValue = { value: [0] };
        }
        if (prevFilter) {
            if (typeOfPrevRelation && equalTypesOfRelations(typeOfPrevRelation, typeOfCurrentRelation)) {
                defaultValue.value = prevFilter.value;
                defaultValue.valueName = prevFilter.valueName || [];
                if (typeOfFilter.type === 'date-time') {
                    defaultValue.value = prevFilter.value.map((value) => dayjs(value, 'YYYY-MM-DDThh:mm:ss'));
                }
            }
        }
        if (typeOfCurrentRelation && equalTypesOfRelations(typeOfCurrentRelation, 'noValue')) {
            defaultValue = { value: [], valueName: [] };
        }
        return defaultValue;
    };
    const getDefaultRelation = (currentRelations, prevRelation) => {
        let defaultRelation = currentRelations[0];
        if (prevRelation) {
            const foundRelation = currentRelations.find((relation) => relation.predicate === prevRelation.predicate);
            if (foundRelation) {
                defaultRelation = {
                    title: foundRelation.title,
                    predicate: prevRelation.predicate,
                    type: foundRelation.type,
                };
            }
        }
        return defaultRelation;
    };
    const getCurrentRelations = (typeOfFilter, isRequired) => {
        let currentRelations;
        if (typeOfFilter.isArray) {
            currentRelations = relations['array'];
        }
        else {
            currentRelations = relations[typeOfFilter.type] || relations['noRelations'];
        }
        if (!isRequired) {
            currentRelations = currentRelations.concat(relations['isNotRequired']);
        }
        return currentRelations;
    };
    const typeOfFilter = genFormattedTypeOfFilter(propName, property);
    const currentRelations = getCurrentRelations(typeOfFilter, isRequired);
    const prevRelation = prevFilterData && prevFilterData.relation;
    const prevRelationType = prevRelation && getTypeOfRelation(currentRelations, prevRelation.predicate);
    const [defaultRelation, setDefaultRelation] = useState(getDefaultRelation(currentRelations, prevRelation));
    const [defaultValues, setDefaultValues] = useState(getDefaultValue(typeOfFilter, prevFilterData, defaultRelation.type, prevRelationType));
    useEffect(() => {
        const newRelation = getDefaultRelation(currentRelations, prevRelation);
        const newValue = getDefaultValue(typeOfFilter, prevFilterData, newRelation.type, prevRelationType);
        setDefaultRelation(newRelation);
        setDefaultValues(newValue);
        setFilter({
            ...filterData,
            relation: newRelation,
            value: newValue.value,
            valueName: newValue.valueName,
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [propName]);
    useEffect(() => {
        setFilter({
            ...filterData,
            relation: defaultRelation,
            value: defaultValues.value,
            valueName: defaultValues.valueName,
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [modalIsMount]);
    const handleChangeRelation = (changedRelation) => {
        if (changedRelation && changedRelation.predicate !== filterData.relation.predicate) {
            let newValue = getDefaultValue(typeOfFilter, prevFilterData, changedRelation.type, prevRelationType);
            if (filterData.relation &&
                changedRelation &&
                equalTypesOfRelations(filterData.relation.type, changedRelation.type)) {
                newValue = { value: filterData.value, valueName: filterData.valueName };
            }
            else {
                setDefaultValues(newValue);
            }
            setFilter({
                ...filterData,
                relation: changedRelation,
                value: newValue.value,
                valueName: newValue.valueName,
            });
            setDefaultRelation(changedRelation);
        }
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Row$1, { style: { marginTop: '20px' } },
            React.createElement(Col, { span: 12 },
                React.createElement("div", null,
                    React.createElement("b", null,
                        " ",
                        property.title,
                        " "),
                    ' ')),
            currentRelations.filter((rel) => rel.type && !rel.type.startsWith('hidden')).length > 1 ? (React.createElement(Col, { span: 12 },
                React.createElement(Select, { style: { minWidth: '120px' }, value: defaultRelation.predicate, onChange: (selectRelName) => {
                        const newRelation = currentRelations.find((rel) => rel.predicate === selectRelName);
                        if (newRelation) {
                            handleChangeRelation(newRelation);
                        }
                    } }, currentRelations
                    .filter((option) => {
                    return option.type && !option.type.startsWith('hidden');
                })
                    .map((option) => {
                    return (React.createElement(Select.Option, { key: option.predicate, value: option.predicate }, option.title));
                })))) : (React.createElement(React.Fragment, null))),
        propName === '@type' ? (React.createElement(BodyOfListFilter, { handleChange: (value) => {
                setFilter({ ...filterData, value: value.value, valueName: value.valueName });
            }, defaultValues: defaultValues, property: property, context: context, type: defaultRelation.type })) : // ) : typeOfFilter.type === 'assetFolder' ? (
            // <BodyOfFolderFilter
            //   handleChange={(value: JsObject) => {
            //     setFilter({ ...filterData, value: value.value, valueName: value.valueName });
            //   }}
            //   defaultValues={defaultValues}
            // />
            typeOfFilter.type === 'object' ? (React.createElement(BodyOfListFilter, { handleChange: (value) => {
                    setFilter({ ...filterData, value: value.value, valueName: value.valueName });
                }, defaultValues: defaultValues, property: property, context: context, type: defaultRelation.type })) : typeOfFilter.isArray ? (React.createElement(React.Fragment, null,
                React.createElement(Typography.Title, null,
                    React.createElement("b", null, defaultValues.valueName)),
                React.createElement(Typography.Title, null, defaultValues.value.toString()))) : typeOfFilter.type === 'integer' ? (React.createElement(BodyOfIntegerFilter, { handleChange: (value) => {
                    setFilter({ ...filterData, ...value });
                }, defaultValues: defaultValues, type: defaultRelation.type })) : typeOfFilter.type === 'date-time' ? (React.createElement(BodyOfDateTimeFilter, { handleChange: (value) => {
                    setFilter({ ...filterData, ...value });
                }, defaultValues: defaultValues, type: defaultRelation.type })) : (React.createElement(BodyOfStringFilter, { handleChange: (value) => {
                    setFilter({ ...filterData, value: value.value, valueName: value.valueName });
                }, defaultValues: defaultValues, type: defaultRelation.type }))));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const CreateFilterModal = ({ loading, handleCancel, handleOk, handleAdd, tags, }) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(QueryIRI);
    const [filter, setFilter] = useState({
        value: [],
        valueName: [],
        relation: { title: '', predicate: '' },
        property: '',
        title: '',
    });
    const [isValidFilter, setIsValidFilter] = useState(false);
    useEffect(() => {
        if (filter && filter.title && filter.property && filter.relation && filter.relation.predicate) {
            if ((filter.relation.title && filter.property !== 'assetFolder') || filter.property === 'assetFolder') {
                if (filter.relation.type !== 'noValue' && (!filter.value || filter.value.length === 0)) {
                    setIsValidFilter(false);
                }
                else {
                    setIsValidFilter(true);
                }
            }
        }
        else {
            setIsValidFilter(false);
        }
    }, [filter]);
    return (React.createElement(Modal, { title: locale.title, visible: true, height: '80%', width: '80%', onOk: () => handleOk(filter), onCancel: handleCancel, footer: [
            React.createElement(Button, { key: 'add', type: 'primary', loading: loading, onClick: () => handleAdd(filter), disabled: !isValidFilter }, locale.add),
            React.createElement(Button, { key: 'submit', type: 'primary', loading: loading, onClick: () => handleOk(filter), disabled: !isValidFilter }, locale.addAndClose),
            React.createElement(Button, { key: 'back', onClick: handleCancel }, locale.close),
        ] },
        React.createElement(CreateFilter, { setFilter: setFilter, filter: filter, isValidFilter: isValidFilter, setIsValidFilter: setIsValidFilter, tags: tags })));
};
const CreateFilter = ({ setFilter, filter, isValidFilter, setIsValidFilter, tags }) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(QueryIRI);
    const [selectedSchema] = useState();
    // useEffect(() => {
    //   setSelectedSchema(artifactTypes[0]);
    // }, [artifactTypes]);
    const artifactTypeFilterItem = { key: '@type', title: locale.artifactType };
    const [selectedFilterType, setSelectedFilterType] = useState(artifactTypeFilterItem.key);
    const [modalIsMount, setModalIsMount] = useState(false);
    const [allProperties, setAllProperties] = useState({ properties: {}, contexts: {} });
    useEffect(() => {
        const loadProperties = async () => {
            const artifactTypes = store.rep.getColl('ArtifactClasses_Coll')?.dataJs || []; //getData('rm:ArtifactClasses');
            const properties = { ...(store.rep.schemas.getOrLoadSchemaByClassIri('rm:Artifact')?.propertiesJs || {}) };
            const contexts = store.rep.schemas.getOrLoadSchemaByClassIri('rm:Artifact')?.['@context'] || {};
            if (properties['@id']) {
                delete properties['@id'];
            }
            properties['@type'] = {
                type: 'string',
                enum: artifactTypes.map((schema) => {
                    return { value: schema['@id'], alias: schema.title || schema['@id'] };
                }),
                format: 'iri',
                '@id': 'rdf:type',
                title: locale.artifactType,
            };
            setAllProperties({ properties, contexts });
        };
        loadProperties();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const [filterTypes, setFilterTypes] = useState([
        { key: '@type', title: locale.artifactType },
    ]);
    useEffect(() => {
        const newFilterTypes = [];
        Object.keys(allProperties.properties).forEach((val) => {
            if (val !== '@type') {
                newFilterTypes.push({ key: val, title: allProperties.properties[val]?.title || val });
            }
        });
        setFilterTypes([...filterTypes, ...newFilterTypes]);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [allProperties]);
    useEffect(() => {
        setFilter({
            ...filter,
            property: filterTypes[0].key,
            title: filterTypes[0].title,
        });
        setModalIsMount(true);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const parseFilter = () => {
        let result = React.createElement(Typography.Text, null, locale.emptyFilter);
        if (filter) {
            const { title } = filter;
            const relation = filter.relation ? filter.relation.title : null;
            const value = filter.valueName && filter.valueName.length > 0 ? filter.valueName.join(', ') : filter.value.join(', ');
            if (title) {
                result = React.createElement(Typography.Text, null, `${title} ${relation || ''} ${value || ''}`);
            }
        }
        let icon = null;
        if (isValidFilter) {
            icon = React.createElement(CheckCircleTwoTone, { style: { marginLeft: '1em' }, twoToneColor: '#52c41a' });
        }
        else {
            icon = React.createElement(ExclamationCircleTwoTone, { style: { marginLeft: '1em' }, twoToneColor: '#f50' });
        }
        return (React.createElement(React.Fragment, null,
            result,
            icon));
    };
    return (React.createElement("div", { style: { height: '100%' } },
        React.createElement(Row$1, null,
            React.createElement(Col, { span: 10 },
                React.createElement("h3", null, locale.choiceAttribute),
                React.createElement(SearchableList, { defaultSelectedKey: filterTypes[0].key, dataSource: filterTypes, onClick: (val) => {
                        setSelectedFilterType(val.key);
                        setFilter({ ...filter, property: val.key, title: val.title });
                    } })),
            React.createElement(Col, { span: 4 }),
            React.createElement(Col, { span: 10 },
                React.createElement("h3", null, locale.choiceValue),
                allProperties.properties[selectedFilterType] && (React.createElement(Filter, { filterData: filter, setFilter: setFilter, prevFilterData: tags.find((tag) => tag.property === selectedFilterType), propName: selectedFilterType, property: allProperties.properties[selectedFilterType], context: allProperties.contexts[selectedFilterType], isRequired: selectedSchema && selectedSchema.required ? selectedSchema.required.includes(selectedFilterType) : false, modalIsMount: modalIsMount })))),
        React.createElement(Row$1, { style: { marginTop: '1em' } },
            React.createElement(Typography.Text, { strong: true },
                " ",
                locale.resultFilter,
                ": ")),
        React.createElement(Row$1, null,
            React.createElement(Typography.Text, null, parseFilter()))));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const QueryIRI = 'aldkg:Query';
const AntQueryWithStore = observer((props) => {
    const { viewKind, viewDescr, schema } = props;
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(QueryIRI);
    const fullTextSearchString = store.fullTextSearchString;
    const setFullTextSearchString = (newValue) => {
        store.fullTextSearchString = newValue;
    };
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const coll = store.rep.getColl(collIriOverride);
    const conditionsJs = coll?.collConstr?.entConstrs[0].conditionsJs;
    const loading = false;
    const removeFilter = (filter) => {
        if (filter?.property && filter?.relation) {
            store.rep.editConn({ toObj: collIriOverride, toProp: filter.property }, { value: filter.value, relation: filter.relation.predicate });
        }
        else if (filter?.property) {
            store.rep.editConn({ toObj: collIriOverride, toProp: filter.property }, filter.value);
        }
    };
    const [inputValue, setInputValue] = useState(fullTextSearchString);
    const [tagsArray, setTags] = useState([]);
    const [isExpanded, setIsExpanded] = useState(false);
    const [modalVisible, setModalVisible] = useState(false);
    const inputRef = useRef(null);
    useEffect(() => {
        let tags = [];
        const filters = {};
        if (conditionsJs) {
            Object.keys(conditionsJs).forEach((k) => {
                if (k !== '@id' && k !== '@type') {
                    let value = conditionsJs[k];
                    let valueName = [];
                    if (typeof value === 'object' && value['@id'] !== undefined && value['@type'] !== undefined) {
                        valueName = [value.title];
                        value = value['@id'];
                    }
                    const newFilter = {
                        title: schema.properties[k].title,
                        property: k,
                        relation: { title: '', predicate: 'equal', type: 'singleString' },
                        value,
                        valueName,
                    };
                    filters[k] = newFilter;
                }
            });
            tags = Object.values(filters);
        }
        const fullTextFilter = tagsArray.find((tag) => tag.title === locale.fullTextSearchValue);
        if (fullTextFilter) {
            tags = tags.concat(fullTextFilter);
        }
        setTags(tags);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [conditionsJs]);
    const handleCloseTag = (removedTag) => {
        if (removedTag.property && removedTag.title !== locale.fullTextSearchValue) {
            removeFilter(removedTag);
        }
        else {
            const newTags = tagsArray.filter((tag) => tag.title !== locale.fullTextSearchValue);
            setTags(newTags);
            setFullTextSearchString('');
        }
    };
    const showModal = () => {
        setModalVisible(true);
    };
    const handleAddFilter = (filter) => {
        if (filter.property === 'assetFolder') ;
    };
    const hideModal = () => {
        setModalVisible(false);
    };
    const handleInputChange = (event) => {
        setInputValue(event.target.value);
    };
    const handleInputConfirm = () => {
        let newTags = tagsArray;
        if (inputValue) {
            newTags = [
                ...newTags.filter((tag) => tag.title !== locale.fullTextSearchValue),
                {
                    title: locale.fullTextSearchValue,
                    value: [inputValue],
                    property: 'search',
                    relation: {
                        predicate: 'contains',
                        title: '',
                    },
                },
            ];
            setFullTextSearchString(inputValue);
        }
        setTags(newTags);
        // setInputVisible(false);
        setInputValue('');
    };
    return (React.createElement("div", null,
        tagsArray &&
            tagsArray.map((tag) => {
                let tagStyle = {};
                if (isExpanded) {
                    tagStyle = {
                        width: '100%',
                        margin: '5px',
                    };
                }
                const value = tag.valueName && tag.valueName.length > 0
                    ? tag.valueName.join(', ')
                    : Array.isArray(tag.value)
                        ? tag.value.join(', ')
                        : tag.value;
                const isLongTag = value.length > 200;
                const tagElem = (React.createElement(Tag, { style: tagStyle, key: tag.property, closable: true, onClose: () => handleCloseTag(tag) },
                    React.createElement("b", null,
                        " ",
                        tag.title,
                        " "),
                    " ",
                    tag.relation && React.createElement("i", null,
                        " ",
                        tag.relation.title,
                        " "),
                    ' ',
                    isLongTag ? `${value.slice(0, 20)}...` : value));
                return isLongTag ? (React.createElement(Tooltip, { title: tag.toString(), key: tag.toString() }, tagElem)) : (tagElem);
            }),
        React.createElement(Input, { ref: inputRef, type: 'text', size: 'small', style: { width: 78, margin: '8px' }, value: inputValue, onChange: handleInputChange, onBlur: handleInputConfirm, onPressEnter: handleInputConfirm }),
        React.createElement(Tag, { onClick: showModal, style: { background: '#fff', borderStyle: 'dashed' } },
            React.createElement(PlusOutlined, null),
            " ",
            locale.add),
        modalVisible && (React.createElement(CreateFilterModal, { loading: loading, handleCancel: () => hideModal(), handleOk: (filter) => {
                if (filter) {
                    handleAddFilter(filter);
                }
                hideModal();
            }, handleAdd: (filter) => {
                if (filter) {
                    handleAddFilter(filter);
                }
            }, tags: tagsArray })),
        tagsArray &&
            (!isExpanded ? (React.createElement(Button, { icon: React.createElement(DownOutlined, null), size: 'small', onClick: () => {
                    setIsExpanded(true);
                } })) : (React.createElement(Button, { icon: React.createElement(UpOutlined, null), size: 'small', onClick: () => {
                    setIsExpanded(false);
                } })))));
});
const antdQueryTester = rankWith(2, uiTypeIs(QueryIRI));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
function getRandomInt(max) {
    return Math.floor(Math.random() * max);
}
const colors = ['#FED6BC', '#FFFADD', '#DEF7FE', '#E7ECFF', '#C3FBD8', '#FDEED9', '#F6FFF8', '#B5F2EA', '#C6D8FF'];
const PanelWithText = (props) => {
    const style = props?.viewKindElement.options?.style;
    const value = props?.viewKindElement.options?.value;
    return (React.createElement("div", { style: { ...style, backgroundColor: colors[getRandomInt(colors.length)] } },
        React.createElement("pre", { style: { verticalAlign: 'center' } }, value)));
};
const panelWithTextControlTester = rankWith(3, uiTypeIs('aldkg:TextPanel'));

/********************************************************************************
 * Copyright (c) 2024 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const { RangePicker } = DatePicker;
const { Title } = Typography;
const rangeProps = ['amountValueMoving30', 'revenueMoving30', 'price', 'commentsCount', 'firstParsedAt', 'parsedAt'];
function isInteger(value) {
    return parseInt(value, 10).toString() === value;
}
function dispatchControl(p) {
    if (p.type === 'integer')
        return (React.createElement(Form$1.Item, { key: `${p.name}`, label: p.title, tooltip: p.description },
            React.createElement(Form$1.Item, { key: `${p.name}-Min`, name: `${p.name}-Min`, rules: [
                    {
                        type: 'integer',
                        message: 'Please input integer!',
                        validator: (rule, value) => new Promise((resolve, reject) => value === undefined || value === '' || (isInteger(value) ? resolve(value) : reject(rule.message))),
                    },
                ], style: { display: 'inline-block', width: 'calc(50% - 10px)' } },
                React.createElement(Input, { style: { width: '100%' } })),
            React.createElement("span", { style: { display: 'inline-block', width: '20px', lineHeight: '32px', textAlign: 'center' } }, "-"),
            React.createElement(Form$1.Item, { key: `${p.name}-Max`, name: `${p.name}-Max`, rules: [
                    {
                        type: 'integer',
                        message: 'Please input integer!',
                        validator: (rule, value) => new Promise((resolve, reject) => value === undefined || value === '' || (isInteger(value) ? resolve(value) : reject(rule.message))),
                    },
                ], style: { display: 'inline-block', width: 'calc(50% - 10px)' } },
                React.createElement(Input, { style: { width: '100%' } }))));
    else if (p.type === 'string') {
        if (p.format === 'date-time') {
            return (React.createElement(Form$1.Item, { key: `${p.name}`, label: p.title, tooltip: p.description },
                React.createElement(RangePicker, { id: {
                        start: `${p.name}-Min`,
                        end: `${p.name}-Max`,
                    }, style: { width: '100%' } })));
        }
        return (React.createElement(Form$1.Item, { key: p.name, name: p.name, label: p.title, tooltip: p.description },
            React.createElement(Input, null)));
    }
}
const QueryFormIRI = 'aldkg:QueryForm';
const QueryForm = observer((props) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(QueryFormIRI);
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const [form] = Form$1.useForm();
    const onFinish = (values) => {
        console.log('QueryForm Success:', values);
        if (!viewKindElement.options)
            return;
        const conn = viewKindElement.options.connections;
        if (!Array.isArray(conn) || conn.length == 0)
            return;
        const cond = conn[0]?.toObj;
        if (!cond)
            return;
        const coll = store.rep.getColl(collIri);
        console.log('QueryForm - coll', coll ? coll['@id'] : undefined);
        if (!coll)
            return;
        console.log('QueryForm - entConstrs', coll.collConstr?.entConstrs);
        if (!coll.collConstr)
            return;
        console.log('QueryForm - cond', coll.collConstr.entConstrs[0].conditions);
        const cond2 = getSnapshot(coll.collConstr.entConstrs[0].conditions);
        const newCond = { '@id': cond2['@id'] };
        if (cond2['@type'])
            newCond['@type'] = cond2['@type'];
        //name prop
        if (values.name) {
            if (values.name === '')
                values.name = undefined;
            else
                newCond.name = { relation: 'contains', value: [values.name] };
        }
        //other range props
        rangeProps.forEach((prop) => {
            const minPropName = prop + 'Min';
            const maxPropName = prop + 'Max';
            if (values[minPropName] && values[maxPropName])
                newCond[prop] = {
                    relation: 'between-incl-both',
                    value: [values[minPropName], values[maxPropName]],
                };
            else if (values[minPropName])
                newCond[prop] = {
                    relation: 'after',
                    value: [values[minPropName]],
                };
            else if (values[maxPropName])
                newCond[prop] = {
                    relation: 'before',
                    value: [values[maxPropName]],
                };
        });
        console.log('QueryForm - newCond', newCond);
        applySnapshot(coll.collConstr.entConstrs[0].conditions, newCond);
    };
    const hiddenProperties = viewKindElement.options?.hiddenProperties || [];
    const coll = store.rep.getColl(collIri);
    const properties = coll?.collConstr?.entConstrs
        ? Object.entries(coll.collConstr.entConstrs[0].schema?.propertiesJs || {})
            .filter((e) => !hiddenProperties.includes(e[0]))
            .map((e) => ({
            name: e[0],
            ...e[1],
        }))
            .sort((e1, e2) => {
            if (e1.order === e2.order)
                return 0;
            if (e1.order === undefined)
                return 1;
            if (e2.order === undefined)
                return -1;
            if (e1.order < e2.order)
                return -1;
            return 1;
        })
        : [];
    //console.log('QueryForm', { properties });
    return (React.createElement("div", { style: viewKindElement.options?.blockStyle || { margin: '10px' } },
        viewKindElement.options.title !== undefined && React.createElement(Title, { level: 4 }, viewKindElement.options.title),
        React.createElement(Form$1, { style: { ...(viewKindElement.options?.formStyle || {}) }, layout: viewKindElement.options?.layout || 'vertical', size: viewKindElement.options?.size || 'small', form: form, initialValues: { ...(viewKindElement.options?.initialValues || {}) }, onFinish: onFinish },
            properties.map((p) => dispatchControl(p)),
            React.createElement(Form$1.Item, { style: { marginTop: '16px' } },
                React.createElement(Button, { type: 'primary', htmlType: 'submit' }, locale.searchBtnTitle)))));
});
const antdQueryFormTester = rankWith(2, uiTypeIs(QueryFormIRI));

const antdControlRenderers = [
    { tester: antdBooleanControlTester, renderer: AntdBooleanControlWithStore },
    { tester: antdButtonControlTester, renderer: AntdButtonControlWithStore },
    { tester: antdDateControlTester, renderer: AntdDateControlWithStore },
    { tester: antdEnumControlTester, renderer: AntdEnumControlWithStore },
    { tester: antdNumberControlTester, renderer: AntdNumberControlWithStore },
    {
        tester: antdRadioGroupControlTester,
        renderer: AntdRadioGroupControlWithStore,
    },
    {
        tester: antdImageControlTester,
        renderer: AntdImageControlWithStore,
    },
    { tester: antdSliderControlTester, renderer: AntdSliderControlWithStore },
    { tester: antdTextControlTester, renderer: AntdTextControlWithStore },
    { tester: panelWithTextControlTester, renderer: PanelWithText },
    { tester: tinyMCEControlTester, renderer: TinyMCEControlWithStore },
    { tester: antdQueryTester, renderer: AntQueryWithStore },
    { tester: antdQueryFormTester, renderer: QueryForm },
];

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const SaveReqDialogIRI = 'aldkg:SaveReqDialog';
const SaveReqDialog = ({ visible = true, onOk = () => { }, onCancel = () => { }, }) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(SaveReqDialogIRI);
    return (React.createElement(Modal, { title: locale.unsavedTitle, visible: visible, onOk: onOk, onCancel: onCancel, cancelText: locale.btnCancel, okText: locale.btnOK },
        React.createElement("p", null, locale.unsavedDescr)));
};
const SaveReqDialogWithStore = withStoreToSaveDialogProps(SaveReqDialog);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const divStyle$3 = {
    padding: '5px',
};
const TableRenderer = React.memo((props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, schema, enabled, child, onSelect, editing } = props;
    const [selected, setSelected] = useState(child[0]);
    const [cacheSelect, setCacheSelect] = useState();
    const [dataSource, setDataSource] = useState(child);
    const [visible, setVisible] = useState(false);
    function columns() {
        return [
            {
                dataIndex: 'title',
                key: 'title',
                title: schema.title,
                render: (text) => React.createElement("a", { href: '#!' }, text),
            },
        ];
    }
    useEffect(() => autorun(() => {
        const newData = [...child];
        setDataSource(newData);
    }), [child]);
    return (React.createElement(React.Fragment, null,
        React.createElement(SplitPane, { split: 'vertical', style: divStyle$3, minSize: 100 },
            React.createElement("div", { style: divStyle$3 },
                React.createElement(Table, { className: 'gx-table-responsive', size: 'small', pagination: false, rowKey: (record) => record.uri, columns: columns(), onRow: (record) => {
                        return {
                            onClick: () => {
                                if (!editing) {
                                    onSelect(record);
                                    setSelected(record);
                                }
                                else {
                                    setVisible(true);
                                    setCacheSelect(record);
                                }
                            },
                        };
                    }, dataSource: dataSource.map((e) => {
                        const newData = { ...e };
                        if (newData.children && newData.children.length === 0) {
                            delete newData.children;
                        }
                        return newData;
                    }) })),
            selected.viewKindElement || viewKindElement.elements ? (React.createElement("div", { style: divStyle$3 },
                React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: selected.viewKindElement || viewKindElement.elements[0], viewDescr: viewDescr, enabled: enabled }))) : null),
        React.createElement(SaveReqDialog, { visible: visible, onOk: () => {
                setVisible(false);
                onSelect(cacheSelect);
                setSelected(cacheSelect);
            }, 
            //schemaUri={viewKindElement.resultsScope}
            onCancel: () => setVisible(false) })));
}, (prevProps, nextProps) => isEqual(prevProps, nextProps));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const NodeRenderer = React.memo(({ title, onContextMenu, onSort, edited, onRename, children, ...otherProps }) => {
    const [edit, setEdit] = useState(edited);
    const [sort, setSort] = useState('noSort');
    const [currentTitle, setCurrentTitle] = useState(title);
    const ref = React.createRef();
    useEffect(() => {
        if (ref.current !== null) {
            ref.current.focus();
        }
    }, [edit, ref]);
    const onClick = (ev) => {
        const newSort = sortSwap[sort];
        ev.stopPropagation();
        setSort(newSort);
        onSort(otherProps['@id'], newSort);
    };
    return (React.createElement("div", { onContextMenu: (ev) => {
            onContextMenu({ event: ev, node: { onRename: () => setEdit(true), title, otherProps: { ...otherProps } } });
        } },
        otherProps['@type'] === 'nav:folder' ? React.createElement(FolderOutlined, { style: { padding: '0 5px 0 0' } }) : null,
        edit ? (React.createElement("span", null,
            React.createElement(Input, { style: { display: 'inline-block', width: '80%' }, ref: ref, size: 'small', defaultValue: currentTitle, onBlur: (ev) => {
                    if (ev.target.value !== title) {
                        setCurrentTitle(ev.target.value);
                        onRename(ev.target.value, otherProps['@id']);
                    }
                    setEdit(false);
                } }))) : (React.createElement("span", { onDoubleClick: () => setEdit(true) }, otherProps.editedTitle && otherProps.editedTitle !== title ? otherProps.editedTitle : currentTitle)),
        React.createElement("div", { className: sort === 'noSort' ? 'antd-sort-icon' : '', style: { display: 'inline', paddingLeft: '5px' } }, sort === 'DESC' ? React.createElement(SortDescendingOutlined, { onClick: onClick }) : React.createElement(SortAscendingOutlined, { onClick: onClick }))));
}, (prevProps, nextProps) => {
    return prevProps['@id'] === nextProps['@id'] && prevProps.editedTitle === nextProps.editedTitle;
});
const sortSwap = {
    ASC: 'DESC',
    DESC: 'noSort',
    noSort: 'ASC',
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ContextMenu$1 = ({ record, selection, visible, x, y, actionsMap, onClick }) => {
    return ((actionsMap && visible && (React.createElement("ul", { className: 'popup', style: { left: `${x}px`, top: `${y}px`, position: 'fixed' } }, actionsMap.map(({ title, action }) => (React.createElement("li", { key: title, onClick: () => {
            action(selection);
            onClick();
        } }, title)))))) ||
        null);
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const divStyle$2 = {
    height: '100%',
    padding: '5px',
    overflow: 'auto',
};
const TreeRenderer = (props) => {
    const { enabled, child, onSelect, viewKindElement, dataSource, viewKind, editing, onDnD, onCreateFolder, onDeleteFolder, onRename, actionsMap, } = props;
    const [visible, setVisible] = useState(false);
    const [selected, setSelected] = useState(child[0]);
    const [treeData, setTreeData] = useState(child);
    const [selectedKey, setSelectedKey] = useState(child.length !== 0 ? [child[0].key] : undefined);
    const [popupCoords, setPopupCoords] = useState({ x: 0, y: 0 });
    const [popupVisible, setPopupVisible] = useState(false);
    const [popupRecord, setPopupRecord] = useState();
    const [expandedKeys, setExpandedKeys] = useState([]);
    const [searchValue, setSearchValue] = useState('');
    const [autoExpandParent, setAutoExpandParent] = useState(true);
    const [beforeSearchExpand, setBeforeSearchExpand] = useState([]);
    useEffect(() => {
        setTreeData(child);
    }, [child]);
    useEffect(() => {
        onSelect(selected);
    }, [selected, onSelect]);
    /*const generateList = (data: any) => {
      for (let i = 0; i < data.length; i++) {
        const node = data[i];
        const { key } = node;
        dataList.push({ key, title: node.title });
        if (node.children) {
          generateList(node.children);
        }
      }
    };
    generateList(child);*/
    const getParentKey = (key, tree) => {
        let parentKey;
        for (let i = 0; i < tree.length; i++) {
            const node = tree[i];
            if (node.children) {
                if (node.children.some((item) => item.key === key)) {
                    parentKey = node.key;
                }
                else if (getParentKey(key, node.children)) {
                    parentKey = getParentKey(key, node.children);
                }
            }
        }
        return parentKey;
    };
    const onChange = (keys, ev) => {
        if (!editing) {
            const newData = ev.node;
            setSelected(newData);
            setSearchValue('');
            setBeforeSearchExpand(expandedKeys);
            setSelectedKey(keys);
        }
        else {
            setVisible(true);
        }
    };
    const onContextMenu = ({ event, node }) => {
        event.preventDefault();
        //event.target.trigger('dblclick');
        if (!popupVisible) {
            document.addEventListener(`click`, function onClickOutside() {
                setPopupVisible(false);
                document.removeEventListener(`click`, onClickOutside);
            });
        }
        setPopupVisible(true);
        setPopupRecord(node);
        setPopupCoords({ x: event.clientX, y: event.clientY });
    };
    const loop = (data, key, callback, node) => {
        for (let i = 0; i < data.length; i++) {
            if (data[i].key === key) {
                return callback(data[i], i, data, node);
            }
            if (data[i].children) {
                loop(data[i].children, key, callback, data[i]);
            }
        }
    };
    const onExpand = (expandedKeys) => {
        setExpandedKeys(expandedKeys);
        setBeforeSearchExpand(expandedKeys);
        setAutoExpandParent(false);
    };
    const titlePropName = viewKindElement?.options?.treeNodeTitleKey || 'title';
    const searchEdit = (data) => data.map((item) => {
        const title = item[titlePropName];
        const index = title.indexOf(searchValue);
        const beforeStr = title.substr(0, index);
        const afterStr = title.substr(index + searchValue.length);
        const foundTitle = index > -1 && searchValue !== '' ? (React.createElement("span", null,
            beforeStr,
            React.createElement("span", { className: 'site-tree-search-value' }, searchValue),
            afterStr)) : (title);
        if (item.children) {
            return { ...item, ...{ editedTitle: foundTitle, children: searchEdit(item.children) } };
        }
        return {
            ...item,
            ...{ editedTitle: foundTitle },
        };
    });
    const onSearch = (e) => {
        setSearchValue(e.target.value);
        const { value } = e.target;
        if (value !== '') {
            const expandedKeys = dataSource
                .map((item) => {
                if (item[titlePropName].indexOf(value) > -1) {
                    return getParentKey(item.key, treeData);
                }
                return null;
            })
                .filter((item, i, self) => item && self.indexOf(item) === i);
            setExpandedKeys(expandedKeys);
        }
        else {
            setExpandedKeys(beforeSearchExpand);
        }
    };
    const onDrop = (info) => {
        const dropKey = info.node.props.eventKey;
        const dragKey = info.dragNode.props.eventKey;
        const dropPos = info.node.props.pos.split('-');
        const dropPosition = info.dropPosition - Number(dropPos[dropPos.length - 1]);
        const data = [...treeData];
        let parent = info.node['@id'];
        let dragObj;
        loop(data, dragKey, (item, index, arr) => {
            arr.splice(index, 1);
            dragObj = item;
        });
        if (!info.dropToGap) {
            loop(data, dropKey, (item) => {
                item.children = item.children || [];
                item.children.push(dragObj);
            });
        }
        else if ((info.node.props.children || []).length > 0 && info.node.props.expanded && dropPosition === 1) {
            loop(data, dropKey, (item) => {
                item.children = item.children || [];
                item.children.unshift(dragObj);
            });
        }
        else {
            let ar;
            let i;
            loop(data, dropKey, (item, index, arr, node) => {
                ar = arr;
                i = index;
                parent = node['@id'];
            });
            if (dropPosition === -1) {
                ar.splice(i, 0, dragObj);
            }
            else {
                ar.splice(i + 1, 0, dragObj);
            }
        }
        onDnD({ childId: info.dragNode['@id'], parentId: parent });
        setTreeData(data);
    };
    const onSortChildren = (key, order) => {
        const data = [...treeData];
        loop(data, key, (item) => {
            item.children.reverse();
            if (order === 'noSort') {
                loop(child, key, (i) => {
                    item.children = i.children;
                });
            }
            else if (order === 'ASC') {
                item.children = sortBy(item.children, [
                    function (o) {
                        return o[titlePropName];
                    },
                ]);
            }
            else {
                item.children = sortBy(item.children, [
                    function (o) {
                        return o[titlePropName];
                    },
                ]).reverse();
            }
        });
        setTreeData(data);
    };
    return (React.createElement("div", { style: divStyle$2 },
        React.createElement(Input.Search, { size: 'small', placeholder: 'Search', style: { marginBottom: 8 }, value: searchValue, onChange: onSearch }),
        React.createElement(Tree, { selectedKeys: selectedKey, draggable: { icon: false }, onExpand: onExpand, expandedKeys: expandedKeys, autoExpandParent: autoExpandParent, titleRender: (props) => (React.createElement(NodeRenderer, { onRename: onRename, onSort: onSortChildren, onContextMenu: onContextMenu, ...props })), height: 600, onDrop: onDrop, onSelect: onChange, treeData: searchEdit(treeData) }),
        React.createElement(SaveReqDialogWithStore, { visible: visible, onOk: () => setVisible(false), schemaUri: viewKindElement.resultsScope, onCancel: () => setVisible(false) }),
        React.createElement(ContextMenu$1, { x: popupCoords.x, y: popupCoords.y, record: popupRecord, selection: popupRecord?.otherProps
                ? [
                    {
                        ...popupRecord.otherProps,
                        parentKey: viewKindElement?.options.treeNodeParentKey || 'parent',
                        titlePropName,
                    },
                ]
                : [], visible: popupVisible, actionsMap: actionsMap, onClick: () => null })));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const GridRenderer = (props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, child, schema } = props;
    const grid = viewKindElement?.options?.grid || { gutter: 16, column: 4 };
    const template = viewKindElement?.options?.elementTemplate || null;
    const style = viewKindElement?.options?.style;
    const createCell = (data, id) => template ? (template.map((e, idx) => (React.createElement(DispatchCell, { id: String(id) + String(idx), key: String(id) + String(idx), viewKind: viewKind, viewKindElement: e, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, data: data, rowData: data })))) : (React.createElement("span", { key: id }, data['@id']));
    return (React.createElement(List, { grid: grid, style: style, dataSource: child, renderItem: (item, idx) => React.createElement(List.Item, null, createCell(item, idx)) }));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const Card = ({ itemId, onClick, children, style }) => {
    return (React.createElement("div", { onClick: () => onClick(), style: {
            display: 'inline-block',
            margin: '0 10px',
            width: '260px',
            userSelect: 'none',
            ...style,
        }, tabIndex: 0, className: 'card' }, children));
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
function Arrow({ children, disabled, onClick, }) {
    return (React.createElement("button", { disabled: disabled, onClick: onClick, className: 'arrow' }, children));
}
function LeftArrow() {
    const { isFirstItemVisible, scrollPrev, visibleItemsWithoutSeparators } = React.useContext(VisibilityContext);
    const [disabled, setDisabled] = React.useState(!visibleItemsWithoutSeparators.length && isFirstItemVisible);
    React.useEffect(() => {
        if (visibleItemsWithoutSeparators.length) {
            setDisabled(isFirstItemVisible);
        }
    }, [isFirstItemVisible, visibleItemsWithoutSeparators]);
    return (React.createElement(Arrow, { disabled: disabled, onClick: () => scrollPrev() }, '<'));
}
function RightArrow() {
    const { isLastItemVisible, scrollNext, visibleItemsWithoutSeparators } = React.useContext(VisibilityContext);
    const [disabled, setDisabled] = React.useState(!visibleItemsWithoutSeparators.length && isLastItemVisible);
    React.useEffect(() => {
        if (visibleItemsWithoutSeparators.length) {
            setDisabled(isLastItemVisible);
        }
    }, [isLastItemVisible, visibleItemsWithoutSeparators]);
    return (React.createElement(Arrow, { disabled: disabled, onClick: () => scrollNext() }, '>'));
}

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
function useDrag() {
    const [clicked, setClicked] = React.useState(false);
    const [dragging, setDragging] = React.useState(false);
    const [position, setPosition] = React.useState(0);
    const [diff, setDiff] = React.useState(0);
    const dragStart = React.useCallback((ev) => {
        setPosition(ev.clientX);
        setDiff(0);
        setClicked(true);
    }, []);
    const dragStop = React.useCallback(() => window.requestAnimationFrame(() => {
        setDragging(false);
        setClicked(false);
    }), []);
    const dragMove = React.useCallback((ev, cb) => {
        const newDiff = position - ev.clientX;
        const movedEnough = Math.abs(newDiff) > 5;
        if (clicked && movedEnough) {
            setDragging(true);
        }
        if (dragging && movedEnough) {
            setPosition(ev.clientX);
            setDiff(newDiff);
            cb(newDiff);
        }
    }, [clicked, dragging, position]);
    return {
        dragStart,
        dragStop,
        dragMove,
        diff,
        dragging,
        position,
        setDragging,
        setDiff,
        setPosition,
    };
}

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const HorizontalScrollRenderer = (props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, child, schema, getData, onSelect } = props;
    const [dataSource, setDataSource] = useState(child);
    const [selected, setSelected] = useState('');
    const template = viewKindElement?.options?.elementTemplate || null;
    const { dragStart, dragStop, dragMove, dragging } = useDrag();
    const handleDrag = ({ scrollContainer }) => (ev) => dragMove(ev, (newPos) => {
        if (scrollContainer.current) {
            const currentScroll = scrollContainer.current.scrollLeft;
            scrollContainer.current.scrollLeft = currentScroll + newPos;
        }
    });
    const handleItemClick = (itemId) => () => {
        if (dragging) {
            return;
        }
        if (selected !== itemId) {
            setSelected(itemId);
            onSelect(itemId);
        }
    };
    const createCell = (data, id) => template ? (template.map((e, idx) => (React.createElement(DispatchCell, { id: String(id) + String(idx), key: String(id) + String(idx), viewKind: viewKind, viewKindElement: e, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, data: data, rowData: data })))) : (React.createElement("span", { key: id }, data['@id']));
    const onUpdate = ({ isLastItemVisible }) => {
        if (isLastItemVisible) {
            //const newDataSource = dataSource.concat(getData());
            console.log('push new items');
            //setDataSource(newDataSource);
        }
    };
    useEffect(() => {
        setDataSource(child);
    }, [child]);
    return (React.createElement("div", { onMouseLeave: dragStop },
        React.createElement(ScrollMenu, { LeftArrow: LeftArrow, RightArrow: RightArrow, onWheel: onWheel, onMouseDown: () => dragStart, onMouseUp: () => dragStop, onMouseMove: handleDrag, onUpdate: onUpdate }, dataSource.map((data, idx) => (React.createElement(Card, { key: data['@id'], itemId: data['@id'], onClick: handleItemClick(data['@id']), style: viewKindElement.options.templateStyle }, createCell(data, idx)))))));
};
function onWheel(apiObj, ev) {
    const isTouchpad = Math.abs(ev.deltaX) !== 0 || Math.abs(ev.deltaY) < 15;
    if (isTouchpad) {
        ev.stopPropagation();
        return;
    }
    if (ev.deltaY < 0) {
        apiObj.scrollNext();
    }
    else if (ev.deltaY > 0) {
        apiObj.scrollPrev();
    }
}

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const renderType = {
    tree: TreeRenderer,
    table: TableRenderer,
    grid: GridRenderer,
    horizontalScroll: HorizontalScrollRenderer,
};
const AntdDataLayout = React.memo((props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, handleChange = () => { }, dataSource, schema, editing, getData, onDnD, onCreateFolder, onDeleteFolder, onRename, actionsMap, } = props;
    const data = treeify(dataSource, '@id', viewKindElement?.options.treeNodeParentKey || 'parent', 'children', strcmp);
    const onSelect = (selected) => {
        handleChange(selected);
    };
    const Render = renderType[viewKindElement?.options.renderType];
    return (React.createElement(Render, { uri: props.uri, enabled: enabled, onSelect: onSelect, child: data, editing: editing, onDnD: onDnD, actionsMap: actionsMap, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, onCreateFolder: onCreateFolder, onDeleteFolder: onDeleteFolder, onRename: onRename, schema: schema, getData: getData, dataSource: dataSource }));
});
const antdDataControlTester = rankWith(2, uiTypeIs('aldkg:DataControl'));
const AntdDataControlWithStore = withStoreToDataControlProps(AntdDataLayout);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const renderLayoutElements = ({ viewKind, viewKindElement, viewDescr, enabled, Render, readOnly, }) => {
    const elements = viewKindElement.elements;
    //const id = viewKind['@id'];
    //const sort = id ? viewKind.properties && viewKind.properties[id] && viewKind.properties[id].order : undefined;
    if (!elements || elements.length === 0)
        return React.createElement(React.Fragment, null);
    return elements.map((el, idx) => {
        el = { ...el, options: { readOnly, ...el.options } };
        return (React.createElement(Render, { key: idx, idx: idx, viewKind: viewKind, viewKindElement: el, viewDescr: viewDescr, enabled: enabled }));
    });
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const PanelLayout = ({ viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, visible, form, }) => {
    const options = viewKindElement.options || {};
    const style = options.style;
    const layoutStyle = {};
    const childLayoutStyle = {};
    if (options.flow === 'horizontal') {
        layoutStyle.whiteSpace = 'nowrap';
    }
    if (options.flow === 'horizontal' || options.flow === 'wrap') {
        childLayoutStyle.display = 'inline-block';
    }
    const Render = ({ idx, viewKind, viewKindElement, viewDescr, enabled }) => {
        const style = viewKindElement.options?.style;
        const newViewKindElement = { ...viewKindElement };
        if (newViewKindElement.options) {
            newViewKindElement.options.style = { ...style, ...childLayoutStyle };
        }
        return (React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: newViewKindElement, viewDescr: viewDescr, enabled: enabled, form: form }));
    };
    return (React.createElement("div", { style: { ...style, ...layoutStyle } }, renderLayoutElements({ viewKind, viewKindElement, viewDescr, enabled, Render })));
};
const panelLayoutTester = rankWith(2, uiTypeIs('aldkg:PanelLayout'));
const PanelLayoutWithStore = withLayoutProps(PanelLayout);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ButtonGroup$1 = Button.Group;
const divStyle$1 = {
    padding: '5px',
};
const FormLayoutIRI = 'aldkg:FormLayout';
const LogicalButton = observer(({ form, onCancel, onSave }) => {
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(FormLayoutIRI);
    //const onValidate = Symbol.for('onValidate');
    const activeSave = false;
    //if (store.onSaveData[form] && store.onSaveData[form][onValidate]) {
    //  activeSave = store.onSaveData[form][onValidate].length !== 0;
    //}
    return (React.createElement(Row$1, { style: divStyle$1, justify: 'end' },
        React.createElement(ButtonGroup$1, null,
            React.createElement(Button, { key: 'cancel', disabled: !store.editingData.get(form), size: 'small', onClick: (ev) => {
                    ev.stopPropagation();
                    onCancel();
                } }, locale.btnCancel),
            React.createElement(Button, { key: 'ok', size: 'small', disabled: !store.editingData.get(form) || activeSave, type: 'primary', onClick: async () => {
                    await onSave();
                    onCancel();
                } }, locale.btnSave))));
});
const AntdFormLayout = ({ viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, title, visible, id, validation, editable, onSave, onCancel, onEdit, editing, }) => {
    const { readOnly, style } = viewKindElement.options;
    return (React.createElement("div", { style: { overflow: 'auto', position: 'relative', ...style }, onClick: () => !readOnly && onEdit() },
        title && React.createElement("span", { style: { padding: '7px', fontSize: '2em' } }, title),
        readOnly ? null : React.createElement(LogicalButton, { form: id, onSave: onSave, onCancel: onCancel }),
        React.createElement(Form$1, { labelAlign: 'left' },
            React.createElement(PanelLayoutWithStore, { id: `${id}Layout`, viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: {}, enabled: enabled, form: id, readOnly: readOnly }))));
};
const antdFormLayoutTester = rankWith(2, uiTypeIs(FormLayoutIRI));
const AntdFormLayoutWithStore = withStoreToFormProps(AntdFormLayout);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdHorizontalLayoutRenderer = ({ viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, visible, form, }) => {
    const Render = ({ idx, viewKind, viewKindElement, viewDescr, enabled }) => {
        viewKindElement.options || {};
        const width = viewKindElement.options?.width;
        const height = viewKindElement.options?.height;
        const style = {};
        if (width === 'all-empty-space') {
            style.flexGrow = 1;
        }
        else {
            style.width = width;
        }
        if (height === 'all-empty-space') {
            style.alignSelf = 'stretch';
        }
        else {
            style.height = height;
        }
        return (React.createElement("div", { key: idx, style: style },
            React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, enabled: enabled, form: form })));
    };
    viewKindElement.options?.justify || 'space-between';
    viewKindElement.options?.align || 'middle';
    if (viewKindElement.options && viewKindElement.options.width === 'all-empty-space')
        ;
    return renderLayoutElements({ viewKind, viewKindElement, viewDescr, enabled, Render });
};
const antdHorizontalLayoutTester = rankWith(2, uiTypeIs('aldkg:HorizontalLayout'));
const AntdHorizontalLayoutWithStore = withLayoutProps(AntdHorizontalLayoutRenderer);

const convertSizeOptionToStyle = ({ width, height, }) => {
    const style = {};
    if (height === 'all-empty-space') {
        style.flexGrow = 1;
    }
    else {
        style.height = height;
    }
    if (width === 'all-empty-space') {
        style.width = '100%';
    }
    else {
        style.width = width;
    }
    return style;
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const MstVkeVerticalLayout = types.compose('MstVerticalLayout', MstViewKindElement, types.model({
    '@type': types.literal('aldkg:VerticalLayout'),
}));
const AntdVerticalLayoutRenderer = ({ viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, visible, form, readOnly, }) => {
    const style = viewKindElement.options?.style;
    const Render = ({ idx, viewKind, viewKindElement, viewDescr, enabled }) => {
        const height = viewKindElement.options?.height;
        const width = viewKindElement.options?.width;
        const grow = viewKindElement.options?.grow;
        const rowStyle = convertSizeOptionToStyle({ height, width });
        return (React.createElement(Row$1, { style: {
                position: 'relative',
                ...rowStyle,
                flexGrow: grow,
            } },
            React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, enabled: enabled, form: form })));
    };
    return (React.createElement("div", { style: { display: 'flex', flexDirection: 'column', ...style } }, renderLayoutElements({ viewKind, viewKindElement, viewDescr, enabled, Render, readOnly })));
};
const antdVerticalLayoutTester = rankWith(2, uiTypeIs('aldkg:VerticalLayout'));
const AntdVerticalLayoutWithStore = withLayoutProps(AntdVerticalLayoutRenderer);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const SplitPaneLayoutRenderer = ({ viewKindElement, viewKind, viewDescr, viewDescrElement, enabled, visible, }) => {
    const elements = viewKindElement.elements;
    const options = viewKindElement.options || {};
    const panes = useMemo(() => elements
        ? elements.map((el, idx) => (React.createElement("div", { key: idx, style: { width: '100%', height: '100%', position: 'relative' } },
            React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: el, viewDescr: viewDescr, enabled: enabled }))))
        : [], [viewKind, elements, viewDescr, enabled]);
    return (React.createElement("div", { style: { position: 'relative', height: '100%', ...options.style } },
        React.createElement(SplitPane$1, { split: options.split || 'vertical', initialSizes: options.initialSizes, minSizes: options.minSizes, collapsedSizes: options.collapsedSizes, collapse: options.collapseDirection && {
                collapseDirection: options.collapseDirection,
            }, resizerOptions: options.resizerOptions }, panes)));
};
const splitPaneLayoutTester = rankWith(2, uiTypeIs('aldkg:SplitPaneLayout'));
const SplitPaneLayoutWithStore = withLayoutProps(SplitPaneLayoutRenderer);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TabsLayout = (props) => {
    const { viewKind, viewKindElement, viewDescr, viewDescrElement, enabled, onSelect = () => { } } = props;
    const elements = viewKindElement.elements;
    const viewTabs = elements
        ? elements.map((e, index) => {
            const title = e.options && e.options.title;
            return (React.createElement(Tabs.TabPane, { tab: title, key: '' + index },
                React.createElement(FormsDispatch, { viewKind: viewKind, viewKindElement: e, viewDescr: viewDescr, enabled: enabled })));
        })
        : [];
    return (React.createElement(React.Fragment, null,
        React.createElement(Tabs, { type: 'card', size: 'small', onChange: onSelect }, viewTabs)));
};
const antdTabsLayoutTester = rankWith(2, uiTypeIs('aldkg:TabsLayout'));
const AntdTabsLayoutWithStore = withLayoutProps(TabsLayout);

const antdLayoutRenderers = [
    {
        tester: antdHorizontalLayoutTester,
        renderer: AntdHorizontalLayoutWithStore,
    },
    { tester: antdVerticalLayoutTester, renderer: AntdVerticalLayoutWithStore, mstVkeType: MstVkeVerticalLayout },
    {
        tester: antdFormLayoutTester,
        renderer: AntdFormLayoutWithStore,
    },
    {
        tester: splitPaneLayoutTester,
        renderer: SplitPaneLayoutWithStore,
    },
    {
        tester: antdTabsLayoutTester,
        renderer: AntdTabsLayoutWithStore,
    },
    {
        tester: panelLayoutTester,
        renderer: PanelLayoutWithStore,
    },
];

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ButtonGroup = Button.Group;
const divStyle = {
    padding: '5px',
};
const SaveControl = (props) => {
    const { editing, setEditing, saveData } = props;
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(FormLayoutIRI);
    const onCancel = () => {
        setEditing(false);
    };
    return (React.createElement("div", null,
        React.createElement(Row$1, { style: divStyle, justify: 'end' },
            React.createElement(ButtonGroup, null,
                React.createElement(Button, { key: 'cancel', disabled: !editing, onClick: () => onCancel() }, locale.btnCancel),
                React.createElement(Button, { key: 'ok', disabled: !editing, type: 'primary', onClick: saveData }, locale.btnSave)))));
};
const saveControlTester = rankWith(1, uiTypeIs('aldkg:SaveControl'));
const AntdSaveControlWithStore = withStoreToSaveButtonProps(SaveControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdSelectControl = (props) => {
    const { handleChange, dataSource } = props;
    //const options = merge({}, viewKindElement.options);
    const data = [];
    const dataObject = {};
    if (Array.isArray(dataSource)) {
        dataSource.forEach((e) => {
            data.push({
                value: e['@id'],
                label: e.title || e['@id'],
            });
            dataObject[e['@id']] = e;
        });
    }
    else {
        for (const key in dataSource) {
            data.push({
                value: dataSource[key]['@id'],
                label: dataSource[key].title || dataSource[key]['@id'],
            });
            dataObject[key] = dataSource[key];
        }
    }
    const onSelect = (selected) => {
        handleChange(dataObject[selected]);
    };
    useEffect(() => {
        if (data[0])
            handleChange(dataObject[data[0].value]);
    }, [data, dataObject, handleChange]);
    return (React.createElement("div", null,
        React.createElement("div", { style: { marginLeft: 24, display: 'flex', alignItems: 'center' } },
            React.createElement("div", { style: {
                    display: 'inline',
                    marginRight: 10,
                    fontSize: 20,
                } },
                props?.options?.label || '',
                ":",
                ' '),
            React.createElement(Select, { defaultValue: data[0]?.value, options: data, onSelect: onSelect, style: { width: 400 } })),
        React.createElement(Divider, { style: { marginTop: 7, marginBottom: 24 } })));
};
const antdSelectControlTester = rankWith(2, uiTypeIs('aldkg:SelectControl'));
const AntdSelectControlWithStore = withStoreToSelectControlProps(AntdSelectControl);

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const MstVkeTabControl = types.compose('MstVkeTabControl', MstViewKindElement, types.model({
    '@type': types.literal('aldkg:TabControl'),
    tabs: types.maybe(types.array(MstJsObject)),
}));
const AntdTabControlWithStore = observer((props) => {
    const { store } = useContext(MstContext);
    //if (viewKindElement.resultsScope && !store.saveLogicTree[viewKindElement.resultsScope]) {
    //  store.setSaveLogic(viewKindElement.resultsScope);
    //}
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride(props);
    const options = viewKindElement.options || {};
    const coll = store.rep.getColl(collIriOverride);
    const data = coll?.data;
    if (!data) {
        return React.createElement(Spin, null);
    }
    const withConnections = options.connections;
    let tabs = getSnapshot(data);
    //if (tabs && tabs[0] && tabs[0].rank) {
    //  tabs = tabs.slice().sort((t1, t2) => t1.rank - t2.rank);
    //}
    const viewKindElementJs = /*getSnapshot(*/ viewKindElement; /*)*/
    let additionalTabs = viewKindElementJs.tabs;
    if (additionalTabs) {
        additionalTabs = additionalTabs.slice().sort((t1, t2) => t1.rank - t2.rank);
        tabs = [
            ...additionalTabs.filter((t) => t.rank <= 100),
            ...tabs,
            ...additionalTabs.filter((t) => t.rank > 100),
        ];
    }
    const handleChange = (data2) => {
        store.setSelectedData(collIriOverride, data2);
        if (withConnections)
            store.rep.editConn(withConnections, data2);
    };
    const onSelect = (key) => {
        const v = tabs[Number(key)];
        if (v['@type'] === 'aldkg:Tab') {
            handleChange(v.value);
        }
        else {
            handleChange(v);
        }
    };
    return (React.createElement("div", { style: { width: 'auto', display: 'flex', alignItems: 'left' } },
        options.title ? (React.createElement(Col, null,
            React.createElement("h2", { style: { margin: '0 24px 0 0' } }, options.title))) : (React.createElement(React.Fragment, null)),
        React.createElement(Col, null,
            React.createElement(Tabs, { size: 'small', onChange: onSelect }, tabs.map((tab, index) => (React.createElement(Tabs.TabPane, { tab: tab.title, key: String(index) })))))));
});
const antdTabControlTester = rankWith(2, uiTypeIs('aldkg:TabControl'));

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TreeContextMenu = ({ node, selection, visible, x, y, onCreateDirectory, onDeleteDirectory, }) => {
    if (visible) {
        return (React.createElement("ul", { className: 'popup', style: { left: `${x}px`, top: `${y}px`, position: 'fixed', zIndex: 1000 } },
            React.createElement("li", { onClick: () => onCreateDirectory(node.otherProps['@id']) },
                React.createElement(PlusOutlined, null),
                React.createElement("span", null, " \u0421\u043E\u0437\u0434\u0430\u0442\u044C")),
            React.createElement(Divider, { style: { margin: '2px' } }),
            React.createElement("li", { onClick: () => onDeleteDirectory(node.otherProps['@id']) },
                React.createElement(DeleteOutlined, null),
                React.createElement("span", null, " \u0423\u0434\u0430\u043B\u0438\u0442\u044C")),
            React.createElement(Divider, { style: { margin: '2px' } }),
            React.createElement("li", { onClick: () => node.onRename() },
                React.createElement(EditOutlined, null),
                React.createElement("span", null, " \u041F\u0435\u0440\u0435\u0438\u043C\u0435\u043D\u043E\u0432\u0430\u0442\u044C"))));
    }
    else {
        return null;
    }
};

const antdDataControlRenderers = [
    { tester: antdDataControlTester, renderer: AntdDataControlWithStore },
    { tester: saveControlTester, renderer: AntdSaveControlWithStore },
    { tester: antdSelectControlTester, renderer: AntdSelectControlWithStore },
    { tester: antdTabControlTester, renderer: AntdTabControlWithStore, mstVkeType: MstVkeTabControl },
];

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const viewDescrCollConstr = {
    '@id': 'aldkg:Views_Coll',
    entConstrs: [
        {
            '@id': 'aldkg:Views_EntConstr0',
            schema: 'aldkg:ViewShape',
        },
    ],
};
const viewKindCollConstr = {
    '@id': 'aldkg:ViewKinds_Coll',
    entConstrs: [
        {
            '@id': 'aldkg:ViewKinds_EntConstr0',
            schema: 'aldkg:ViewKindShape',
        },
    ],
};

/********************************************************************************
 * Copyright (c) 2024 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const FormMstRepository = types
    .model('FormMstRepository', {
    rep: MstRepository,
    locale: types.map(types.frozen()),
    editingData: types.map(types.boolean),
    selectedData: types.map(types.frozen()),
})
    /**
     * Views
     */
    .views((self) => {
    return {
        getSelectedDataJs(iri) {
            if (!iri)
                return undefined;
            const coll = self.rep.getColl(iri);
            if (!coll)
                return undefined;
            const id = self.selectedData.get(iri);
            if (id) {
                const data = coll.dataByIri(id);
                if (data)
                    return getSnapshot(data);
            }
            return undefined;
        },
        getLocaleId() {
            const localeId = self.locale.get('locale');
            return localeId || 'en_US';
        },
        getLocaleJs(iri) {
            if (!iri)
                return undefined;
            return self.locale.get(iri);
        },
    };
})
    /**
     * Actions
     */
    .actions((self) => {
    return {
        //////////  Selection Service  ///////////
        setSelectedData(iri, data) {
            //console.log('setSelectedData START', { iri, data });
            if (iri) {
                //let id: string;
                //if (typeof data === 'string') {
                //  id = data;
                //} else {
                //  id = data['@id'];
                //
                //if (id && typeof id === 'string') {
                //  self.selectedData.set(iri, id);
                //  console.log('setSelectedData UPDATE', { iri, id });
                //}
                self.selectedData.set(iri, data);
            }
            //console.log('setSelectedData END', { iri, data });
        },
        //////////  FORM  ///////////
        onSaveFormData(formId) { },
        onCancelForm(id) { },
        setEditing(schemaUri, state, reset = false) {
            if (self.editingData.get(schemaUri) !== state) {
                self.editingData.set(schemaUri, state);
                //if (schemaUri === 'root' && this.setParentEditing) {
                //  self.setParentEditing(state);
                //}
                //if (this.saveLogicTree[schemaUri] && this.saveLogicTree[schemaUri].parent) {
                //  this.setEditingChanges(this.saveLogicTree[schemaUri].parent as string, state, schemaUri);
                //}
                //if (reset) {
                //  this.resetEditingChanges(schemaUri);
                //}
            }
        },
        setModalVisible(uri, state) { },
        setOnValidate(form, id, state) { },
        onChangeData(path, data) { },
        setSaveLogic(parent, child) { },
        setEditingChanges(parentUri, state, childUri) { },
        onChangeFormData(form, path, data) { },
        resetEditingChanges(schemaUri) { },
    };
});
const createFormModelFromState2 = (repId, client, initialState, additionalColls = undefined) => {
    const model = FormMstRepository.create(initialState, { client });
    model.rep.setId(repId);
    //model.ns.reloadNs();
    if (additionalColls && additionalColls.length > 0) {
        // wait until namespaces loads then add additionalState
        when(() => Object.keys(model.rep.ns.currentJs).length > 5, () => {
            additionalColls.forEach((collState) => {
                let existedColl;
                // Add new Coll with CollConstr and dataIntrnl to the model
                if (typeof collState.constr === 'object') {
                    existedColl = model.rep.getColl(collState.constr['@id']);
                    if (existedColl === undefined) {
                        try {
                            const coll = model.rep.addColl(collState.constr, collState.opt, collState.data);
                            if (!coll) {
                                console.warn(`Warn: Coll insertion failed! Coll ${collState.constr['@id']} is undefined`);
                            }
                        }
                        catch (err) {
                            console.error(`Err: Coll insertion failed! Coll ${collState.constr['@id']} is undefined`);
                            console.error(err);
                        }
                    }
                }
                else if (typeof collState.constr === 'string') {
                    existedColl = model.rep.getColl(collState.constr);
                    if (existedColl === undefined) {
                        try {
                            const coll = model.rep.addCollByConstrRef(collState.constr, collState.opt, collState.data);
                            if (!coll) {
                                console.warn(`Warn: Coll insertion failed! Coll ${collState.constr} is undefined`);
                            }
                        }
                        catch (err) {
                            console.error(`Err: Coll insertion failed! Coll ${collState.constr} is undefined`);
                            console.error(err);
                        }
                    }
                }
                // If CollConstr with '@id' existed, adds data to the Coll's dataIntrnl
                if (existedColl !== undefined && collState.data && Array.isArray(collState.data)) {
                    existedColl.addElems(collState.data);
                }
            });
        });
    }
    return model;
};
const createUiModelFromState = (repId, client, initialState, additionalColls) => {
    registerMstCollSchema(MstViewKind);
    registerMstCollSchema(MstViewDescr);
    return createFormModelFromState2(repId, client, initialState, additionalColls);
};
const createAdditionalColls = (viewKinds, viewDescrs, data = undefined) => {
    const additionalColls = [
        // ViewKinds Collection
        {
            constr: viewKindCollConstr,
            data: Array.isArray(viewKinds) ? viewKinds : [viewKinds],
            opt: {
                updPeriod: undefined,
                lastSynced: dayjs().valueOf(),
                //resolveCollConstrs: false, // disable data loading from the server for viewKinds.collConstrs
            },
        },
        // ViewDescrs Collection
        {
            constr: viewDescrCollConstr,
            data: Array.isArray(viewDescrs) ? viewDescrs : [viewDescrs],
            opt: {
                updPeriod: undefined,
                lastSynced: dayjs().valueOf(),
                //resolveCollConstrs: false, // 'true' here (by default) triggers data loading from the server
                // for viewDescrs.collConstrs (it loads lazily -- after the first access)
            },
        },
    ];
    if (data) {
        additionalColls.push({
            constr: Array.isArray(viewKinds) ? viewKinds[0].collsConstrs[0] : viewKinds.collsConstrs[0],
            data: Array.isArray(data) ? data : [data],
            opt: {
                updPeriod: undefined,
                lastSynced: dayjs().valueOf(),
                //resolveCollConstrs: false, // 'true' here (by default) triggers data loading from the server
                // for viewDescrs.collConstrs (it loads lazily -- after the first access)
            },
        });
    }
    return additionalColls;
};

const ViewShapeSchema = {
    $schema: 'http://json-schema.org/draft-07/schema#',
    '@id': 'aldkg:ViewShape',
    '@type': 'sh:NodeShape',
    title: 'View Shape',
    description: 'Artifact Shape',
    targetClass: 'aldkg:ViewDescr',
    type: 'object',
    '@context': {
        '@type': 'rdf:type',
    },
    properties: {
        '@id': {
            title: 'URI',
            type: 'string',
            format: 'iri',
        },
        '@type': {
            title: 'Class',
            type: 'string',
            format: 'iri',
        },
    },
    required: ['@id', '@type'],
};
const ViewKindShapeSchema = {
    $schema: 'http://json-schema.org/draft-07/schema#',
    '@id': 'aldkg:ViewShape',
    '@type': 'sh:NodeShape',
    title: 'View Shape',
    description: 'Artifact Shape',
    targetClass: 'aldkg:ViewDescr',
    type: 'object',
    '@context': {
        '@type': 'rdf:type',
    },
    properties: {
        '@id': {
            title: 'URI',
            type: 'string',
            format: 'iri',
        },
        '@type': {
            title: 'Class',
            type: 'string',
            format: 'iri',
        },
    },
    required: ['@id', '@type'],
};

/********************************************************************************
 * Copyright (c) 2020 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const AntdModal = observer(({ viewKind, viewKindElement, viewDescr, viewDescrElement, id, schema, enabled, cells, childrenId, Render }) => {
    const [visible, setVisible] = useState(false);
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(FormLayoutIRI);
    const onCancel = () => {
        const editing = store.editingData.get(id);
        if (!editing) {
            store.setModalVisible(id, false);
        }
        else {
            setVisible(true);
        }
    };
    const onOk = () => {
        store.setEditing(id, false, true);
        store.setModalVisible(id, false);
        // save Data
    };
    return (React.createElement(Modal, { title: 'Test', visible: store.modals[id], onOk: onOk, onCancel: onCancel, cancelText: locale.btnCancel, width: 1200, okText: locale.btnSave },
        React.createElement(Render, { viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, enabled: enabled, cells: cells, id: childrenId }),
        React.createElement(SaveReqDialog, { visible: visible, onOk: () => {
                store.setEditing(id, false, true);
                setVisible(false);
                store.setModalVisible(id, false);
            }, onCancel: () => setVisible(false) })));
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const InitTinyMCE = () => {
    const i18n = { language: 'ru_RU' };
    return (React.createElement("div", { style: { display: 'none' } },
        React.createElement(Editor, { initialValue: '', inline: true, init: {
                branding: false,
                menubar: false,
                toolbar: false,
                plugins: ['autolink codesample link lists media paste table image quickbars textpattern'],
                quickbars_insert_toolbar: 'paste quicktable image',
                quickbars_selection_toolbar: 'bold italic | h2 h3 | blockquote quicklink',
                contextmenu: 'inserttable | cell row column deletetable | link image imagetools | codesample',
                language_url: `${process.env.PUBLIC_URL}/lang/ru.js`,
                language: i18n.language,
                skin: 'oxide',
                skin_url: `${process.env.PUBLIC_URL}/skins/ui/oxide`,
                content_css: `${process.env.PUBLIC_URL}/skins/ui/oxide/content.min.css`,
                theme: 'silver',
                theme_url: `${process.env.PUBLIC_URL}/themes/silver/theme.min.js`,
            } })));
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ContextMenu = ({ selection, visible, x, y, actionsMap }) => {
    return ((actionsMap && visible && (React.createElement("ul", { className: 'popup', style: { left: `${x}px`, top: `${y}px`, position: 'fixed' } }, actionsMap.map(({ title, action }) => (React.createElement("li", { key: title, onClick: () => {
            action(selection.getSelectedItems());
            selection.deselectAll();
        } }, title)))))) ||
        null);
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const Overlay = ({ isLoading }) => {
    return isLoading ? (React.createElement("div", { className: 'overlay' },
        React.createElement(Spin, { className: 'spin' }))) : null;
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const SelectionCell = observer(({ id, selection, multiselect, onSelect }) => {
    const checked = selection.selection.get(id)?.selected;
    const handleChange = () => {
        if (multiselect) {
            selection.toggleItem(id);
        }
        else {
            selection.deselectAll();
            selection.selectItem(id);
        }
        onSelect(selection.getSelectedItems());
    };
    return multiselect ? (React.createElement(Checkbox, { checked: checked, onChange: handleChange })) : (React.createElement(Radio, { checked: checked, onChange: handleChange }));
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const SelectionHeaderCell = observer(({ selection, multiselect, onSelect }) => {
    const checked = selection.isAllSelected;
    const handleChange = () => {
        checked ? selection.deselectAll() : selection.selectAll();
        onSelect(selection.getSelectedItems());
    };
    return multiselect ? React.createElement(Checkbox, { onChange: handleChange, checked: checked }) : null;
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const getSelectionColumn = ({ selection, multiselect, onSelect }) => ({
    key: '__selection__',
    dataKey: '__selection__',
    rowKey: '@id',
    width: 30,
    flexShrink: 0,
    resizable: false,
    frozen: Column.FrozenDirection.LEFT,
    cellRenderer: ({ rowData }) => {
        const id = rowData['@id'];
        return React.createElement(SelectionCell, { selection: selection, id: id, multiselect: multiselect, onSelect: onSelect });
    },
    headerRenderer: () => React.createElement(SelectionHeaderCell, { selection: selection, multiselect: multiselect, onSelect: onSelect }),
    dataSize: 10,
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const renderTester = (prevProps, nextProps) => {
    return (isEqual$1(prevProps.viewKind, nextProps.viewKind) &&
        isEqual$1(prevProps.viewKindElement, nextProps.viewKindElement) &&
        isEqual$1(prevProps.viewDescr, nextProps.viewDescr) &&
        isEqual$1(prevProps.viewDescrElement, nextProps.viewDescrElement) &&
        isEqual$1(prevProps.rowData, nextProps.rowData) &&
        isEqual$1(prevProps.column, nextProps.column) &&
        isEqual$1(prevProps.cellData, nextProps.cellData) &&
        isEqual$1(prevProps.enabled, nextProps.enabled) &&
        isEqual$1(prevProps.rowData, nextProps.rowData) &&
        isEqual$1(prevProps.uri, nextProps.uri) &&
        isEqual$1(prevProps.heightCache, nextProps.heightCache) &&
        isEqual$1(prevProps.rowIndex, nextProps.rowIndex));
};
const EditableCell = React.memo(({ viewKind, viewKindElement, viewDescr, viewDescrElement, cellData, schema, enabled, rowData, column, uri, heightCache, rowIndex, isScroling, ...restProps }) => {
    const onChange = (measure) => {
        heightCache[rowData['@id']] = measure;
        restProps.container._updateRowHeights();
    };
    return cellData ? (React.createElement(DispatchCell, { id: '', viewKind: viewKind, viewKindElement: viewKindElement, viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: schema, CKey: column.key, onMeasureChange: onChange, width: column.width, height: heightCache[rowData['@id']], rowData: rowData, data: cellData, uri: uri, enabled: enabled })) : null;
}, (prevProps, nextProps) => renderTester(prevProps, nextProps));

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const CELL_TYPE = 'aldkg:cell';
const ROW_KEY = '@id';

const createViewKindElement = ({ viewKind, key }) => {
    const options = viewKind?.options?.[key];
    const viewKindElement = {
        '@id': `${viewKind['@id']}-${key}-cell`,
        '@type': CELL_TYPE,
        options,
    };
    return viewKindElement;
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const getColumnFilterProps = (dataIndex) => ({
    filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters }) => (React.createElement("div", { style: { padding: 8 } },
        React.createElement(Input, { placeholder: `Search ${dataIndex}`, value: selectedKeys[0], onChange: (e) => setSelectedKeys(e.target.value ? [e.target.value] : []), onPressEnter: () => confirm(), style: { width: 188, marginBottom: 8, display: 'block' } }),
        React.createElement(Button, { type: 'primary', onClick: () => confirm(), icon: React.createElement(SearchOutlined, null), size: 'small', style: { width: 90, marginRight: 8 } }, "Search"),
        React.createElement(Button, { onClick: () => clearFilters(), size: 'small', style: { width: 90 } }, "Reset"))),
    filterIcon: (filtered) => React.createElement(SearchOutlined, { style: { color: filtered ? '#1890ff' : undefined } }),
    onFilter: (value, record) => record[dataIndex].toString().toLowerCase().includes(value.toLowerCase()),
    onFilterDropdownVisibleChange: (visible) => { },
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const createColumn = ({ property, propertySchema, columnsHeight, viewKind, viewKindElement, viewDescr, viewDescrElement, }) => {
    const order = viewKindElement?.options?.order || [];
    const options = viewKindElement?.options?.[property] || {};
    const disabled = (isEmpty(options) && !order.includes(property)) || !!options?.disabled;
    const newColumn = {
        editable: false,
        hidden: false,
        disabled,
        width: 200,
        resizable: true,
        title: propertySchema.title || property,
        key: property,
        dataKey: property,
        renderer: propertySchema?.formatters?.default,
        ...options,
        ...(options.searchFilter && getColumnFilterProps(property)),
        cellRenderer: ({ cellData, ...rest }) => (React.createElement(EditableCell, { viewKind: viewKind, viewKindElement: createViewKindElement({ viewKind: viewKindElement, key: property }), viewDescr: viewDescr, viewDescrElement: viewDescrElement, schema: propertySchema, editable: options.editable, heightCache: columnsHeight, dataIndex: property, title: propertySchema.title || property, cellData: cellData, ...rest })),
    };
    return newColumn;
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const createSort = (order) => (item1, item2) => {
    const idx1 = order.indexOf(item1.key);
    const idx2 = order.indexOf(item2.key);
    if (idx1 < idx2)
        return 1;
    if (idx1 > idx2)
        return -1;
    return 0;
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const DraggableHandle = SortableHandle(({ children }) => children);
const getSystemColumn = ({ menuItems }) => ({
    key: 'settings',
    dataKey: 'settings',
    width: 20,
    frozen: Column.FrozenDirection.LEFT,
    cellRenderer: ({ cellData }) => (React.createElement(DraggableHandle, null,
        React.createElement(MoreOutlined, { className: 'handler' }))),
    headerRenderer: () => (React.createElement(Dropdown, { overlay: React.createElement(Menu, null, menuItems) },
        React.createElement(SettingOutlined, null))),
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ExtendedTableMenu = ({ dataSource, sortKey, leftColumns, rightColumns, onChange, ...restProps }) => {
    const [transferVisible, setTransferVisible] = useState(false);
    const [targetKeys, setTargetKeys] = useState([]);
    const { store } = useContext(MstContext);
    const locale = store.getLocaleJs(ArrayIRI);
    const onSave = () => {
        setTransferVisible(false);
        const idxs = dataSource.reduce((res, e, idx) => {
            if (targetKeys.includes(e.key)) {
                res.push({ idx, hidden: true });
            }
            else {
                res.push({ idx, hidden: false });
            }
            return res;
        }, []);
        onChange(idxs);
    };
    const onCancel = () => {
        setTransferVisible(false);
        const newData = dataSource.reduce((res, e) => {
            if (e[sortKey]) {
                res.push(e.key);
            }
            return res;
        }, []);
        setTargetKeys(newData);
    };
    const handleChange = (nextTargetKeys) => {
        setTargetKeys(nextTargetKeys);
    };
    useEffect(() => {
        const newData = dataSource.reduce((res, e) => {
            if (e[sortKey]) {
                res.push(e.key);
            }
            return res;
        }, []);
        setTargetKeys(newData);
    }, [dataSource, sortKey]);
    return (React.createElement(React.Fragment, null,
        React.createElement("span", { onClick: () => setTransferVisible(true) }, locale.extSettings),
        React.createElement(Modal, { title: locale.columnSettings, visible: transferVisible, onOk: onSave, onCancel: onCancel, cancelText: locale.btnCancel, okText: locale.btnSave },
            React.createElement(Transfer, { dataSource: dataSource, targetKeys: targetKeys, onChange: handleChange, showSelectAll: false }, ({ direction, filteredItems, onItemSelectAll, onItemSelect, selectedKeys: listSelectedKeys, disabled: listDisabled, }) => {
                const columns = direction === 'left' ? leftColumns : rightColumns;
                const rowSelection = {
                    getCheckboxProps: (item) => ({ disabled: listDisabled || item.disabled }),
                    onSelectAll(selected, selectedRows) {
                        const treeSelectedKeys = selectedRows.filter((item) => !item.disabled).map(({ key }) => key);
                        const diffKeys = selected
                            ? difference(treeSelectedKeys, listSelectedKeys)
                            : difference(listSelectedKeys, treeSelectedKeys);
                        onItemSelectAll(diffKeys, selected);
                    },
                    onSelect({ key }, selected) {
                        onItemSelect(key, selected);
                    },
                    selectedRowKeys: listSelectedKeys,
                };
                return (React.createElement(Table, { rowSelection: rowSelection, columns: columns, dataSource: filteredItems, size: 'small', onRow: ({ key, disabled: itemDisabled }) => ({
                        onClick: () => {
                            if (itemDisabled || listDisabled)
                                return;
                            onItemSelect(key || '', !listSelectedKeys.includes(key || ''));
                        },
                    }) }));
            }))));
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const MenuItem = ({ title, onClick, colState }) => {
    const [pickOn, setPickOn] = useState(!colState);
    useEffect(() => {
        setPickOn(!colState);
    }, [colState]);
    return (React.createElement("div", { onClick: (e) => {
            setPickOn(!pickOn);
            onClick(pickOn);
        } },
        pickOn ? React.createElement(CheckOutlined, null) : React.createElement("div", { style: { display: 'inline-block', width: '14px' } }),
        React.createElement("span", null, title)));
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const TableColumns = [
    {
        dataIndex: 'title',
        title: 'Имя',
    },
];
const getConfigMenuItems = ({ columns, setColumnsVisible }) => {
    const menuItems = columns.map((col, idx) => {
        return (React.createElement(Menu.Item, { key: idx },
            React.createElement(MenuItem, { title: col.title, colState: col.hidden, onClick: (state) => setColumnsVisible([{ idx, hidden: state }]) })));
    });
    menuItems.push(React.createElement(Menu.Item, { key: 'additional' },
        React.createElement(ExtendedTableMenu, { dataSource: columns, sortKey: 'hidden', rightColumns: TableColumns, leftColumns: TableColumns, onChange: setColumnsVisible })));
    return menuItems;
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const useColumns = ({ schemaProperties, viewKind, viewKindElement, viewDescr, viewDescrElement, selection, multiselect, order, onSelect, }) => {
    const columnsHeightRef = useRef({});
    const [visibleColumns, setVisibleColumns] = useState([]);
    const parsedColumns = useMemo(() => Object.entries(schemaProperties).map(([property, propertySchema]) => {
        const test = createColumn({
            property,
            propertySchema,
            columnsHeight: columnsHeightRef.current,
            viewKind,
            viewKindElement,
            viewDescr,
            viewDescrElement,
        });
        return test;
    }), [schemaProperties, viewKind, viewKindElement, viewDescr, viewDescrElement]);
    const setColumnsVisible = useCallback((idxs) => setVisibleColumns((prevVisibleColumns) => {
        const newVisibleColmns = [...prevVisibleColumns];
        idxs.forEach((e) => {
            const newData = Object.assign({}, newVisibleColmns[e.idx], { hidden: e.hidden });
            newVisibleColmns[e.idx] = newData;
        });
        return newVisibleColmns;
    }), [setVisibleColumns]);
    const menuItems = useMemo(() => getConfigMenuItems({ columns: visibleColumns, setColumnsVisible }), [visibleColumns, setColumnsVisible]);
    const systemColumn = useMemo(() => getSystemColumn({ menuItems }), [menuItems]);
    const selectionColumn = useMemo(() => getSelectionColumn({ selection, multiselect, onSelect }), [selection, multiselect, onSelect]);
    const columns = useMemo(() => [systemColumn, selectionColumn, ...visibleColumns], [systemColumn, selectionColumn, visibleColumns]);
    useEffect(() => {
        setVisibleColumns(() => {
            const visibleColumns = parsedColumns.filter((column) => !column.disabled);
            const customOrder = [...(order || [])];
            return visibleColumns.sort(createSort(customOrder.reverse()));
        });
    }, [parsedColumns, setVisibleColumns, order]);
    return { visibleColumns: columns, setColumnsVisible };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const useDraggableContainer = (setData) => {
    const tableRef = React.useRef();
    const getContainer = useCallback(() => tableRef.current?.getDOMNode().querySelector('.BaseTable__table-frozen-left .BaseTable__body'), []);
    const getHelperContainer = useCallback(() => tableRef.current?.getDOMNode().querySelector('.BaseTable__table-frozen-left'), []);
    const handleSortEnd = useCallback(({ oldIndex, newIndex }) => {
        setData((oldData) => {
            const newData = [...oldData];
            const [removed] = newData.splice(oldIndex, 1);
            newData.splice(newIndex, 0, removed);
            return newData;
        });
    }, [setData]);
    return { tableRef, getContainer, getHelperContainer, handleSortEnd };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const useRows = ({ selection, onSelect, closePopup, openPopup, setPopupCoords, isPopupVisible, multiselect, }) => {
    const rowEventHandlers = useMemo(() => ({
        onContextMenu: ({ event }) => {
            event.preventDefault();
            if (!isPopupVisible) {
                document.addEventListener(`click`, function onClickOutside() {
                    closePopup();
                    document.removeEventListener(`click`, onClickOutside);
                });
            }
            openPopup();
            setPopupCoords({ x: event.clientX, y: event.clientY });
        },
        onClick: ({ rowData, event }) => {
            if (event.target.nodeName !== 'INPUT') {
                if (multiselect) {
                    selection.toggleItem(rowData['@id']);
                }
                else {
                    selection.deselectAll();
                    selection.selectItem(rowData['@id']);
                }
                onSelect(selection.getSelectedItems());
            }
        },
    }), [closePopup, openPopup, isPopupVisible, setPopupCoords, selection, onSelect, multiselect]);
    const rowClassName = useCallback(({ rowData }) => {
        return selection.isItemSelected(rowData['@id']) ? 'row-selected' : '';
    }, [selection]);
    return { rowEventHandlers, rowClassName };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const useTableData = ({ viewKindElement: baseViewKindElement, viewDescr, actions }) => {
    const { store } = useContext(MstContext);
    const [id, collIri, collIriOverride, inCollPath, viewKindElement, viewDescrElement] = processViewKindOverride({ viewKindElement: baseViewKindElement, viewDescr });
    const viewOptions = useMemo(() => viewKindElement.options || {}, [viewKindElement]);
    const coll = useMemo(() => store.rep.getColl(collIriOverride), [collIriOverride, store]);
    const actionsMap = useMemo(() => mapViewKindPropsToActions({ actions, viewKindActionProps: viewOptions.selectActions, coll, root: store }), [store, coll, actions, viewOptions]);
    const handleEndReached = useCallback(() => {
        if (!coll.isLoading) {
            coll.loadMore();
        }
    }, [coll]);
    const onSelect = useCallback((data) => {
        const withConnections = viewOptions.connections;
        if (data && data.length) {
            store.setSelectedData(collIriOverride, data[0]);
            withConnections && store.editConn(withConnections, data[0]);
        }
    }, [store, viewOptions, collIriOverride]);
    //заглушка
    const setData = useCallback((cb) => {
        const newData = cb(getSnapshot(coll?.data));
        applySnapshot(coll?.data, newData);
    }, [coll]);
    const getCollConstrJs = useCallback(() => {
        return getSnapshot(coll.collConstr);
    }, [coll.collConstr]);
    const setCollConstrJs = useCallback((collConstr) => {
        applySnapshot(coll.collConstr, collConstr);
    }, [coll.collConstr]);
    return {
        sourceData: getSnapshot(coll?.data) || [],
        isLoading: coll.isLoading,
        viewOptions,
        actionsMap,
        handleEndReached,
        onSelect,
        setData,
        getCollConstrJs,
        setCollConstrJs,
    };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const usePopupState = (initialState) => {
    const [isPopupVisible, setPopupVisibility] = useState(initialState);
    const [popupCoords, setPopupCoords] = useState({ x: 0, y: 0 });
    const toggle = useCallback(() => setPopupVisibility((value) => !value), [setPopupVisibility]);
    const close = useCallback(() => setPopupVisibility(false), [setPopupVisibility]);
    const open = useCallback(() => setPopupVisibility(true), [setPopupVisibility]);
    return { isPopupVisible, toggle, close, open, popupCoords, setPopupCoords };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const MstJsObjectWithSelection = types.model({
    item: types.frozen(),
    selected: types.boolean,
});
const getId = (item) => item['@id'];
const SelectionStore = types
    .model('SelectionStore', {
    selection: types.map(MstJsObjectWithSelection),
    selectionLength: 0,
})
    .views((self) => {
    return {
        get isAllSelected() {
            return self.selection.size && self.selectionLength === self.selection.size;
        },
    };
})
    .actions((self) => {
    return {
        isItemSelected(id) {
            return self.selection.get(id)?.selected;
        },
        deselectItem(id) {
            if (self.selection.has(id)) {
                self.selection.get(id).selected = false;
                self.selectionLength--;
            }
        },
        selectItem(id) {
            if (self.selection.has(id)) {
                self.selection.get(id).selected = true;
                self.selectionLength++;
            }
        },
        toggleItem(id) {
            return self.isItemSelected(id) ? self.deselectItem(id) : self.selectItem(id);
        },
        selectAll() {
            self.selection.forEach((item) => {
                item.selected = true;
                self.selectionLength = self.selection.size;
            });
        },
        deselectAll() {
            self.selection.forEach((item) => {
                item.selected = false;
                self.selectionLength = 0;
            });
        },
        setItems(data) {
            data.forEach((element) => {
                const id = getId(element);
                if (!self.selection.has(id))
                    self.selection.set(id, { item: element, selected: false });
            });
        },
        getSelectedItems() {
            return [...self.selection.values()].reduce((acc, selectObject) => {
                if (selectObject.selected)
                    acc.push(selectObject.item);
                return acc;
            }, []);
        },
    };
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const useSelection = ({ sourceData }) => {
    const selection = useMemo(() => SelectionStore.create({ selection: {} }), []);
    useEffect(() => selection.setItems(sourceData), [sourceData, selection]);
    return { selection };
};

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const DraggableElement = SortableElement(({ children }) => children);
const rowRenderer = ({ rowData, cells, ...rest }) => {
    if (rowData === 'test')
        return (React.createElement("div", { style: { margin: '0 auto', height: '50px' } },
            React.createElement(Spin, { style: { marginTop: '15px' } })));
    return cells;
};
const Row = React.memo(React.forwardRef(({ key, index, children, ...rest }, ref) => {
    if (children.length === 2) {
        return (React.createElement(DraggableElement, { key: key, index: index },
            React.createElement("div", { ...rest, ref: ref }, children)));
    }
    return (React.createElement("div", { ...rest, ref: ref }, children));
}), (p, n) => isEqual$1(p.children, n.children));
const rowProps = ({ rowData, ...rest }) => ({
    tagName: Row,
    index: rest.rowIndex,
    key: rowData['@id'],
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const DraggableContainer = SortableContainer(({ children }) => children);
const EditableTable = observer(({ viewKind, viewKindElement, viewDescr, viewDescrElement, schema, actions }) => {
    const { sourceData, viewOptions, actionsMap, isLoading, handleEndReached, onSelect, setData, getCollConstrJs, setCollConstrJs, } = useTableData({
        viewKindElement,
        viewDescr,
        actions,
    });
    const { isPopupVisible, close, open, popupCoords, setPopupCoords } = usePopupState(false);
    const { tableRef, getContainer, getHelperContainer, handleSortEnd } = useDraggableContainer(setData);
    const { selection } = useSelection({ sourceData });
    const { rowEventHandlers, rowClassName } = useRows({
        selection,
        onSelect,
        closePopup: close,
        openPopup: open,
        isPopupVisible,
        setPopupCoords,
        multiselect: viewOptions.multiselect,
    });
    const { visibleColumns } = useColumns({
        schemaProperties: schema.properties,
        viewKind,
        viewKindElement,
        viewDescr,
        viewDescrElement,
        selection,
        onSelect,
        order: viewOptions.order,
    });
    const collConstr = getCollConstrJs();
    const initSortState = collConstr?.orderBy
        ? collConstr.orderBy.reduce((obj, item) => {
            return {
                ...obj,
                [item.variable]: item.descending ? 'desc' : 'asc',
            };
        }, {})
        : {};
    const [sortState, setSortState] = useState(initSortState);
    const onSort = (property, sortDir) => {
        const collConstr = getCollConstrJs();
        const orderBy = collConstr.orderBy;
        let newOrder = undefined;
        //console.log({ property, sortDir, sortState, collConstr });
        if (sortState[property] === 'desc') {
            //console.log('reset sorting order');
            // reset sorting order
            sortDir = null;
            if (Array.isArray(orderBy)) {
                const idx = orderBy.findIndex((e) => e.variable === property);
                //console.log({ idx, orderBy });
                if (idx !== -1) {
                    newOrder = [...orderBy];
                    newOrder.splice(idx, 1);
                    if (newOrder.length === 0)
                        newOrder = undefined;
                }
            }
        }
        else {
            const isDesc = sortDir === 'desc' ? true : false;
            const sortQuery = { variable: property, descending: isDesc };
            //console.log('NO reset sorting order', sortQuery);
            if (!orderBy) {
                newOrder = [sortQuery];
            }
            else if (Array.isArray(orderBy)) {
                newOrder = [...orderBy];
                const idx = orderBy.findIndex((e) => e.variable === property);
                if (idx !== -1) {
                    newOrder[idx] = sortQuery;
                }
                else {
                    newOrder.push(sortQuery);
                }
            }
        }
        const newCollConstr = { ...collConstr, orderBy: newOrder };
        setCollConstrJs(newCollConstr);
        const newSortState = { ...sortState, [property]: sortDir };
        //console.log({ newSortState, newCollConstr });
        setSortState(newSortState);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(InitTinyMCE, null),
        React.createElement(AutoResizer, null, ({ width, height }) => (React.createElement(DraggableContainer, { useDragHandle: true, getContainer: getContainer, helperContainer: getHelperContainer, onSortEnd: handleSortEnd },
            React.createElement(BaseTable, { fixed: true, useIsScrolling: true, rowRenderer: rowRenderer, rowClassName: rowClassName, width: width, height: height, ref: tableRef, expandColumnKey: viewOptions.expandColumnKey || ROW_KEY, estimatedRowHeight: 50, rowKey: ROW_KEY, onEndReachedThreshold: 300, overlayRenderer: React.createElement(Overlay, { isLoading: isLoading }), columns: visibleColumns, data: sourceData, onEndReached: handleEndReached, rowEventHandlers: rowEventHandlers, rowProps: rowProps, sortState: sortState, onColumnSort: (args) => {
                    if (typeof args.key === 'string')
                        onSort(args.key, args.order);
                } })))),
        React.createElement(ContextMenu, { x: popupCoords.x, y: popupCoords.y, selection: selection, visible: isPopupVisible, actionsMap: actionsMap })));
});

/********************************************************************************
 * Copyright (c) 2021 Agentlab and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the GNU General Public License v. 3.0 which is available at
 * https://www.gnu.org/licenses/gpl-3.0.html.
 *
 * SPDX-License-Identifier: GPL-3.0-only
 ********************************************************************************/
const ArrayIRI = 'aldkg:Array';
const BaseTableArrayControlRenderer = (props) => {
    return (props.schema && React.createElement(EditableTable, { ...props })) || React.createElement("div", null);
};
const tableArrayControlTester = rankWith(3, uiTypeIs('aldkg:Array'));
const tableRenderers = [
    {
        tester: tableArrayControlTester,
        renderer: BaseTableArrayControlRenderer,
    },
];

export { AntQueryWithStore, AntdBooleanCell, AntdBooleanCellWithStore, AntdBooleanControl, AntdBooleanControlWithStore, AntdButtonCell, AntdButtonCellWithStore, AntdButtonControl, AntdButtonControlWithStore, AntdCellCardLayout, AntdCellG2, AntdCellG2WithStore, AntdCellHorizontalLayoutRenderer, AntdCheckbox, AntdDataControlWithStore, AntdDataLayout, AntdDateControl, AntdDateControlWithStore, AntdEnumCell, AntdEnumCellWithStore, AntdEnumControl, AntdEnumControlWithStore, AntdFormLayout, AntdFormLayoutWithStore, AntdHorizontalLayoutRenderer, AntdHorizontalLayoutWithStore, AntdImageCell, AntdImageCellWithStore, AntdImageControl, AntdImageControlWithStore, AntdInputControl, AntdInputDate, AntdInputNumber, AntdInputText, AntdInputTime, AntdIntegerCellWithStore, AntdModal, AntdNumberCell, AntdNumberCellWithStore, AntdNumberControl, AntdNumberControlWithStore, AntdRadioGroupControl, AntdRadioGroupControlWithStore, AntdRateCell, AntdRateCellWithStore, AntdSaveControlWithStore, AntdSelect, AntdSelectControl, AntdSelectControlWithStore, AntdSliderControl, AntdSliderControlWithStore, AntdTabControlWithStore, AntdTabsLayoutWithStore, AntdTextCell, AntdTextCellWithStore, AntdTextControl, AntdTextControlWithStore, AntdTimeCell, AntdTimeCellWithStore, AntdVerticalLayoutRenderer, AntdVerticalLayoutWithStore, ArrayIRI, BaseTableArrayControlRenderer, DispatchCell, ErrorFallback, Form, FormLayoutIRI, FormMstRepository, FormsDispatch, LegacyTypeIcon, LogicalButton, MstContext, MstContextProvider, MstViewDescr, MstViewDescrDataType, MstViewDescrElement, MstViewKind, MstViewKindDataType, MstViewKindElement, MstVkeTabControl, MstVkeVerticalLayout, NOT_APPLICABLE, NodeRenderer, QueryForm, QueryFormIRI, QueryIRI, SaveControl, SaveReqDialog, SaveReqDialogIRI, SaveReqDialogWithStore, SplitPaneLayoutRenderer, SplitPaneLayoutWithStore, TableRenderer, TabsLayout, TinyMCE$1 as TinyMCE, TinyMCECell, TinyMCECellWithStore, TinyMCEControl, TinyMCEControlWithStore, TreeContextMenu, TreeRenderer, UnknownRenderer, ViewKindShapeSchema, ViewShapeSchema, actions, and, antdBooleanCellTester, antdBooleanControlTester, antdButtonCellTester, antdButtonControlTester, antdCellCardLayoutTester, antdCellG2Tester, antdCellHorizontalLayoutTester, antdCells, antdControlRenderers, antdDataControlRenderers, antdDataControlTester, antdDateControlTester, antdEnumCellTester, antdEnumControlTester, antdFormLayoutTester, antdHorizontalLayoutTester, antdImageCellTester, antdImageControlTester, antdIntegerCellTester, antdLayoutRenderers, antdNumberCellTester, antdNumberControlTester, antdQueryFormTester, antdQueryTester, antdRadioGroupControlTester, antdRateCellTester, antdSelectControlTester, antdSliderControlTester, antdTabControlTester, antdTabsLayoutTester, antdTextCellTester, antdTextControlTester, antdTimeCellTester, antdVerticalLayoutTester, areEqual, compareByIri, computeLabel, createAdditionalColls, createCleanLabel, createFormModelFromState2, createLabelDescriptionFrom, createUiModelFromState, createViewDescrElementIri, formatIs, formatters, hasType, isBooleanControl, isDateControl, isDateTimeControl, isEnumControl, isIntegerControl, isNumberControl, isPlainLabel, isRangeControl, isStringControl, mapViewKindPropsToActions, mstJsonLdIds, optionIs, or, processViewKindOverride, rankWith, registerMstViewDescrSchema, registerMstViewKindSchema, renderLayoutElements, saveControlTester, schemaMatches, schemaTypeIs, splitPaneLayoutTester, strcmp, tableArrayControlTester, tableRenderers, tinyMCECellTester, tinyMCEControlTester, treeify, uiTypeIs, unregisterMstViewDescrSchema, unregisterMstViewKindSchema, viewDescrCollConstr, viewKindCollConstr, withLayoutProps, withStoreToButtonProps, withStoreToCellProps, withStoreToCollapseProps, withStoreToControlProps, withStoreToDataControlProps, withStoreToFormProps, withStoreToMenuProps, withStoreToModalProps, withStoreToSaveButtonProps, withStoreToSaveDialogProps, withStoreToSelectControlProps, withStoreToViewClassProps, withStoreToViewProps };
//# sourceMappingURL=index.js.map
